@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2017 Red Hat, Inc.
+# Copyright (C) 2020 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -11,13 +11,12 @@ import pytest
 import os
 import time
 import ldap
-import subprocess
 from random import sample
 from lib389.utils import ds_is_older, ensure_list_bytes, ensure_bytes, ensure_str
 from lib389.topologies import topology_m1h1c1 as topo, topology_st, topology_m2 as topo_m2
 from lib389._constants import *
 from lib389.plugins import MemberOfPlugin
-from lib389 import agreement, Entry
+from lib389 import Entry
 from lib389.idm.user import UserAccount, UserAccounts, TEST_USER_PROPERTIES
 from lib389.idm.group import Groups, Group
 from lib389.replica import ReplicationManager
@@ -26,7 +25,8 @@ from lib389.idm.nscontainer import nsContainers
 
 
 # Skip on older versions
-pytestmark = pytest.mark.skipif(ds_is_older('1.3.7'), reason="Not implemented")
+pytestmark = [pytest.mark.tier1,
+              pytest.mark.skipif(ds_is_older('1.3.7'), reason="Not implemented")]
 
 USER_CN = 'user_'
 GROUP_CN = 'group1'
@@ -47,9 +47,9 @@ def add_users(topo_m2, users_num, suffix):
     Return the list of added user DNs.
     """
     users_list = []
-    users = UserAccounts(topo_m2.ms["master1"], suffix, rdn=None)
+    users = UserAccounts(topo_m2.ms["supplier1"], suffix, rdn=None)
     log.info('Adding %d users' % users_num)
-    for num in sample(range(1000), users_num):
+    for num in sample(list(range(1000)), users_num):
         num_ran = int(round(num))
         USER_NAME = 'test%05d' % num_ran
         user = users.create(properties={
@@ -76,8 +76,8 @@ def config_memberof(server):
     for ent in ents:
         log.info('update %s to add nsDS5ReplicatedAttributeListTotal' % ent.dn)
         server.agreement.setProperties(agmnt_dn=ents[0].dn,
-                                       properties={RA_FRAC_EXCLUDE:'(objectclass=*) $ EXCLUDE memberOf',
-                                                   RA_FRAC_EXCLUDE_TOTAL_UPDATE:'(objectclass=*) $ EXCLUDE '})
+                                       properties={RA_FRAC_EXCLUDE: '(objectclass=*) $ EXCLUDE memberOf',
+                                                   RA_FRAC_EXCLUDE_TOTAL_UPDATE: '(objectclass=*) $ EXCLUDE '})
 
 
 def send_updates_now(server):
@@ -88,22 +88,22 @@ def send_updates_now(server):
 
 
 def _find_memberof(server, member_dn, group_dn):
-    #To get the specific server's (M1, C1 and H1) user and group
+    # To get the specific server's (M1, C1 and H1) user and group
     user = UserAccount(server, member_dn)
     assert user.exists()
     group = Group(server, group_dn)
     assert group.exists()
 
-    #test that the user entry should have memberof attribute with sepecified group dn value
-    assert group._dn in user.get_attr_vals_utf8('memberOf')
+    # test that the user entry should have memberof attribute with specified group dn value
+    assert group._dn.lower() in user.get_attr_vals_utf8_l('memberOf')
 
 
 @pytest.mark.bz1352121
 def test_memberof_with_repl(topo):
     """Test that we allowed to enable MemberOf plugin in dedicated consumer
 
-    :id: 60c11636-55a1-4704-9e09-2c6bcc828de4
-    :setup: 1 Master - 1 Hub - 1 Consumer
+    :id: ef71cd7c-e792-41bf-a3c0-b3b38391cbe5
+    :setup: 1 Supplier - 1 Hub - 1 Consumer
     :steps:
         1. Configure replication to EXCLUDE memberof
         2. Enable memberof plugin
@@ -146,9 +146,10 @@ def test_memberof_with_repl(topo):
         19. user_0 should be memberof group_0 on M,H,C
     """
 
-    M1 = topo.ms["master1"]
+    M1 = topo.ms["supplier1"]
     H1 = topo.hs["hub1"]
     C1 = topo.cs["consumer1"]
+    repl = ReplicationManager(DEFAULT_SUFFIX)
 
     # Step 1 & 2
     M1.config.enable_log('audit')
@@ -168,8 +169,8 @@ def test_memberof_with_repl(topo):
     test_groups = []
 
     # Step 3
-    #In for loop create users and add them in the user list
-    #it creates user_0 to user_9 (range is fun)
+    # In for loop create users and add them in the user list
+    # it creates user_0 to user_9 (range is fun)
     for i in range(10):
         CN = '%s%d' % (USER_CN, i)
         users = UserAccounts(M1, SUFFIX)
@@ -179,26 +180,26 @@ def test_memberof_with_repl(topo):
         time.sleep(2)
         test_users.append(testuser)
 
-    #In for loop create groups and add them to the group list
-    #it creates group_0 to group_2 (range is fun)
+    # In for loop create groups and add them to the group list
+    # it creates group_0 to group_2 (range is fun)
     for i in range(3):
         CN = '%s%d' % (GROUP_CN, i)
         groups = Groups(M1, SUFFIX)
-        testgroup = groups.create(properties={'cn' : CN})
+        testgroup = groups.create(properties={'cn': CN})
         time.sleep(2)
         test_groups.append(testgroup)
 
     # Step 4
-    #Now start testing by adding differnt user to differn group
+    # Now start testing by adding differnt user to differn group
     if not ds_is_older('1.3.7'):
-           test_groups[0].remove('objectClass', 'nsMemberOf')
+        test_groups[0].remove('objectClass', 'nsMemberOf')
 
     member_dn = test_users[0].dn
     grp0_dn = test_groups[0].dn
     grp1_dn = test_groups[1].dn
 
     test_groups[0].add_member(member_dn)
-    time.sleep(5)
+    repl.wait_while_replication_is_progressing(M1, C1)
 
     # Step 5
     for i in [M1, H1, C1]:
@@ -206,12 +207,12 @@ def test_memberof_with_repl(topo):
 
     # Step 6
     test_groups[1].add_member(test_groups[0].dn)
-    time.sleep(5)
+    repl.wait_while_replication_is_progressing(M1, C1)
 
     # Step 7
     for i in [grp0_dn, grp1_dn]:
         for inst in [M1, H1, C1]:
-             _find_memberof(inst, member_dn, i)
+            _find_memberof(inst, member_dn, i)
 
     # Step 8
     for i in [M1, H1, C1]:
@@ -219,18 +220,18 @@ def test_memberof_with_repl(topo):
 
     # Step 9
     test_groups[1].remove_member(test_groups[0].dn)
-    time.sleep(5)
+    time.sleep(2)
 
     # Step 10
     # For negative testcase, we are using assertionerror
     for inst in [M1, H1, C1]:
         for i in [grp0_dn, member_dn]:
-             with pytest.raises(AssertionError):
+            with pytest.raises(AssertionError):
                 _find_memberof(inst, i, grp1_dn)
 
     # Step 11
     test_groups[0].remove_member(member_dn)
-    time.sleep(5)
+    time.sleep(2)
 
     # Step 12
     for inst in [M1, H1, C1]:
@@ -244,7 +245,7 @@ def test_memberof_with_repl(topo):
 
     # Step 14
     test_groups[0].add_member(member_dn)
-    time.sleep(5)
+    repl.wait_while_replication_is_progressing(M1, C1)
 
     # Step 15
     for i in [M1, H1]:
@@ -265,6 +266,7 @@ def test_memberof_with_repl(topo):
 
     # Step 18
     memberof.fixup(SUFFIX)
+    # have to sleep instead of task.wait() because the task opens a thread and exits
     time.sleep(5)
 
     # Step 19
@@ -296,7 +298,7 @@ def test_scheme_violation_errors_logged(topo_m2):
         6. Errors should be logged
     """
 
-    inst = topo_m2.ms["master1"]
+    inst = topo_m2.ms["supplier1"]
     memberof = MemberOfPlugin(inst)
     memberof.enable()
     memberof.set_autoaddoc('nsMemberOf')
@@ -330,7 +332,7 @@ def test_memberof_with_changelog_reset(topo_m2):
     """Test that replication does not break, after DS stop-start, due to changelog reset
 
     :id: 60c11636-55a1-4704-9e09-2c6bcc828de4
-    :setup: 2 Masters
+    :setup: 2 Suppliers
     :steps:
         1. On M1 and M2, Enable memberof
         2. On M1, add 999 entries allowing memberof
@@ -347,8 +349,8 @@ def test_memberof_with_changelog_reset(topo_m2):
         4. M1 should be stopped
         5. Replication should be working fine
     """
-    m1 = topo_m2.ms["master1"]
-    m2 = topo_m2.ms["master2"]
+    m1 = topo_m2.ms["supplier1"]
+    m2 = topo_m2.ms["supplier2"]
 
     log.info("Configure memberof on M1 and M2")
     memberof = MemberOfPlugin(m1)
@@ -369,7 +371,7 @@ def test_memberof_with_changelog_reset(topo_m2):
                          'objectclass': ensure_list_bytes(['top', 'groupOfNames'])}
 
     for user in users_list:
-        dic_of_attributes.setdefault('member',[])
+        dic_of_attributes.setdefault('member', [])
         dic_of_attributes['member'].append(user.dn)
 
     log.info('Adding the test group using async function')
@@ -427,7 +429,7 @@ def rename_entry(server, cn, from_subtree, to_subtree):
     server.rename_s(dn, nrdn, newsuperior=to_subtree, delold=0)
 
 
-def _find_memberof(server, user_dn=None, group_dn=None, find_result=True):
+def _find_memberof_ext(server, user_dn=None, group_dn=None, find_result=True):
     assert (server)
     assert (user_dn)
     assert (group_dn)
@@ -451,7 +453,7 @@ def _find_memberof(server, user_dn=None, group_dn=None, find_result=True):
 def test_memberof_group(topology_st):
     """Test memberof does not fail if group is moved into scope
 
-    :id: 552850aa-agc3-473e-9d39-aae812b46f11
+    :id: d1d276ae-6375-4ad8-9437-6a0afcbee7d2
 
     :setup: Single instance
 
@@ -495,18 +497,19 @@ def test_memberof_group(topology_st):
     dn2 = '%s,%s' % ('uid=test_m2', SUBTREE_1)
     g1 = '%s,%s' % ('cn=g1', SUBTREE_1)
     g2 = '%s,%s' % ('cn=g2', SUBTREE_2)
-    _find_memberof(inst, dn1, g1, True)
-    _find_memberof(inst, dn2, g1, True)
-    _find_memberof(inst, dn1, g2, False)
-    _find_memberof(inst, dn2, g2, False)
+    _find_memberof_ext(inst, dn1, g1, True)
+    _find_memberof_ext(inst, dn2, g1, True)
+    _find_memberof_ext(inst, dn1, g2, False)
+    _find_memberof_ext(inst, dn2, g2, False)
 
     rename_entry(inst, 'cn=g2', SUBTREE_2, SUBTREE_1)
 
     g2n = '%s,%s' % ('cn=g2-new', SUBTREE_1)
-    _find_memberof(inst, dn1, g1, True)
-    _find_memberof(inst, dn2, g1, True)
-    _find_memberof(inst, dn1, g2n, True)
-    _find_memberof(inst, dn2, g2n, True)
+    _find_memberof_ext(inst, dn1, g1, True)
+    _find_memberof_ext(inst, dn2, g1, True)
+    _find_memberof_ext(inst, dn1, g2n, True)
+    _find_memberof_ext(inst, dn2, g2n, True)
+
 
 def _config_memberof_entrycache_on_modrdn_failure(server):
 
@@ -517,11 +520,13 @@ def _config_memberof_entrycache_on_modrdn_failure(server):
                                          (ldap.MOD_REPLACE, 'memberOfEntryScope', peoplebase.encode()),
                                          (ldap.MOD_REPLACE, 'memberOfAutoAddOC', b'nsMemberOf')])
 
+
 def _disable_auto_oc_memberof(server):
     MEMBEROF_PLUGIN_DN = ('cn=' + PLUGIN_MEMBER_OF + ',cn=plugins,cn=config')
     server.modify_s(MEMBEROF_PLUGIN_DN,
         [(ldap.MOD_REPLACE, 'memberOfAutoAddOC', b'nsContainer')])
 
+
 @pytest.mark.ds49967
 def test_entrycache_on_modrdn_failure(topology_st):
     """This test checks that when a modrdn fails, the destination entry is not returned by a search
@@ -657,7 +662,7 @@ def test_entrycache_on_modrdn_failure(topology_st):
         topology_st.standalone.rename_s(group2_dn, 'cn=group_in2', newsuperior=peoplebase, delold=0)
         topology_st.standalone.log.info("This is unexpected, modrdn should fail as the member entry have not the appropriate objectclass")
         assert False
-    except ldap.OPERATIONS_ERROR:
+    except ldap.OBJECT_CLASS_VIOLATION:
         pass
 
     # retrieve the entry having the specific description value
@@ -671,9 +676,11 @@ def test_entrycache_on_modrdn_failure(topology_st):
             assert ent.dn == group2_dn
     assert found
 
+
 def _config_memberof_silent_memberof_failure(server):
     _config_memberof_entrycache_on_modrdn_failure(server)
 
+
 def test_silent_memberof_failure(topology_st):
     """This test checks that if during a MODRDN, the memberof plugin fails
     then MODRDN also fails
@@ -708,7 +715,7 @@ def test_silent_memberof_failure(topology_st):
         10. should succeed
         11. should fail OPERATION_ERROR because memberof plugin fails to add 'memberof' to members.
         12. should succeed
-        14. should fail OPERATION_ERROR because memberof plugin fails to add 'memberof' to members
+        13. should fail OPERATION_ERROR because memberof plugin fails to add 'memberof' to members
         14. should succeed
     """
     # only scopes peoplebase
@@ -817,7 +824,7 @@ def test_silent_memberof_failure(topology_st):
         topology_st.standalone.rename_s(group2_dn, 'cn=group_in2', newsuperior=peoplebase, delold=0)
         topology_st.standalone.log.info("This is unexpected, modrdn should fail as the member entry have not the appropriate objectclass")
         assert False
-    except ldap.OPERATIONS_ERROR:
+    except ldap.OBJECT_CLASS_VIOLATION:
         pass
 
     # Check the those entries have not memberof
@@ -843,7 +850,7 @@ def test_silent_memberof_failure(topology_st):
     except ldap.OPERATIONS_ERROR:
         pass
 
-    # Check the those entries have not memberof
+    # Check the those entries do not have memberof
     for i in (4, 5):
         user_dn = 'cn=user%d,%s' % (i, peoplebase)
         ent = topology_st.standalone.getEntry(user_dn, ldap.SCOPE_BASE, "(objectclass=*)", ['memberof'])
