@@ -20,10 +20,6 @@
 #include "repl5_prot_private.h"
 #include <errno.h>
 
-#define BDB_IMPL "bdb"                         /* changelog type */
-#define BDB_REPLPLUGIN "libreplication-plugin" /* This backend plugin */
-
-
 #define CL5_TYPE "Changelog5" /* changelog type */
 #define VERSION_SIZE 127      /* size of the buffer to hold changelog version */
 #define CL5_DEFAULT_CONFIG -1 /* value that indicates to changelog to use default */
@@ -36,15 +32,6 @@
 
 /***** Data Structures *****/
 
-/* changelog configuration structure */
-typedef struct cl5dbconfig
-{
-    uint32_t pageSize;           /* page size in bytes */
-    PRInt32 fileMode;          /* file mode */
-    char *encryptionAlgorithm; /* nsslapd-encryptionalgorithm */
-    char *symmetricKey;
-} CL5DBConfig;
-
 /* changelog entry format */
 typedef struct cl5entry
 {
@@ -81,12 +68,14 @@ typedef struct cl5entry
 /* data structure that allows iteration through changelog */
 typedef struct cl5replayiterator CL5ReplayIterator;
 
+/* database information for the changelog */
+typedef struct cl5DBFileHandle cldb_Handle;
+
 /* changelog state */
 typedef enum {
-    CL5_STATE_NONE,    /* changelog has not been initialized */
-    CL5_STATE_CLOSING, /* changelog is about to close; all threads must exit */
-    CL5_STATE_CLOSED,  /* changelog has been initialized, but not opened, or open and then closed */
-    CL5_STATE_OPEN     /* changelog is opened */
+    CL5_STATE_CLOSED,  /* changelog is not opened */
+    CL5_STATE_OPEN,    /* changelog is opened */
+    CL5_STATE_IMPORT   /* Changelog is being initialized by LDIF */
 } CL5State;
 
 /* error codes */
@@ -116,64 +105,43 @@ enum
 /***** Module APIs *****/
 
 /* Name:        cl5Init
-   Description:    initializes changelog module; must be called by a single thread
+   Description: initializes changelog module; must be called by a single thread
                 before any function of the module.
    Parameters:  none
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid directory is passed;
                 CL5_SYSTEM error if NSPR call fails.
  */
 int cl5Init(void);
 
-/* Name:        cl5Cleanup
-   Description:    performs cleanup of the changelog module. Must be called by a single
-                thread. It will closed db if it is still open.
-   Parameters:  none
-   Return:      none
- */
-void cl5Cleanup(void);
-
 /* Name:        cl5Open
-   Description:    opens changelog ; must be called after changelog is
+   Description: opens changelog ; must be called after changelog is
                 initialized using cl5Init. It is thread safe and the second
                 call is ignored.
-   Parameters:  dir - changelog dir
-                config - db configuration parameters; currently not used
-                openMode - open mode
-   Return:        CL5_SUCCESS if successful;
+   Return:      CL5_SUCCESS if successful;
                 CL5_BAD_DATA if invalid directory is passed;
                 CL5_BAD_DBVERSION if dbversion file is missing or has unexpected data
                 CL5_SYSTEM_ERROR if NSPR error occurred (during db directory creation);
                 CL5_MEMORY_ERROR if memory allocation fails;
                 CL5_DB_ERROR if db initialization or open fails.
  */
-int cl5Open(const char *dir, const CL5DBConfig *config);
+int cl5Open(void);
 
 /* Name:        cl5Close
-   Description:    closes changelog and cleanups changelog module; waits until
+   Description: closes changelog and cleanups changelog module; waits until
                 all threads are done using changelog
    Parameters:  none
-   Return:        CL5_SUCCESS if successful;
+   Return:      CL5_SUCCESS if successful;
                 CL5_BAD_STATE if db is not in the open state;
                 CL5_SYSTEM_ERROR if NSPR call fails
  */
 int cl5Close(void);
 
-/* Name:        cl5Delete
-   Description:    removes changelog
-   Parameters:  dir - changelog directory
-   Return:        CL5_SUCCESS if successful;
-                CL5_BAD_STATE if the changelog is not in closed state;
-                CL5_BAD_DATA if invalid directory supplied
-                CL5_SYSTEM_ERROR if NSPR call fails
- */
-int cl5Delete(const char *dir);
-
-/* Name:        cl5DeleteDBSync
-   Description: The same as cl5DeleteDB except the function does not return
-                until the file is removed.
+/* Name:        cldb_RemoveReplicaDB
+   Description: Clear the cldb information from the replica 
+                and delete the database file
 */
-int cl5DeleteDBSync(Object *replica);
+int cldb_RemoveReplicaDB(Replica *replica);
 
 /* Name:        cl5GetUpperBoundRUV
    Description: retrieves vector that represent the upper bound of changes
@@ -190,145 +158,88 @@ int cl5DeleteDBSync(Object *replica);
 int cl5GetUpperBoundRUV(Replica *r, RUV **ruv);
 
 /* Name:        cl5ExportLDIF
-   Description:    dumps changelog to an LDIF file; changelog can be open or closed.
+   Description: dumps changelog to an LDIF file; changelog can be open or closed.
    Parameters:  clDir - changelog dir
                 ldifFile - full path to ldif file to write
                 replicas - optional list of replicas whose changes should be exported;
                            if the list is NULL, entire changelog is exported.
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid parameter is passed;
                 CL5_BAD_STATE if changelog is not initialized;
                 CL5_DB_ERROR if db api fails;
                 CL5_SYSTEM_ERROR if NSPR call fails;
                 CL5_MEMORY_ERROR if memory allocation fails.
  */
-int cl5ExportLDIF(const char *ldifFile, Object **replicas);
+int cl5ExportLDIF(const char *ldifFile, Replica *replica);
 
 /* Name:        cl5ImportLDIF
-   Description:    imports ldif file into changelog; changelog must be in the closed state
+   Description: imports ldif file into changelog; changelog must be in the closed state
    Parameters:  clDir - changelog dir
                 ldifFile - absolute path to the ldif file to import
                 replicas - optional list of replicas whose data should be imported;
                            if the list is NULL, all data in the file is imported.
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid parameter is passed;
                 CL5_BAD_STATE if changelog is open or not initialized;
                 CL5_DB_ERROR if db api fails;
                 CL5_SYSTEM_ERROR if NSPR call fails;
                 CL5_MEMORY_ERROR if memory allocation fails.
  */
-int cl5ImportLDIF(const char *clDir, const char *ldifFile, Object **replicas);
-
-/* Name:        cl5GetState
-   Description:    returns database state
-   Parameters:  none
-   Return:        changelog state
- */
-
-int cl5GetState(void);
+int cl5ImportLDIF(const char *clDir, const char *ldifFile, Replica *replica);
 
 /* Name:        cl5ConfigTrimming
-   Description:    sets changelog trimming parameters
+   Description: sets changelog trimming parameters
    Parameters:  maxEntries - maximum number of entries in the log;
                 maxAge - maximum entry age;
-                compactInterval - interval to compact changelog db;
                 trimInterval - interval for changelog trimming.
-   Return:        CL5_SUCCESS if successful;
+   Return:      CL5_SUCCESS if successful;
                 CL5_BAD_STATE if changelog has not been open
  */
-int cl5ConfigTrimming(int maxEntries, const char *maxAge, int compactInterval, int trimInterval);
+int cl5ConfigTrimming(Replica *replica, int maxEntries, const char *maxAge, int trimInterval);
 
-/* Name:        cl5GetOperation
-   Description:    retrieves operation specified by its csn and databaseid
-   Parameters:  op - must contain csn and databaseid; the rest of data is
-                filled if function is successful
-   Return:        CL5_SUCCESS if function is successful;
-                CL5_BAD_DATA if invalid op is passed;
-                CL5_BAD_STATE if db has not been initialized;
-                CL5_NOTFOUND if entry was not found;
-                CL5_DB_ERROR if any other db error occurred;
-                CL5_BADFORMAT if db data format does not match entry format.
- */
-int cl5GetOperation(Object *replica, slapi_operation_parameters *op);
-
-/* Name:        cl5GetFirstOperation
-   Description: retrieves first operation for a particular database
-                replica - replica for which the operation should be retrieved.
-   Parameters:  op - buffer to store the operation;
-                iterator - to be passed to the call to cl5GetNextOperation
-   Return:        CL5_SUCCESS, if successful
-                CL5_BADDATA, if operation is NULL
-                CL5_BAD_STATE, if changelog is not open
-                CL5_DB_ERROR, if db call fails
- */
-int cl5GetFirstOperation(Object *replica, slapi_operation_parameters *op, void **iterator);
-
-/* Name:        cl5GetNextOperation
-   Description: retrieves the next op from the changelog as defined by the iterator
-   Parameters:  replica - replica for which the operation should be retrieved.
-                op - returned operation, if function is successful
-                iterator - in: identifies op to retrieve; out: identifies next op
-   Return:        CL5_SUCCESS, if successful
-                CL5_BADDATA, if invalid parameter is supplied
-                CL5_BAD_STATE, if changelog is not open
-                CL5_NOTFOUND, empty changelog
-                CL5_DB_ERROR, if db call fails
- */
-int cl5GetNextOperation(slapi_operation_parameters *op, void *iterator);
-
-/* Name:        cl5DestroyIterator
-   Description: destroys iterator once iteration through changelog is done
-   Parameters:  iterator - iterator to destroy
-   Return:        CL5_SUCCESS, if successful
-                CL5_BADDATA, if invalid parameters is supplied
-                CL5_BAD_STATE, if changelog is not open
-                CL5_DB_ERROR, if db call fails
- */
 void cl5DestroyIterator(void *iterator);
 
 /* Name:        cl5WriteOperationTxn
-   Description:    writes operation to changelog as part of a containing transaction
+   Description: writes operation to changelog as part of a containing transaction
    Parameters:  repl_name - name of the replica to which operation applies
                 repl_gen - replica generation for the operation
                 !!!Note that we pass name and generation rather than
                    replica object since generation can change while operation
                    is in progress (if the data is reloaded). !!!
                 op - operation to write
-                local - this is a non-replicated operation
                 txn - the containing transaction
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid op is passed;
                 CL5_BAD_STATE if db has not been initialized;
                 CL5_MEMORY_ERROR if memory allocation failed;
                 CL5_DB_ERROR if any other db error occurred;
  */
-int cl5WriteOperationTxn(const char *repl_name, const char *repl_gen, const slapi_operation_parameters *op, PRBool local, void *txn);
+int cl5WriteOperationTxn(cldb_Handle *cldb, const slapi_operation_parameters *op, void *txn);
 
 /* Name:        cl5WriteOperation
-   Description:    writes operation to changelog
+   Description: writes operation to changelog
    Parameters:  repl_name - name of the replica to which operation applies
                 repl_gen - replica generation for the operation
                 !!!Note that we pass name and generation rather than
                    replica object since generation can change while operation
                    is in progress (if the data is reloaded). !!!
                 op - operation to write
-                local - this is a non-replicated operation
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid op is passed;
                 CL5_BAD_STATE if db has not been initialized;
                 CL5_MEMORY_ERROR if memory allocation failed;
                 CL5_DB_ERROR if any other db error occurred;
  */
-int cl5WriteOperation(const char *repl_name, const char *repl_gen, const slapi_operation_parameters *op, PRBool local);
+int cl5WriteOperation(cldb_Handle *cldb, const slapi_operation_parameters *op);
 
 /* Name:        cl5CreateReplayIterator
-   Description:    creates an iterator that allows to retrieve changes that should
+   Description: creates an iterator that allows to retrieve changes that should
                 to be sent to the consumer identified by ruv The iteration is performed by
                 repeated calls to cl5GetNextOperationToReplay.
    Parameters:  replica - replica whose data we wish to iterate;
                 ruv - consumer ruv;
                 iterator - iterator to be passed to cl5GetNextOperationToReplay call
-   Return:        CL5_SUCCESS, if function is successful;
+   Return:      CL5_SUCCESS, if function is successful;
                 CL5_MISSING_DATA, if data that should be in the changelog is missing
                 CL5_PURGED_DATA, if some data that consumer needs has been purged.
                 Note that the iterator can be non null if the supplier contains
@@ -344,13 +255,13 @@ int cl5CreateReplayIteratorEx(Private_Repl_Protocol *prp, const RUV *consumerRuv
 
 
 /* Name:        cl5GetNextOperationToReplay
-   Description:    retrieves next operation to be sent to the consumer and
-                that was created on a particular master. Consumer and master info
+   Description: retrieves next operation to be sent to the consumer and
+                that was created on a particular supplier. Consumer and supplier info
                 is encoded in the iterator parameter that must be created by calling
                 to cl5CreateIterator.
    Parameters:  iterator - iterator that identifies next entry to retrieve;
                 op - operation retrieved if function is successful
-   Return:        CL5_SUCCESS if function is successful;
+   Return:      CL5_SUCCESS if function is successful;
                 CL5_BAD_DATA if invalid parameter is passed;
                 CL5_NOTFOUND if end of iteration list is reached
                 CL5_DB_ERROR if any other db error occurred;
@@ -361,45 +272,30 @@ int cl5GetNextOperationToReplay(CL5ReplayIterator *iterator,
                                 CL5Entry *entry);
 
 /* Name:        cl5DestroyReplayIterator
-   Description:    destroys iterator
+   Description: destroys iterator
    Parameters:  iterator - iterator to destroy
-   Return:        none
+   Parameters:  replica for changelog info
+   Return:      none
  */
-void cl5DestroyReplayIterator(CL5ReplayIterator **iterator);
+void cl5DestroyReplayIterator(CL5ReplayIterator **iterator, Replica *replica);
 
-/* Name:        cl5DeleteOnClose
-   Description:    marks changelog for deletion when it is closed
-   Parameters:  flag; if flag = 1 then delete else don't
-   Return:        none
+/* Name:        cl5GetLdifDir
+   Description: returns the default ldif directory; must be freed by the caller;
+   Parameters:  backend used for export/import
+   Return:      copy of the directory; caller needs to free the string
  */
 
-void cl5DeleteOnClose(PRBool rm);
-
-/* Name:        cl5GetDir
-   Description:    returns changelog directory; must be freed by the caller;
-   Parameters:  none
-   Return:        copy of the directory; caller needs to free the string
- */
-
-char *cl5GetDir(void);
-
-/* Name: cl5Exist
-   Description: checks if a changelog exists in the specified directory
-   Parameters: clDir - directory to check;
-   Return: 1 - if changelog exists; 0 - otherwise
- */
-
-PRBool cl5Exist(const char *clDir);
+char *cl5GetLdifDir(Slapi_Backend *be);
 
 /* Name: cl5GetOperationCount
    Description: returns number of entries in the changelog. The changelog must be
                 open for the value to be meaningful.
    Parameters:  replica - optional parameter that specifies the replica whose operations
                 we wish to count; if NULL all changelog entries are counted
-   Return:        number of entries in the changelog
+   Return:      number of entries in the changelog
  */
 
-int cl5GetOperationCount(Object *replica);
+int cl5GetOperationCount(Replica *replica);
 
 /* Name: cl5_operation_parameters_done
    Description: frees all parameters that are not freed by operation_parameters_done
@@ -411,45 +307,31 @@ void cl5_operation_parameters_done(struct slapi_operation_parameters *sop);
 
 /* Name: cl5CreateDirIfNeeded
    Description: Create the directory if it doesn't exist yet
-   Parameters:    dir - Contains the name of the directory to create. Must not be NULL
-   Return:        CL5_SUCCESS if succeeded or existed,
+   Parameters:  dir - Contains the name of the directory to create. Must not be NULL
+   Return:      CL5_SUCCESS if succeeded or existed,
                 CL5_SYSTEM_ERROR if failed.
 */
 
 int cl5CreateDirIfNeeded(const char *dir);
-int cl5DBData2Entry(const char *data, PRUint32 len, CL5Entry *entry);
+int cl5DBData2Entry(const char *data, PRUint32 len, CL5Entry *entry, void *clcrypt_handle);
 
 PRBool cl5HelperEntry(const char *csnstr, CSN *csn);
 CSN **cl5BuildCSNList(const RUV *consRuv, const RUV *supRuv);
 void cl5DestroyCSNList(CSN ***csns);
 
-int cl5_is_diskfull(void);
-int cl5_diskspace_is_available(void);
+int cl5Export(Slapi_PBlock *pb);
+int cl5Import(Slapi_PBlock *pb);
 
-/* Name: cl5DbDirIsEmpty
-   Description: See if the given cldb directory is empty or doesn't yet exist.
-   Parameters:    dir - Contains the name of the directory.
-   Return:        TRUE - directory does not exist or is empty, is NULL, or is
-                       an empty string
-                FALSE - otherwise
-*/
-int cl5DbDirIsEmpty(const char *dir);
+int cl5NotifyRUVChange(Replica *replica);
 
-/* Name: cl5WriteRUV
-   Description: Write RUVs into changelog db's.  Called before backup.
-   Parameters:    none
-   Return:        TRUE
-*/
-int cl5WriteRUV(void);
-
-/* Name: cl5DeleteRUV
-   Description: Read and delete RUVs from changelog db's.  Called after backup.
-   Parameters:    none
-   Return:        TRUE
-*/
-int cl5DeleteRUV(void);
-void cl5CleanRUV(ReplicaId rid);
+void cl5CleanRUV(ReplicaId rid, Replica *replica);
 void cl5NotifyCleanup(int rid);
 void trigger_cl_purging(cleanruv_purge_data *purge_data);
+int cldb_SetReplicaDB(Replica *replica, void *arg);
+int cldb_UnSetReplicaDB(Replica *replica, void *arg);
+int cldb_StartTrimming(Replica *replica);
+int cldb_StopTrimming(Replica *replica, void *arg);
+int cldb_StopThreads(Replica *replica, void *arg);
+int32_t cldb_is_open(Replica *replica);
 
 #endif
