@@ -1,6 +1,6 @@
 /** BEGIN COPYRIGHT BLOCK
  * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
- * Copyright (C) 2005 Red Hat, Inc.
+ * Copyright (C) 2021 Red Hat, Inc.
  * Copyright (C) 2009 Hewlett-Packard Development Company, L.P.
  * All rights reserved.
  *
@@ -64,7 +64,7 @@ static const char *replica_get_purl_for_op(const Replica *r, Slapi_PBlock *pb, c
  * do you know which to use? Offer a choice in replication config?
  */
 int
-multimaster_set_local_purl()
+multisupplier_set_local_purl()
 {
     int rc = 0;
     Slapi_Entry **entries;
@@ -74,26 +74,26 @@ multimaster_set_local_purl()
 
     slapi_search_internal_set_pb(pb, "cn=config", LDAP_SCOPE_BASE,
                                  "objectclass=*", purl_attrs, 0, NULL, NULL,
-                                 repl_get_plugin_identity(PLUGIN_MULTIMASTER_REPLICATION), 0);
+                                 repl_get_plugin_identity(PLUGIN_MULTISUPPLIER_REPLICATION), 0);
     slapi_search_internal_pb(pb);
     slapi_pblock_get(pb, SLAPI_PLUGIN_INTOP_RESULT, &rc);
     if (rc != 0) {
-        slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "multimaster_set_local_purl - "
+        slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "multisupplier_set_local_purl - "
                                                        "unable to read server configuration: error %d\n",
                       rc);
     } else {
         slapi_pblock_get(pb, SLAPI_PLUGIN_INTOP_SEARCH_ENTRIES, &entries);
         if (NULL == entries || NULL == entries[0]) {
-            slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "multimaster_set_local_purl - "
+            slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "multisupplier_set_local_purl - "
                                                            "Server configuration missing\n");
             rc = -1;
         } else {
-            char *host = slapi_entry_attr_get_charptr(entries[0], "nsslapd-localhost");
-            char *port = slapi_entry_attr_get_charptr(entries[0], "nsslapd-port");
-            char *sslport = slapi_entry_attr_get_charptr(entries[0], "nsslapd-secureport");
+            char *host = (char *)slapi_entry_attr_get_ref(entries[0], "nsslapd-localhost");
+            char *port = (char *)slapi_entry_attr_get_ref(entries[0], "nsslapd-port");
+            char *sslport = (char *)slapi_entry_attr_get_ref(entries[0], "nsslapd-secureport");
             if (host == NULL || ((port == NULL && sslport == NULL))) {
                 slapi_log_err(SLAPI_LOG_WARNING, repl_plugin_name,
-                              "multimaster_set_local_purl - Invalid server "
+                              "multisupplier_set_local_purl - Invalid server "
                               "configuration\n");
             } else {
                 if (slapi_is_ipv6_addr(host)) {
@@ -103,11 +103,6 @@ multimaster_set_local_purl()
                     local_purl = slapi_ch_smprintf("ldap://%s:%s", host, port);
                 }
             }
-
-            /* slapi_ch_free acceptS NULL pointer */
-            slapi_ch_free((void **)&host);
-            slapi_ch_free((void **)&port);
-            slapi_ch_free((void **)&sslport);
         }
     }
     slapi_free_search_results_internal(pb);
@@ -118,23 +113,23 @@ multimaster_set_local_purl()
 
 
 const char *
-multimaster_get_local_purl()
+multisupplier_get_local_purl()
 {
     return local_purl;
 }
 
 
-/* ================= Multimaster Pre-Op Plugin Points ================== */
+/* ================= Multisupplier Pre-Op Plugin Points ================== */
 
 
 int
-multimaster_preop_bind(Slapi_PBlock *pb __attribute__((unused)))
+multisupplier_preop_bind(Slapi_PBlock *pb __attribute__((unused)))
 {
     return 0;
 }
 
 int
-multimaster_preop_add(Slapi_PBlock *pb)
+multisupplier_preop_add(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
     int is_replicated_operation;
@@ -163,7 +158,7 @@ multimaster_preop_add(Slapi_PBlock *pb)
                 int drc = decode_NSDS50ReplUpdateInfoControl(ctrlp, &target_uuid, &superior_uuid, &csn, NULL /* modrdn_mods */);
                 if (-1 == drc) {
                     slapi_log_err(SLAPI_LOG_ERR, REPLICATION_SUBSYSTEM,
-                                  "multimaster_preop_add - %s An error occurred while decoding the replication update "
+                                  "multisupplier_preop_add - %s An error occurred while decoding the replication update "
                                   "control - Add\n",
                                   sessionid);
                 } else if (1 == drc) {
@@ -199,20 +194,18 @@ multimaster_preop_add(Slapi_PBlock *pb)
                          * same as the one in the entry.
                          */
                         Slapi_Entry *addentry;
-                        char *entry_uuid;
+                        const char *entry_uuid;
                         slapi_pblock_get(pb, SLAPI_ADD_ENTRY, &addentry);
-                        entry_uuid = slapi_entry_attr_get_charptr(addentry, SLAPI_ATTR_UNIQUEID);
+                        entry_uuid = slapi_entry_attr_get_ref(addentry, SLAPI_ATTR_UNIQUEID);
                         if (entry_uuid == NULL) {
                             /* Odd that the entry doesn't have a Unique Identifier. But, we can fix it. */
                             slapi_entry_set_uniqueid(addentry, slapi_ch_strdup(target_uuid)); /* JCMREPL - strdup EVIL! There should be a uuid dup function. */
                         } else {
                             if (strcasecmp(entry_uuid, target_uuid) != 0) {
                                 slapi_log_err(SLAPI_LOG_WARNING, REPLICATION_SUBSYSTEM,
-                                              "multimaster_preop_add - %s Replicated Add received with Control_UUID=%s and Entry_UUID=%s.\n",
+                                              "multisupplier_preop_add - %s Replicated Add received with Control_UUID=%s and Entry_UUID=%s.\n",
                                               sessionid, target_uuid, entry_uuid);
                             }
-
-                            slapi_ch_free((void **)&entry_uuid);
                         }
                     }
                 }
@@ -232,7 +225,7 @@ multimaster_preop_add(Slapi_PBlock *pb)
 }
 
 int
-multimaster_preop_delete(Slapi_PBlock *pb)
+multisupplier_preop_delete(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
     int is_replicated_operation;
@@ -260,7 +253,7 @@ multimaster_preop_delete(Slapi_PBlock *pb)
                 int drc = decode_NSDS50ReplUpdateInfoControl(ctrlp, &target_uuid, NULL, &csn, NULL /* modrdn_mods */);
                 if (-1 == drc) {
                     slapi_log_err(SLAPI_LOG_ERR, REPLICATION_SUBSYSTEM,
-                                  "multimaster_preop_delete - %s An error occurred while decoding the replication update "
+                                  "multisupplier_preop_delete - %s An error occurred while decoding the replication update "
                                   "control - Delete\n",
                                   sessionid);
                 } else if (1 == drc) {
@@ -272,7 +265,7 @@ multimaster_preop_delete(Slapi_PBlock *pb)
                                                "or csn ignored",
                                                0, 0);
                         slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
-                                      "multimaster_preop_delete - %s replication operation not processed, replica unavailable "
+                                      "multisupplier_preop_delete - %s replication operation not processed, replica unavailable "
                                       "or csn ignored\n",
                                       sessionid);
                         csn_free(&csn);
@@ -306,7 +299,7 @@ multimaster_preop_delete(Slapi_PBlock *pb)
 }
 
 int
-multimaster_preop_modify(Slapi_PBlock *pb)
+multisupplier_preop_modify(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
     int is_replicated_operation;
@@ -334,7 +327,7 @@ multimaster_preop_modify(Slapi_PBlock *pb)
                 int drc = decode_NSDS50ReplUpdateInfoControl(ctrlp, &target_uuid, NULL, &csn, NULL /* modrdn_mods */);
                 if (-1 == drc) {
                     slapi_log_err(SLAPI_LOG_ERR, REPLICATION_SUBSYSTEM,
-                                  "multimaster_preop_modify - %s An error occurred while decoding the replication update "
+                                  "multisupplier_preop_modify - %s An error occurred while decoding the replication update "
                                   "control- Modify\n",
                                   sessionid);
                 } else if (1 == drc) {
@@ -346,7 +339,7 @@ multimaster_preop_modify(Slapi_PBlock *pb)
                                                "or csn ignored",
                                                0, 0);
                         slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
-                                      "multimaster_preop_modify - %s replication operation not processed, replica unavailable "
+                                      "multisupplier_preop_modify - %s replication operation not processed, replica unavailable "
                                       "or csn ignored\n",
                                       sessionid);
                         csn_free(&csn);
@@ -366,7 +359,7 @@ multimaster_preop_modify(Slapi_PBlock *pb)
             } else {
                 /*  PR_ASSERT(0); JCMREPL - A Replicated Operation with no Repl Baggage control... What does that mean? */
                 /*
-                 *  This could be RI plugin responding to a replicated update from AD or some other master that is not
+                 *  This could be RI plugin responding to a replicated update from AD or some other supplier that is not
                  *  using the RI plugin, so don't PR_ASSERT here.  This only happens if we configure the RI plugin with
                  *  "nsslapd-pluginAllowReplUpdates: on", also the RI plugin only issues "modifies".
                  */
@@ -383,7 +376,7 @@ multimaster_preop_modify(Slapi_PBlock *pb)
 }
 
 int
-multimaster_preop_modrdn(Slapi_PBlock *pb)
+multisupplier_preop_modrdn(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
     int is_replicated_operation;
@@ -414,7 +407,7 @@ multimaster_preop_modrdn(Slapi_PBlock *pb)
                                                              &csn, &modrdn_mods);
                 if (-1 == drc) {
                     slapi_log_err(SLAPI_LOG_ERR, REPLICATION_SUBSYSTEM,
-                                  "multimaster_preop_modrdn - %s An error occurred while decoding the replication update "
+                                  "multisupplier_preop_modrdn - %s An error occurred while decoding the replication update "
                                   "control - ModRDN\n",
                                   sessionid);
                 } else if (1 == drc) {
@@ -482,19 +475,19 @@ multimaster_preop_modrdn(Slapi_PBlock *pb)
 }
 
 int
-multimaster_preop_search(Slapi_PBlock *pb __attribute__((unused)))
+multisupplier_preop_search(Slapi_PBlock *pb __attribute__((unused)))
 {
     return SLAPI_PLUGIN_SUCCESS;
 }
 
 int
-multimaster_preop_compare(Slapi_PBlock *pb __attribute__((unused)))
+multisupplier_preop_compare(Slapi_PBlock *pb __attribute__((unused)))
 {
     return SLAPI_PLUGIN_SUCCESS;
 }
 
 int
-multimaster_ruv_search(Slapi_PBlock *pb)
+multisupplier_ruv_search(Slapi_PBlock *pb)
 {
     Slapi_Entry *e, *e_alt;
     Slapi_DN *suffix_sdn;
@@ -530,7 +523,6 @@ static void
 purge_entry_state_information(Slapi_PBlock *pb)
 {
     CSN *purge_csn = NULL;
-    Object *repl_obj;
     Replica *replica;
 
     /* we don't want to  trim RUV tombstone because we will
@@ -538,101 +530,107 @@ purge_entry_state_information(Slapi_PBlock *pb)
     if (ruv_tombstone_op(pb))
         return;
 
-    repl_obj = replica_get_replica_for_op(pb);
-    if (NULL != repl_obj) {
-        replica = object_get_data(repl_obj);
-        if (NULL != replica) {
-            purge_csn = replica_get_purge_csn(replica);
-        }
-        if (NULL != purge_csn) {
-            Slapi_Entry *e;
-            int optype;
+    replica = replica_get_replica_for_op(pb);
+    if (NULL != replica) {
+        purge_csn = replica_get_purge_csn(replica);
+    }
+    if (NULL != purge_csn) {
+        Slapi_Entry *e;
+        int optype;
 
-            slapi_pblock_get(pb, SLAPI_OPERATION_TYPE, &optype);
-            switch (optype) {
-            case SLAPI_OPERATION_MODIFY:
-                slapi_pblock_get(pb, SLAPI_MODIFY_EXISTING_ENTRY, &e);
-                break;
-            case SLAPI_OPERATION_MODRDN:
-                /* XXXggood - the following always gives a NULL entry - why? */
-                slapi_pblock_get(pb, SLAPI_MODRDN_EXISTING_ENTRY, &e);
-                break;
-            case SLAPI_OPERATION_DELETE:
-                slapi_pblock_get(pb, SLAPI_DELETE_EXISTING_ENTRY, &e);
-                break;
-            default:
-                e = NULL; /* Don't purge on ADD - not needed */
-                break;
-            }
-            if (NULL != e) {
-                entry_purge_state_information(e, purge_csn);
-                /* conntion is always null */
-                if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
-                    char csn_str[CSN_STRSIZE];
-                    slapi_log_err(SLAPI_LOG_REPL, repl_plugin_name,
-                                  "purge_entry_state_information -  From entry %s up to "
-                                  "CSN %s\n",
-                                  slapi_entry_get_dn(e),
-                                  csn_as_string(purge_csn, PR_FALSE, csn_str));
-                }
-            }
-            csn_free(&purge_csn);
+        slapi_pblock_get(pb, SLAPI_OPERATION_TYPE, &optype);
+        switch (optype) {
+        case SLAPI_OPERATION_MODIFY:
+            slapi_pblock_get(pb, SLAPI_MODIFY_EXISTING_ENTRY, &e);
+            break;
+        case SLAPI_OPERATION_MODRDN:
+            /* XXXggood - the following always gives a NULL entry - why? */
+            slapi_pblock_get(pb, SLAPI_MODRDN_EXISTING_ENTRY, &e);
+            break;
+        case SLAPI_OPERATION_DELETE:
+            slapi_pblock_get(pb, SLAPI_DELETE_EXISTING_ENTRY, &e);
+            break;
+        default:
+            e = NULL; /* Don't purge on ADD - not needed */
+            break;
         }
-        object_release(repl_obj);
+        if (NULL != e) {
+            entry_purge_state_information(e, purge_csn);
+            /* conntion is always null */
+            if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+                char csn_str[CSN_STRSIZE];
+                slapi_log_err(SLAPI_LOG_REPL, repl_plugin_name,
+                              "purge_entry_state_information -  From entry %s up to "
+                              "CSN %s\n",
+                              slapi_entry_get_dn(e),
+                              csn_as_string(purge_csn, PR_FALSE, csn_str));
+            }
+        }
+        csn_free(&purge_csn);
     }
 }
 int
 
-multimaster_mmr_preop (Slapi_PBlock *pb, int flags)
+multisupplier_mmr_preop (Slapi_PBlock *pb, int flags)
 {
 	int rc= SLAPI_PLUGIN_SUCCESS;
 
+    if (!is_mmr_replica(pb)) {
+        return rc;
+    }
+
 	switch (flags)
 	{
 	case SLAPI_PLUGIN_BE_PRE_ADD_FN:
-		rc = multimaster_bepreop_add(pb);
+		rc = multisupplier_bepreop_add(pb);
 		break;
 	case SLAPI_PLUGIN_BE_PRE_MODIFY_FN:
-		rc = multimaster_bepreop_modify(pb);
+		rc = multisupplier_bepreop_modify(pb);
 		break;
 	case SLAPI_PLUGIN_BE_PRE_MODRDN_FN:
-		rc = multimaster_bepreop_modrdn(pb);
+		rc = multisupplier_bepreop_modrdn(pb);
 		break;
 	case SLAPI_PLUGIN_BE_PRE_DELETE_FN:
-		rc = multimaster_bepreop_delete(pb);
+		rc = multisupplier_bepreop_delete(pb);
 		break;
 	}
 	return rc;
 }
 
 int
-multimaster_mmr_postop (Slapi_PBlock *pb, int flags)
+multisupplier_mmr_postop (Slapi_PBlock *pb, int flags)
 {
 	int rc= SLAPI_PLUGIN_SUCCESS;
 
+    if (!is_mmr_replica(pb)) {
+        return rc;
+    }
+
 	switch (flags)
 	{
 	case SLAPI_PLUGIN_BE_TXN_POST_ADD_FN:
-		rc = multimaster_be_betxnpostop_add(pb);
+		rc = multisupplier_be_betxnpostop_add(pb);
 		break;
 	case SLAPI_PLUGIN_BE_TXN_POST_DELETE_FN:
-		rc = multimaster_be_betxnpostop_delete(pb);
+		rc = multisupplier_be_betxnpostop_delete(pb);
 		break;
 	case SLAPI_PLUGIN_BE_TXN_POST_MODIFY_FN:
-		rc = multimaster_be_betxnpostop_modify(pb);
+		rc = multisupplier_be_betxnpostop_modify(pb);
 		break;
 	case SLAPI_PLUGIN_BE_TXN_POST_MODRDN_FN:
-		rc = multimaster_be_betxnpostop_modrdn(pb);
+		rc = multisupplier_be_betxnpostop_modrdn(pb);
 		break;
 	}
-	slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
-                     "multimaster_mmr_postop - error %d for oparation %d.\n", rc, flags);
+    if (rc) {
+	    slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
+                     "multisupplier_mmr_postop - error %d for operation %d.\n", rc, flags);
+	}
 	return rc;
 }
 
 /* pure bepreop's -- should be done before transaction starts */
 int
-multimaster_bepreop_add(Slapi_PBlock *pb)
+multisupplier_bepreop_add(Slapi_PBlock *pb)
 {
     int rc = SLAPI_PLUGIN_SUCCESS;
     Slapi_Operation *op;
@@ -656,7 +654,7 @@ multimaster_bepreop_add(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepreop_delete(Slapi_PBlock *pb)
+multisupplier_bepreop_delete(Slapi_PBlock *pb)
 {
     int rc = SLAPI_PLUGIN_SUCCESS;
     Slapi_Operation *op;
@@ -680,7 +678,7 @@ multimaster_bepreop_delete(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepreop_modify(Slapi_PBlock *pb)
+multisupplier_bepreop_modify(Slapi_PBlock *pb)
 {
     int rc = SLAPI_PLUGIN_SUCCESS;
     Slapi_Operation *op;
@@ -707,7 +705,7 @@ multimaster_bepreop_modify(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepreop_modrdn(Slapi_PBlock *pb)
+multisupplier_bepreop_modrdn(Slapi_PBlock *pb)
 {
     int rc = SLAPI_PLUGIN_SUCCESS;
     Slapi_Operation *op;
@@ -734,7 +732,7 @@ multimaster_bepreop_modrdn(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepostop_add(Slapi_PBlock *pb)
+multisupplier_bepostop_add(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
 
@@ -746,7 +744,7 @@ multimaster_bepostop_add(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepostop_modrdn(Slapi_PBlock *pb)
+multisupplier_bepostop_modrdn(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
 
@@ -758,7 +756,7 @@ multimaster_bepostop_modrdn(Slapi_PBlock *pb)
 }
 
 int
-multimaster_bepostop_delete(Slapi_PBlock *pb)
+multisupplier_bepostop_delete(Slapi_PBlock *pb)
 {
     Slapi_Operation *op;
 
@@ -771,94 +769,94 @@ multimaster_bepostop_delete(Slapi_PBlock *pb)
 
 /* postop - write to changelog */
 int
-multimaster_postop_bind(Slapi_PBlock *pb __attribute__((unused)))
+multisupplier_postop_bind(Slapi_PBlock *pb __attribute__((unused)))
 {
     return SLAPI_PLUGIN_SUCCESS;
 }
 
 int
-multimaster_postop_add(Slapi_PBlock *pb)
+multisupplier_postop_add(Slapi_PBlock *pb)
 {
     return process_postop(pb);
 }
 
 int
-multimaster_postop_delete(Slapi_PBlock *pb)
+multisupplier_postop_delete(Slapi_PBlock *pb)
 {
     return process_postop(pb);
 }
 
 int
-multimaster_postop_modify(Slapi_PBlock *pb)
+multisupplier_postop_modify(Slapi_PBlock *pb)
 {
     return process_postop(pb);
 }
 
 int
-multimaster_postop_modrdn(Slapi_PBlock *pb)
+multisupplier_postop_modrdn(Slapi_PBlock *pb)
 {
     return process_postop(pb);
 }
 
 int
-multimaster_betxnpostop_delete(Slapi_PBlock *pb)
+multisupplier_betxnpostop_delete(Slapi_PBlock *pb)
 {
     return write_changelog_and_ruv(pb);
 }
 
 int
-multimaster_betxnpostop_modrdn(Slapi_PBlock *pb)
+multisupplier_betxnpostop_modrdn(Slapi_PBlock *pb)
 {
     return write_changelog_and_ruv(pb);
 }
 
 int
-multimaster_betxnpostop_add(Slapi_PBlock *pb)
+multisupplier_betxnpostop_add(Slapi_PBlock *pb)
 {
     return write_changelog_and_ruv(pb);
 }
 
 int
-multimaster_betxnpostop_modify(Slapi_PBlock *pb)
+multisupplier_betxnpostop_modify(Slapi_PBlock *pb)
 {
     return write_changelog_and_ruv(pb);
 }
 
 /* If nsslapd-pluginbetxn is on */
 int
-multimaster_be_betxnpostop_delete(Slapi_PBlock *pb)
+multisupplier_be_betxnpostop_delete(Slapi_PBlock *pb)
 {
     int rc = SLAPI_PLUGIN_SUCCESS;
     /* original betxnpost */
     rc = write_changelog_and_ruv(pb);
     /* original bepost */
-    rc |= multimaster_bepostop_delete(pb);
+    rc |= multisupplier_bepostop_delete(pb);
     return rc;
 }
 
 int
-multimaster_be_betxnpostop_modrdn(Slapi_PBlock *pb)
+multisupplier_be_betxnpostop_modrdn(Slapi_PBlock *pb)
 {
     int rc = 0;
     /* original betxnpost */
     rc = write_changelog_and_ruv(pb);
     /* original bepost */
-    rc |= multimaster_bepostop_modrdn(pb);
+    rc |= multisupplier_bepostop_modrdn(pb);
     return rc;
 }
 
 int
-multimaster_be_betxnpostop_add(Slapi_PBlock *pb)
+multisupplier_be_betxnpostop_add(Slapi_PBlock *pb)
 {
     int rc = 0;
     /* original betxnpost */
     rc = write_changelog_and_ruv(pb);
-    rc |= multimaster_bepostop_add(pb);
+    rc |= multisupplier_bepostop_add(pb);
     return rc;
 }
 
 int
-multimaster_be_betxnpostop_modify(Slapi_PBlock *pb)
+multisupplier_be_betxnpostop_modify(Slapi_PBlock *pb)
 {
     int rc = 0;
     /* original betxnpost */
@@ -880,42 +878,34 @@ copy_operation_parameters(Slapi_PBlock *pb)
     Slapi_Operation *op = NULL;
     struct slapi_operation_parameters *op_params;
     supplier_operation_extension *opext;
-    Object *repl_obj;
     Replica *replica;
 
-    repl_obj = replica_get_replica_for_op(pb);
+    replica = replica_get_replica_for_op(pb);
 
     /* we are only interested in the updates to replicas */
-    if (repl_obj) {
-        /* we only save the original operation parameters for replicated operations
-           since client operations don't go through urp engine and pblock data can be logged */
-        slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-        if (NULL == op) {
-            slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
-                          "copy_operation_parameters - operation is null.\n");
-            return;
-        }
-        replica = (Replica *)object_get_data(repl_obj);
-        if (NULL == replica) {
-            slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
-                          "copy_operation_parameters - replica is null.\n");
-            return;
-        }
-        opext = (supplier_operation_extension *)repl_sup_get_ext(REPL_SUP_EXT_OP, op);
-        if (operation_is_flag_set(op, OP_FLAG_REPLICATED) &&
-            !operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) {
-            slapi_pblock_get(pb, SLAPI_OPERATION_PARAMETERS, &op_params);
-            opext->operation_parameters = operation_parameters_dup(op_params);
-        }
+    if (NULL == replica) {
+        return;
+    }
+    /* we only save the original operation parameters for replicated operations
+       since client operations don't go through urp engine and pblock data can be logged */
+    slapi_pblock_get(pb, SLAPI_OPERATION, &op);
+    if (NULL == op) {
+        slapi_log_err(SLAPI_LOG_REPL, REPLICATION_SUBSYSTEM,
+                      "copy_operation_parameters - operation is null.\n");
+        return;
+    }
+    opext = (supplier_operation_extension *)repl_sup_get_ext(REPL_SUP_EXT_OP, op);
+    if (operation_is_flag_set(op, OP_FLAG_REPLICATED) &&
+        !operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) {
+        slapi_pblock_get(pb, SLAPI_OPERATION_PARAMETERS, &op_params);
+        opext->operation_parameters = operation_parameters_dup(op_params);
+    }
 
-        /* this condition is needed to avoid re-entering backend serial lock
-           when ruv state is updated */
-        if (!operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) {
-            /* save replica generation in case it changes */
-            opext->repl_gen = replica_get_generation(replica);
-        }
-
-        object_release(repl_obj);
+    /* this condition is needed to avoid re-entering backend serial lock
+       when ruv state is updated */
+    if (!operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) {
+        /* save replica generation in case it changes */
+        opext->repl_gen = replica_get_generation(replica);
     }
 }
 
@@ -958,7 +948,6 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
     CSNPL_CTX *prim_csn;
     int rc;
     slapi_operation_parameters *op_params = NULL;
-    Object *repl_obj = NULL;
     int return_value = SLAPI_PLUGIN_SUCCESS;
     Replica *r;
     Slapi_Backend *be;
@@ -985,13 +974,10 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
         return return_value;
     }
     /* we only log changes for operations applied to a replica */
-    repl_obj = replica_get_replica_for_op(pb);
-    if (repl_obj == NULL)
+    r = replica_get_replica_for_op(pb);
+    if (r == NULL)
         return return_value;
 
-    r = (Replica *)object_get_data(repl_obj);
-    PR_ASSERT(r);
-
     slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
     if (rc) { /* op failed - just return */
         cancel_opcsn(pb);
@@ -1001,19 +987,20 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
 
     replica_check_release_timeout(r, pb);
 
-    if (replica_is_flag_set(r, REPLICA_LOG_CHANGES) &&
-        (cl5GetState() == CL5_STATE_OPEN)) {
+    if (replica_is_flag_set(r, REPLICA_LOG_CHANGES)) {
         supplier_operation_extension *opext = NULL;
-        const char *repl_name;
-        char *repl_gen;
+        cldb_Handle *cldb  = NULL;
 
         opext = (supplier_operation_extension *)repl_sup_get_ext(REPL_SUP_EXT_OP, op);
         PR_ASSERT(opext);
 
-        /* get replica generation and replica name to pass to the write function */
-        repl_name = replica_get_name(r);
-        repl_gen = opext->repl_gen;
-        PR_ASSERT(repl_name && repl_gen);
+        /* changelog database information to pass to the write function */
+        cldb = replica_get_cl_info(r);
+        if (cldb == NULL) {
+            slapi_log_err(SLAPI_LOG_REPL, repl_plugin_name_cl,
+                          "write_changelog_and_ruv - changelog is not initialized\n");
+            return return_value;
+        }
 
         /* for replicated operations, we log the original, non-urp data which is
            saved in the operation extension */
@@ -1056,6 +1043,9 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
 
         if (op_params->csn && is_cleaned_rid(csn_get_replicaid(op_params->csn))) {
             /* this RID has been cleaned */
+            if (!operation_is_flag_set(op, OP_FLAG_REPLICATED)) {
+                slapi_ch_free((void **)&op_params->target_address.uniqueid);
+            }
             goto common_return;
         }
 
@@ -1066,6 +1056,9 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
         {
             slapi_log_err(SLAPI_LOG_REPL, "write_changelog_and_ruv",
                           "Skipping internal operation on read-only replica\n");
+            if (!operation_is_flag_set(op, OP_FLAG_REPLICATED)) {
+                slapi_ch_free((void **)&op_params->target_address.uniqueid);
+            }
             goto common_return;
         }
 
@@ -1075,11 +1068,6 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
             op_params->p.p_modify.modify_mods != NULL) {
             void *txn = NULL;
             char csn_str[CSN_STRSIZE];
-            if (cl5_is_diskfull() && !cl5_diskspace_is_available()) {
-                slapi_log_err(SLAPI_LOG_CRIT, repl_plugin_name,
-                              "write_changelog_and_ruv - Skipped due to DISKFULL\n");
-                goto common_return;
-            }
             slapi_pblock_get(pb, SLAPI_TXN, &txn);
             slapi_log_err(SLAPI_LOG_REPL, repl_plugin_name,
                           "write_changelog_and_ruv - Writing change for "
@@ -1088,8 +1076,7 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
                           op_params->target_address.uniqueid,
                           op_params->operation_type,
                           csn_as_string(op_params->csn, PR_FALSE, csn_str));
-            rc = cl5WriteOperationTxn(repl_name, repl_gen, op_params,
-                                      !operation_is_flag_set(op, OP_FLAG_REPLICATED), txn);
+            rc = cl5WriteOperationTxn(cldb, op_params, txn);
             if (rc != CL5_SUCCESS) {
                 /* ONREPL - log error */
                 slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name,
@@ -1118,7 +1105,6 @@ write_changelog_and_ruv(Slapi_PBlock *pb)
     */
     if (0 == return_value) {
         char csn_str[CSN_STRSIZE] = {'\0'};
-        int rc;
         const char *dn = op_params ? REPL_GET_DN(&op_params->target_address) : "unknown";
         Slapi_DN *sdn = op_params ? (&op_params->target_address)->sdn : NULL;
         char *uniqueid = op_params ? op_params->target_address.uniqueid : "unknown";
@@ -1160,9 +1146,6 @@ common_return:
             set_thread_primary_csn(NULL, NULL);
         }
     }
-    if (repl_obj) {
-        object_release(repl_obj);
-    }
     return return_value;
 }
 
@@ -1271,10 +1254,7 @@ process_postop(Slapi_PBlock *pb)
                 connext = consumer_connection_extension_acquire_exclusive_access(conn, connid, opid);
                 if (connext && connext->replica_acquired) {
                     int zero = 0;
-                    Replica *r = (Replica *)object_get_data((Object *)connext->replica_acquired);
-
-                    replica_relinquish_exclusive_access(r, connid, opid);
-                    object_release((Object *)connext->replica_acquired);
+                    replica_relinquish_exclusive_access(connext->replica_acquired, connid, opid);
                     connext->replica_acquired = NULL;
                     connext->isreplicationsession = 0;
                     slapi_pblock_set(pb, SLAPI_CONN_IS_REPLICATION_SESSION, &zero);
@@ -1309,30 +1289,27 @@ process_postop(Slapi_PBlock *pb)
 static int
 cancel_opcsn(Slapi_PBlock *pb)
 {
-    Object *repl_obj;
+    Replica *replica = NULL;
     Slapi_Operation *op = NULL;
 
     if (NULL == pb) {
         return SLAPI_PLUGIN_SUCCESS;
     }
-    repl_obj = replica_get_replica_for_op(pb);
+    replica = replica_get_replica_for_op(pb);
     slapi_pblock_get(pb, SLAPI_OPERATION, &op);
     if (NULL == op) {
         return SLAPI_PLUGIN_SUCCESS;
     }
-    if (repl_obj) {
-        Replica *r;
+    if (replica) {
         Object *gen_obj;
         CSNGen *gen;
         CSN *opcsn;
 
-        r = (Replica *)object_get_data(repl_obj);
-        PR_ASSERT(r);
         opcsn = operation_get_csn(op);
 
         if (!operation_is_flag_set(op, OP_FLAG_REPLICATED)) {
             /* get csn generator for the replica */
-            gen_obj = replica_get_csngen(r);
+            gen_obj = replica_get_csngen(replica);
             PR_ASSERT(gen_obj);
             gen = (CSNGen *)object_get_data(gen_obj);
 
@@ -1344,13 +1321,11 @@ cancel_opcsn(Slapi_PBlock *pb)
         } else if (!operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) {
             Object *ruv_obj;
 
-            ruv_obj = replica_get_ruv(r);
+            ruv_obj = replica_get_ruv(replica);
             PR_ASSERT(ruv_obj);
-            ruv_cancel_csn_inprogress(r, (RUV *)object_get_data(ruv_obj), opcsn, replica_get_rid(r));
+            ruv_cancel_csn_inprogress(replica, (RUV *)object_get_data(ruv_obj), opcsn, replica_get_rid(replica));
             object_release(ruv_obj);
         }
-
-        object_release(repl_obj);
     }
 
     return SLAPI_PLUGIN_SUCCESS;
@@ -1380,14 +1355,13 @@ ruv_tombstone_op(Slapi_PBlock *pb)
 static PRBool
 process_operation(Slapi_PBlock *pb, const CSN *csn)
 {
-    Object *r_obj;
     Replica *r;
     Object *ruv_obj;
     RUV *ruv;
     int rc;
 
-    r_obj = replica_get_replica_for_op(pb);
-    if (r_obj == NULL) {
+    r = replica_get_replica_for_op(pb);
+    if (r == NULL) {
         char sessionid[REPL_SESSION_ID_SIZE];
         get_repl_session_id(pb, sessionid, NULL);
         slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "process_operation - "
@@ -1396,9 +1370,6 @@ process_operation(Slapi_PBlock *pb, const CSN *csn)
         return PR_FALSE;
     }
 
-    r = (Replica *)object_get_data(r_obj);
-    PR_ASSERT(r);
-
     ruv_obj = replica_get_ruv(r);
     PR_ASSERT(ruv_obj);
 
@@ -1408,7 +1379,6 @@ process_operation(Slapi_PBlock *pb, const CSN *csn)
     rc = ruv_add_csn_inprogress(r, ruv, csn);
 
     object_release(ruv_obj);
-    object_release(r_obj);
 
     return (rc == RUV_SUCCESS);
 }
@@ -1416,13 +1386,12 @@ process_operation(Slapi_PBlock *pb, const CSN *csn)
 static PRBool
 is_mmr_replica(Slapi_PBlock *pb)
 {
-    Object *r_obj;
+    Replica *replica;
 
-    r_obj = replica_get_replica_for_op(pb);
-    if (r_obj == NULL) {
+    replica = replica_get_replica_for_op(pb);
+    if (replica == NULL) {
         return PR_FALSE;
     }
-    object_release(r_obj);
 
     return PR_TRUE;
 }
@@ -1436,7 +1405,7 @@ replica_get_purl_for_op(const Replica *r __attribute__((unused)), Slapi_PBlock *
     slapi_pblock_get(pb, SLAPI_IS_MMR_REPLICATED_OPERATION, &is_replicated_op);
 
     if (!is_replicated_op) {
-        purl = multimaster_get_local_purl();
+        purl = multisupplier_get_local_purl();
     } else {
         /* Get the appropriate partial URL from the supplier RUV */
         Slapi_Connection *conn;
@@ -1464,40 +1433,36 @@ replica_get_purl_for_op(const Replica *r __attribute__((unused)), Slapi_PBlock *
 
 /* this function is called when state of a backend changes */
 void
-multimaster_be_state_change(void *handle __attribute__((unused)), char *be_name, int old_be_state, int new_be_state)
+multisupplier_be_state_change(void *handle __attribute__((unused)), char *be_name, int old_be_state, int new_be_state)
 {
-    Object *r_obj;
     Replica *r;
 
     /* check if we have replica associated with the backend */
-    r_obj = replica_get_for_backend(be_name);
-    if (r_obj == NULL)
+    r = replica_get_for_backend(be_name);
+    if (r == NULL) {
         return;
-
-    r = (Replica *)object_get_data(r_obj);
-    PR_ASSERT(r);
+    }
 
     if (new_be_state == SLAPI_BE_STATE_ON) {
         /* backend came back online - restart replication */
-        slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multimaster_be_state_change - "
+        slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multisupplier_be_state_change - "
                                                           "Replica %s is coming online; enabling replication\n",
                       slapi_sdn_get_ndn(replica_get_root(r)));
         replica_enable_replication(r);
     } else if (new_be_state == SLAPI_BE_STATE_OFFLINE) {
         /* backend is about to be taken down - disable replication */
-        slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multimaster_be_state_change - "
+        slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multisupplier_be_state_change - "
                                                           "Replica %s is going offline; disabling replication\n",
                       slapi_sdn_get_ndn(replica_get_root(r)));
-        replica_disable_replication(r, r_obj);
+        replica_disable_replication(r);
     } else if (new_be_state == SLAPI_BE_STATE_DELETE) {
         /* backend is about to be removed - disable replication */
         if (old_be_state == SLAPI_BE_STATE_ON) {
-            slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multimaster_be_state_change - "
+            slapi_log_err(SLAPI_LOG_NOTICE, repl_plugin_name, "multisupplier_be_state_change - "
                                                               "Replica %s is about to be deleted; disabling replication\n",
                           slapi_sdn_get_ndn(replica_get_root(r)));
-            replica_disable_replication(r, r_obj);
+            replica_disable_replication(r);
         }
     }
 
-    object_release(r_obj);
 }
