@@ -1,6 +1,6 @@
 /** BEGIN COPYRIGHT BLOCK
  * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
- * Copyright (C) 2005 Red Hat, Inc.
+ * Copyright (C) 2020 Red Hat, Inc.
  * All rights reserved.
  *
  * License: GPL (version 3 or any later version).
@@ -15,7 +15,7 @@
 /* repl5_tot_protocol.c */
 /*
 
- The tot_protocol object implements the DS 5.0 multi-master total update
+ The tot_protocol object implements the DS 5.0 multi-supplier total update
  replication protocol, used to (re)populate a replica.
 
 */
@@ -45,7 +45,7 @@ typedef struct callback_data
     unsigned long num_entries;
     time_t sleep_on_busy;
     time_t last_busy;
-    PRLock *lock;                            /* Lock to protect access to this structure, the message id list and to force memory barriers */
+    pthread_mutex_t lock;                    /* Lock to protect access to this structure, the message id list and to force memory barriers */
     PRThread *result_tid;                    /* The async result thread */
     operation_id_list_item *message_id_list; /* List of IDs for outstanding operations */
     int abort;                               /* Flag used to tell the sending thread asyncronously that it should abort (because an error came up in a result) */
@@ -113,7 +113,7 @@ repl5_tot_result_threadmain(void *param)
     while (!finished) {
         int message_id = 0;
         time_t time_now = 0;
-        time_t start_time = slapi_current_utc_time();
+        time_t start_time = slapi_current_rel_time_t();
         int backoff_time = 1;
 
         /* Read the next result */
@@ -130,7 +130,7 @@ repl5_tot_result_threadmain(void *param)
                 /* We need to a) check that the 'real' timeout hasn't expired and
                  * b) implement a backoff sleep to avoid spinning */
                 /* Did the connection's timeout expire ? */
-                time_now = slapi_current_utc_time();
+                time_now = slapi_current_rel_time_t();
                 if (conn_get_timeout(conn) <= (time_now - start_time)) {
                     /* We timed out */
                     conres = CONN_TIMEOUT;
@@ -142,11 +142,11 @@ repl5_tot_result_threadmain(void *param)
                     backoff_time <<= 1;
                 }
                 /* Should we stop ? */
-                PR_Lock(cb->lock);
+                pthread_mutex_lock(&(cb->lock));
                 if (cb->stop_result_thread) {
                     finished = 1;
                 }
-                PR_Unlock(cb->lock);
+                pthread_mutex_unlock(&(cb->lock));
             } else {
                 /* Something other than a timeout, so we exit the loop */
                 break;
@@ -164,21 +164,21 @@ repl5_tot_result_threadmain(void *param)
         /* Was the result itself an error ? */
         if (0 != conres) {
             /* If so then we need to take steps to abort the update process */
-            PR_Lock(cb->lock);
+            pthread_mutex_lock(&(cb->lock));
             cb->abort = 1;
             if (conres == CONN_NOT_CONNECTED) {
                 cb->rc = LDAP_CONNECT_ERROR;
             }
-            PR_Unlock(cb->lock);
+            pthread_mutex_unlock(&(cb->lock));
         }
         /* Should we stop ? */
-        PR_Lock(cb->lock);
+        pthread_mutex_lock(&(cb->lock));
         /* if the connection is not connected, then we cannot read any more
            results - we are finished */
         if (cb->stop_result_thread || (conres == CONN_NOT_CONNECTED)) {
             finished = 1;
         }
-        PR_Unlock(cb->lock);
+        pthread_mutex_unlock(&(cb->lock));
     }
 }
 
@@ -209,9 +209,9 @@ repl5_tot_destroy_async_result_thread(callback_data *cb_data)
     int retval = 0;
     PRThread *tid = cb_data->result_tid;
     if (tid) {
-        PR_Lock(cb_data->lock);
+        pthread_mutex_lock(&(cb_data->lock));
         cb_data->stop_result_thread = 1;
-        PR_Unlock(cb_data->lock);
+        pthread_mutex_unlock(&(cb_data->lock));
         (void)PR_JoinThread(tid);
     }
     return retval;
@@ -248,7 +248,7 @@ repl5_tot_waitfor_async_results(callback_data *cb_data)
     /* Keep pulling results off the LDAP connection until we catch up to the last message id stored in the rd */
     while (!done) {
         /* Lock the structure to force memory barrier */
-        PR_Lock(cb_data->lock);
+        pthread_mutex_lock(&(cb_data->lock));
         /* Are we caught up ? */
         slapi_log_err(SLAPI_LOG_REPL, repl_plugin_name,
                       "repl5_tot_waitfor_async_results - %d %d\n",
@@ -260,7 +260,7 @@ repl5_tot_waitfor_async_results(callback_data *cb_data)
         if (cb_data->abort && LOST_CONN_ERR(cb_data->rc)) {
             done = 1; /* no connection == no more results */
         }
-        PR_Unlock(cb_data->lock);
+        pthread_mutex_unlock(&(cb_data->lock));
         /* If not then sleep a bit */
         DS_Sleep(PR_SecondsToInterval(1));
         loops++;
@@ -307,16 +307,14 @@ check_suffix_entryID(Slapi_Backend *be, Slapi_Entry *suffix)
     }
 
     /* Second retrieve the suffix entryid from the suffix entry itself */
-    entryid_str = slapi_entry_attr_get_charptr(suffix, "entryid");
+    entryid_str = (char*)slapi_entry_attr_get_ref(suffix, "entryid");
     if (entryid_str == NULL) {
         char *dn;
         dn = slapi_entry_get_ndn(suffix);
         slapi_log_err(SLAPI_LOG_ERR, "check_suffix_entryID", "Unable to retrieve entryid of the suffix entry %s\n", dn ? dn : "<unknown>");
-        slapi_ch_free_string(&entryid_str);
         return;
     }
     entryid = (u_int32_t) atoi(entryid_str);
-    slapi_ch_free_string(&entryid_str);
 
     if (!bck_info.key_found || bck_info.id != entryid) {
         /* The suffix entryid is not present in parentid index
@@ -353,9 +351,7 @@ repl5_tot_run(Private_Repl_Protocol *prp)
     Slapi_DN *area_sdn = NULL;
     CSN *remote_schema_csn = NULL;
     int init_retry = 0;
-    Replica *replica;
     ReplicaId rid = 0; /* Used to create the replica keep alive subentry */
-    Slapi_Entry *suffix = NULL;
     char **instances = NULL;
     Slapi_Backend *be = NULL;
     int is_entryrdn = 0;
@@ -445,7 +441,6 @@ retry:
 
     pb = slapi_pblock_new();
 
-    replica = (Replica *)object_get_data(prp->replica_object);
     /*
      * Get the info about the entryrdn vs. entrydn from the backend.
      * If NOT is_entryrdn, its ancestor entries are always found prior to an entry.
@@ -458,6 +453,8 @@ retry:
         goto done;
     }
     be = slapi_be_select_by_instance_name(instances[0]);
+    slapi_ch_array_free(instances);
+
     if (!be) {
         slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "repl5_tot_run - Unable to "
                                                        "get the instance for the suffix \"%s\".\n",
@@ -471,7 +468,9 @@ retry:
          * Entires are retireved sorted by parentid without the allid threshold.
          */
         /* Get suffix */
-        rc = slapi_search_internal_get_entry(area_sdn, NULL, &suffix, repl_get_plugin_identity(PLUGIN_MULTIMASTER_REPLICATION));
+        Slapi_Entry *suffix = NULL;
+        Slapi_PBlock *suffix_pb = NULL;
+        rc = slapi_search_get_entry(&suffix_pb, area_sdn, NULL, &suffix, repl_get_plugin_identity(PLUGIN_MULTISUPPLIER_REPLICATION));
         if (rc) {
             slapi_log_err(SLAPI_LOG_ERR, repl_plugin_name, "repl5_tot_run -  Unable to "
                                                            "get the suffix entry \"%s\".\n",
@@ -483,9 +482,9 @@ retry:
         cb_data.rc = 0;
         cb_data.num_entries = 1UL;
         cb_data.sleep_on_busy = 0UL;
-        cb_data.last_busy = slapi_current_utc_time();
+        cb_data.last_busy = slapi_current_rel_time_t();
         cb_data.flowcontrol_detection = 0;
-        cb_data.lock = PR_NewLock();
+        pthread_mutex_init(&(cb_data.lock), NULL);
 
         /* This allows during perform_operation to check the callback data
          * especially to do flow contol on delta send msgid / recv msgid
@@ -508,17 +507,18 @@ retry:
         ctrls[1] = create_backend_control(area_sdn);
 
         /* Time to make sure it exists a keep alive subentry for that replica */
-        if (replica) {
-            rid = replica_get_rid(replica);
+        if (prp->replica) {
+            rid = replica_get_rid(prp->replica);
         }
-        replica_subentry_check(area_sdn, rid);
+        replica_subentry_check(slapi_sdn_get_dn(area_sdn), rid);
 
         /* Send the subtree of the suffix in the order of parentid index plus ldapsubentry and nstombstone. */
         check_suffix_entryID(be, suffix);
         slapi_search_internal_set_pb(pb, slapi_sdn_get_dn(area_sdn),
                                      LDAP_SCOPE_SUBTREE, "(parentid>=1)", NULL, 0, ctrls, NULL,
-                                     repl_get_plugin_identity(PLUGIN_MULTIMASTER_REPLICATION), OP_FLAG_BULK_IMPORT);
+                                     repl_get_plugin_identity(PLUGIN_MULTISUPPLIER_REPLICATION), OP_FLAG_BULK_IMPORT);
         cb_data.num_entries = 0UL;
+        slapi_search_get_entry_done(&suffix_pb);
     } else {
         /* Original total update */
         /* we need to provide managedsait control so that referral entries can
@@ -528,23 +528,22 @@ retry:
         ctrls[1] = create_backend_control(area_sdn);
 
         /* Time to make sure it exists a keep alive subentry for that replica */
-        replica = (Replica *)object_get_data(prp->replica_object);
-        if (replica) {
-            rid = replica_get_rid(replica);
+        if (prp->replica) {
+            rid = replica_get_rid(prp->replica);
         }
-        replica_subentry_check(area_sdn, rid);
+        replica_subentry_check(slapi_sdn_get_dn(area_sdn), rid);
 
         slapi_search_internal_set_pb(pb, slapi_sdn_get_dn(area_sdn),
                                      LDAP_SCOPE_SUBTREE, "(|(objectclass=ldapsubentry)(objectclass=nstombstone)(nsuniqueid=*))", NULL, 0, ctrls, NULL,
-                                     repl_get_plugin_identity(PLUGIN_MULTIMASTER_REPLICATION), 0);
+                                     repl_get_plugin_identity(PLUGIN_MULTISUPPLIER_REPLICATION), 0);
 
         cb_data.prp = prp;
         cb_data.rc = 0;
         cb_data.num_entries = 0UL;
         cb_data.sleep_on_busy = 0UL;
-        cb_data.last_busy = slapi_current_utc_time();
+        cb_data.last_busy = slapi_current_rel_time_t();
         cb_data.flowcontrol_detection = 0;
-        cb_data.lock = PR_NewLock();
+        pthread_mutex_init(&(cb_data.lock), NULL);
 
         /* This allows during perform_operation to check the callback data
          * especially to do flow contol on delta send msgid / recv msgid
@@ -634,9 +633,7 @@ done:
                       type_nsds5ReplicaFlowControlWindow);
     }
     conn_set_tot_update_cb(prp->conn, NULL);
-    if (cb_data.lock) {
-        PR_DestroyLock(cb_data.lock);
-    }
+    pthread_mutex_destroy(&(cb_data.lock));
     prp->stopped = 1;
 }
 
@@ -646,13 +643,11 @@ repl5_tot_stop(Private_Repl_Protocol *prp)
     int return_value;
     PRIntervalTime start, maxwait, now;
     PRUint64 timeout = DEFAULT_PROTOCOL_TIMEOUT;
-    Replica *replica = NULL;
 
     if ((timeout = agmt_get_protocol_timeout(prp->agmt)) == 0) {
         timeout = DEFAULT_PROTOCOL_TIMEOUT;
-        if (prp->replica_object) {
-            replica = object_get_data(prp->replica_object);
-            if ((timeout = replica_get_protocol_timeout(replica)) == 0) {
+        if (prp->replica) {
+            if ((timeout = replica_get_protocol_timeout(prp->replica)) == 0) {
                 timeout = DEFAULT_PROTOCOL_TIMEOUT;
             }
         }
@@ -703,7 +698,9 @@ Private_Repl_Protocol *
 Repl_5_Tot_Protocol_new(Repl_Protocol *rp)
 {
     repl5_tot_private *rip = NULL;
-    Private_Repl_Protocol *prp = (Private_Repl_Protocol *)slapi_ch_malloc(sizeof(Private_Repl_Protocol));
+    pthread_condattr_t cattr;
+    Private_Repl_Protocol *prp = (Private_Repl_Protocol *)slapi_ch_calloc(1, sizeof(Private_Repl_Protocol));
+
     prp->delete = repl5_tot_delete;
     prp->run = repl5_tot_run;
     prp->stop = repl5_tot_stop;
@@ -713,12 +710,19 @@ Repl_5_Tot_Protocol_new(Repl_Protocol *rp)
     prp->notify_window_opened = repl5_tot_noop;
     prp->notify_window_closed = repl5_tot_noop;
     prp->update_now = repl5_tot_noop;
-    if ((prp->lock = PR_NewLock()) == NULL) {
+    if (pthread_mutex_init(&(prp->lock), NULL) != 0) {
         goto loser;
     }
-    if ((prp->cvar = PR_NewCondVar(prp->lock)) == NULL) {
+    if (pthread_condattr_init(&cattr) != 0) {
         goto loser;
     }
+    if (pthread_condattr_setclock(&cattr, CLOCK_MONOTONIC) != 0) {
+        goto loser;
+    }
+    if (pthread_cond_init(&(prp->cvar), &cattr) != 0) {
+        goto loser;
+    }
+    pthread_condattr_destroy(&cattr);
     prp->stopped = 1;
     prp->terminate = 0;
     prp->eventbits = 0;
@@ -731,7 +735,7 @@ Repl_5_Tot_Protocol_new(Repl_Protocol *rp)
     prp->repl50consumer = 0;
     prp->repl71consumer = 0;
     prp->repl90consumer = 0;
-    prp->replica_object = prot_get_replica_object(rp);
+    prp->replica = prot_get_replica(rp);
     return prp;
 loser:
     repl5_tot_delete(&prp);
@@ -746,15 +750,9 @@ repl5_tot_delete(Private_Repl_Protocol **prpp)
         (*prpp)->stopped = 1;
         (*prpp)->stop(*prpp);
     }
-    /* Then, delete all resources used by the protocol */
-    if ((*prpp)->lock) {
-        PR_DestroyLock((*prpp)->lock);
-        (*prpp)->lock = NULL;
-    }
-    if ((*prpp)->cvar) {
-        PR_DestroyCondVar((*prpp)->cvar);
-        (*prpp)->cvar = NULL;
-    }
+    /* Then, release all resources used by the protocol */
+    pthread_mutex_destroy(&((*prpp)->lock));
+    pthread_cond_destroy(&(*prpp)->cvar);
     slapi_ch_free((void **)&(*prpp)->private);
     slapi_ch_free((void **)prpp);
 }
@@ -827,9 +825,9 @@ send_entry(Slapi_Entry *e, void *cb_data)
 
     /* see if the result reader thread encountered
        a fatal error */
-    PR_Lock(((callback_data *)cb_data)->lock);
+    pthread_mutex_lock((&((callback_data *)cb_data)->lock));
     rc = ((callback_data *)cb_data)->abort;
-    PR_Unlock(((callback_data *)cb_data)->lock);
+    pthread_mutex_unlock((&((callback_data *)cb_data)->lock));
     if (rc) {
         conn_disconnect(prp->conn);
         ((callback_data *)cb_data)->rc = -1;
@@ -892,7 +890,7 @@ send_entry(Slapi_Entry *e, void *cb_data)
         }
 
         if (rc == CONN_BUSY) {
-            time_t now = slapi_current_utc_time();
+            time_t now = slapi_current_rel_time_t();
             if ((now - *last_busyp) < (*sleep_on_busyp + 10)) {
                 *sleep_on_busyp += 5;
             } else {
