@@ -63,7 +63,7 @@ ldbm_back_add(Slapi_PBlock *pb)
     back_txn txn = {0};
     back_txnid parent_txn;
     int retval = -1;
-    char *msg;
+    const char *msg;
     int managedsait;
     int ldap_result_code = LDAP_SUCCESS;
     char *ldap_result_message = NULL;
@@ -97,6 +97,8 @@ ldbm_back_add(Slapi_PBlock *pb)
     PRUint64 conn_id;
     int op_id;
     int result_sent = 0;
+    int32_t parent_op = 0;
+    struct timespec parent_time;
 
     if (slapi_pblock_get(pb, SLAPI_CONN_ID, &conn_id) < 0) {
         conn_id = 0; /* connection is NULL */
@@ -147,13 +149,21 @@ ldbm_back_add(Slapi_PBlock *pb)
     slapi_entry_delete_values(e, numsubordinates, NULL);
 
     dblayer_txn_init(li, &txn);
+
+    if (txn.back_txn_txn == NULL) {
+        /* This is the parent operation, get the time */
+        parent_op = 1;
+        parent_time = slapi_current_rel_time_hr();
+    }
+
     /* the calls to perform searches require the parent txn if any
        so set txn to the parent_txn until we begin the child transaction */
     if (parent_txn) {
         txn.back_txn_txn = parent_txn;
     } else {
         parent_txn = txn.back_txn_txn;
-        slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
+        if (parent_txn)
+            slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
     }
 
     /* The dblock serializes writes to the database,
@@ -570,7 +580,7 @@ ldbm_back_add(Slapi_PBlock *pb)
                         slapi_log_err(SLAPI_LOG_REPL, "ldbm_back_add", "Resurrection of %s - Added Conflict reason %s\n",
                                       dn, reason);
                     }
-                    slapi_ch_free((void **)&reason);
+                    slapi_ch_free_string(&reason);
                 }
                 /* Clear the Tombstone Flag in the entry */
                 slapi_entry_clear_flag(addingentry->ep_entry, SLAPI_ENTRY_FLAG_TOMBSTONE);
@@ -635,7 +645,13 @@ ldbm_back_add(Slapi_PBlock *pb)
                          * Current op is a user request. Opcsn will be assigned
                          * if the dn is in an updatable replica.
                          */
-                        opcsn = entry_assign_operation_csn(pb, e, parententry ? parententry->ep_entry : NULL);
+                        if (entry_assign_operation_csn(pb, e, parententry ? parententry->ep_entry : NULL, &opcsn) != 0) {
+                            slapi_log_err(SLAPI_LOG_ERR, "ldbm_back_add",
+                                    "failed to generate add CSN for entry (%s), aborting operation\n",
+                                    slapi_entry_get_dn(e));
+                            ldap_result_code = LDAP_OPERATIONS_ERROR;
+                            goto error_return;
+                        }
                     }
                     if (opcsn != NULL) {
                         entry_set_csn(e, opcsn);
@@ -736,6 +752,7 @@ ldbm_back_add(Slapi_PBlock *pb)
                                                SLAPI_ATTR_VALUE_PARENT_UNIQUEID,
                                                operation->o_params.p.p_add.parentuniqueid);
                     }
+                    slapi_sdn_done(&nscpEntrySDN);
                 } else {
                         /* if an entry is explicitely added as tombstone the entry flag has to be set */
                         if (slapi_entry_attr_hasvalue(addingentry->ep_entry,
@@ -774,10 +791,10 @@ ldbm_back_add(Slapi_PBlock *pb)
                 pid = parententry->ep_id;
 
                 /* We may need to adjust the DN since parent could be a resurrected conflict entry... */
-                /* TBD better handle tombstone parents, 
+                /* TBD better handle tombstone parents,
                  * we have the entry dn as nsuniqueid=nnnn,<rdn>,parentdn
                  * so is parent will return false
-                 */ 
+                 */
                  if (!is_tombstone_operation && !slapi_sdn_isparent(slapi_entry_get_sdn_const(parententry->ep_entry),
                                                                     slapi_entry_get_sdn_const(addingentry->ep_entry))) {
                     Slapi_DN adjustedsdn = {0};
@@ -914,7 +931,7 @@ ldbm_back_add(Slapi_PBlock *pb)
         }
 
         retval = id2entry_add_ext(be, addingentry, &txn, 1, &myrc);
-        if (DB_LOCK_DEADLOCK == retval) {
+        if (DBI_RC_RETRY == retval) {
             slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 1 DEADLOCK\n");
             /* Retry txn */
             continue;
@@ -936,8 +953,8 @@ ldbm_back_add(Slapi_PBlock *pb)
 
             retval = index_addordel_string(be, SLAPI_ATTR_OBJECTCLASS, SLAPI_ATTR_VALUE_TOMBSTONE,
                                            addingentry->ep_id, BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
-                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 2 DB_LOCK_DEADLOCK\n");
+            if (DBI_RC_RETRY == retval) {
+                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 2 DBI_RC_RETRY\n");
                 /* Retry txn */
                 continue;
             }
@@ -958,8 +975,8 @@ ldbm_back_add(Slapi_PBlock *pb)
                 csn_as_string(tombstone_csn, PR_FALSE, deletion_csn_str);
                 retval = index_addordel_string(be, SLAPI_ATTR_TOMBSTONE_CSN, deletion_csn_str,
                                                tombstoneentry->ep_id, BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
-                    slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 3 DB_LOCK_DEADLOCK\n");
+                if (DBI_RC_RETRY == retval) {
+                    slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 3 DBI_RC_RETRY\n");
                     /* Retry txn */
                     continue;
                 }
@@ -977,8 +994,8 @@ ldbm_back_add(Slapi_PBlock *pb)
             }
 
             retval = index_addordel_string(be, SLAPI_ATTR_UNIQUEID, slapi_entry_get_uniqueid(addingentry->ep_entry), addingentry->ep_id, BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
-                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 4 DB_LOCK_DEADLOCK\n");
+            if (DBI_RC_RETRY == retval) {
+                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 4 DBI_RC_RETRY\n");
                 /* Retry txn */
                 continue;
             }
@@ -998,8 +1015,8 @@ ldbm_back_add(Slapi_PBlock *pb)
                                            slapi_sdn_get_ndn(sdn),
                                            addingentry->ep_id,
                                            BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
-                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 5 DB_LOCK_DEADLOCK\n");
+            if (DBI_RC_RETRY == retval) {
+                slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 5 DBI_RC_RETRY\n");
                 /* Retry txn */
                 continue;
             }
@@ -1033,7 +1050,7 @@ ldbm_back_add(Slapi_PBlock *pb)
         } else {
             retval = index_addordel_entry(be, addingentry, BE_INDEX_ADD, &txn);
         }
-        if (DB_LOCK_DEADLOCK == retval) {
+        if (DBI_RC_RETRY == retval) {
             slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 5 DEADLOCK\n");
             /* retry txn */
             continue;
@@ -1055,7 +1072,7 @@ ldbm_back_add(Slapi_PBlock *pb)
             slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_add",
                           "conn=%" PRIu64 " op=%d modify_update_all: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                           conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add", "add 6 DEADLOCK\n");
                 /* Retry txn */
                 continue;
@@ -1077,7 +1094,7 @@ ldbm_back_add(Slapi_PBlock *pb)
          */
         if (!is_ruv) {
             retval = vlv_update_all_indexes(&txn, be, pb, NULL, addingentry);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_ARGS, "ldbm_back_add",
                               "add DEADLOCK vlv_update_index\n");
                 /* Retry txn */
@@ -1110,7 +1127,7 @@ ldbm_back_add(Slapi_PBlock *pb)
 
         if (ruv_c_init) {
             retval = modify_update_all(be, pb, &ruv_c, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 /* Abort and re-try */
                 continue;
             }
@@ -1239,10 +1256,14 @@ ldbm_back_add(Slapi_PBlock *pb)
     goto common_return;
 
 error_return:
+    /* Revert the caches if this is the parent operation */
+    if (parent_op) {
+        revert_cache(inst, &parent_time);
+    }
     if (addingentry_id_assigned) {
         next_id_return(be, addingentry->ep_id);
     }
-    if (rc == DB_RUNRECOVERY) {
+    if (rc == DBI_RC_RUNRECOVERY) {
         dblayer_remember_disk_filled(li);
         ldbm_nasty("ldbm_back_add", "Add", 80, rc);
         disk_full = 1;
