@@ -37,7 +37,7 @@ ldbm_back_delete(Slapi_PBlock *pb)
     back_txn txn;
     back_txnid parent_txn;
     int retval = -1;
-    char *msg;
+    const char *msg;
     char *errbuf = NULL;
     int retry_count = 0;
     int disk_full = 0;
@@ -57,7 +57,7 @@ ldbm_back_delete(Slapi_PBlock *pb)
     int is_ruv = 0; /* True if the current entry is RUV */
     int is_replicated_operation = 0;
     int is_tombstone_entry = 0;     /* True if the current entry is alreday a tombstone        */
-    int delete_tombstone_entry = 0; /* We must remove the given tombstone entry from the DB    */
+    int delete_tombstone_entry = 0; /* We must remove the given tombstone entry from the dbi_db_t    */
     int create_tombstone_entry = 0; /* We perform a "regular" LDAP delete but since we use    */
                                     /* replication, we must create a new tombstone entry    */
     int remove_e_from_cache = 0;
@@ -79,6 +79,8 @@ ldbm_back_delete(Slapi_PBlock *pb)
     ID tomb_ep_id = 0;
     int result_sent = 0;
     Connection *pb_conn;
+    int32_t parent_op = 0;
+    struct timespec parent_time;
 
     if (slapi_pblock_get(pb, SLAPI_CONN_ID, &conn_id) < 0) {
         conn_id = 0; /* connection is NULL */
@@ -96,15 +98,36 @@ ldbm_back_delete(Slapi_PBlock *pb)
     slapi_sdn_init(&nscpEntrySDN);
     slapi_sdn_init(&parentsdn);
 
+    inst = (ldbm_instance *)be->be_instance_info;
+    if (inst && inst->inst_ref_count) {
+        slapi_counter_increment(inst->inst_ref_count);
+    } else {
+        slapi_log_err(SLAPI_LOG_ERR,
+                      "ldbm_back_delete", "Instance \"%s\" does not exist.\n",
+                      inst ? inst->inst_name : "null instance");
+        ldap_result_code = LDAP_UNWILLING_TO_PERFORM;
+        ldap_result_message = "Backend instance is not available.";
+        /* error_return code dereferences "inst" but diskfull_return one does not. */
+        goto diskfull_return;
+    }
+
     /* dblayer_txn_init needs to be called before "goto error_return" */
     dblayer_txn_init(li, &txn);
     /* the calls to perform searches require the parent txn if any
        so set txn to the parent_txn until we begin the child transaction */
+
+    if (txn.back_txn_txn == NULL) {
+        /* This is the parent operation, get the time */
+        parent_op = 1;
+        parent_time = slapi_current_rel_time_hr();
+    }
+
     if (parent_txn) {
         txn.back_txn_txn = parent_txn;
     } else {
         parent_txn = txn.back_txn_txn;
-        slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
+        if (parent_txn)
+            slapi_pblock_set(pb, SLAPI_TXN, parent_txn);
     }
 
     if (pb_conn) {
@@ -131,16 +154,6 @@ ldbm_back_delete(Slapi_PBlock *pb)
     is_ruv = operation_is_flag_set(operation, OP_FLAG_REPL_RUV);
     delete_tombstone_entry = operation_is_flag_set(operation, OP_FLAG_TOMBSTONE_ENTRY);
 
-    inst = (ldbm_instance *)be->be_instance_info;
-    if (inst && inst->inst_ref_count) {
-        slapi_counter_increment(inst->inst_ref_count);
-    } else {
-        slapi_log_err(SLAPI_LOG_ERR,
-                      "ldbm_back_delete", "Instance \"%s\" does not exist.\n",
-                      inst ? inst->inst_name : "null instance");
-        goto error_return;
-    }
-
     /* The dblock serializes writes to the database,
      * which reduces deadlocking in the db code,
      * which means that we run faster.
@@ -259,11 +272,11 @@ ldbm_back_delete(Slapi_PBlock *pb)
 
         if (0 == retry_count) { /* just once */
             /* find and lock the entry we are about to modify */
-            /* 
-             * A corner case: 
-             * If a conflict occurred in a MMR topology, a replicated delete 
-             * op from another master could target a conflict entry; while the
-             * corresponding entry on this server could have been already 
+            /*
+             * A corner case:
+             * If a conflict occurred in a MMR topology, a replicated delete
+             * op from another supplier could target a conflict entry; while the
+             * corresponding entry on this server could have been already
              * deleted.  That is, the entry 'e' found with "addr" is a tomb-
              * stone.  If it is the case, we need to back off.
              */
@@ -280,7 +293,7 @@ replace_entry:
             /* JCMACL - Shouldn't the access check be before the has children check...
              * otherwise we're revealing the fact that an entry exists and has children */
             /* Before has children to mask the presence of children disclosure. */
-            ldap_result_code = plugin_call_acl_plugin (pb, e->ep_entry, NULL, NULL, SLAPI_ACL_DELETE, 
+            ldap_result_code = plugin_call_acl_plugin (pb, e->ep_entry, NULL, NULL, SLAPI_ACL_DELETE,
                                                        ACLPLUGIN_ACCESS_DEFAULT, &errbuf );
             if ( ldap_result_code != LDAP_SUCCESS ) {
                 ldap_result_message= errbuf;
@@ -307,7 +320,7 @@ replace_entry:
 
             /* Don't call pre-op for Tombstone entries */
             if (!delete_tombstone_entry) {
-                /* 
+                /*
                  * Some present state information is passed through the PBlock to the
                  * backend pre-op plugin. To ensure a consistent snapshot of this state
                  * we wrap the reading of the entry with the dblock.
@@ -365,14 +378,14 @@ replace_entry:
                 }
                 if (retval)
                 {
-                    /* 
+                    /*
                      * Plugin indicated some kind of failure,
                      * or that this Operation became a No-Op.
                      */
                     slapi_pblock_get(pb, SLAPI_RESULT_CODE, &ldap_result_code);
                     if (!ldap_result_code) {
                         if (LDAP_ALREADY_EXISTS == ldap_result_code) {
-                            /* 
+                            /*
                              * The target entry is already a tombstone.
                              * We need to treat this as a success,
                              * but we need to remove the entry e from the entry cache.
@@ -394,7 +407,10 @@ replace_entry:
                 delete_tombstone_entry = operation_is_flag_set(operation, OP_FLAG_TOMBSTONE_ENTRY);
             }
 
-            /* call the transaction pre delete plugins just after the 
+            /* Save away a copy of the entry, before modifications */
+            slapi_pblock_set(pb, SLAPI_ENTRY_PRE_OP, slapi_entry_dup(e->ep_entry));
+
+            /* call the transaction pre delete plugins just after the
              * to-be-deleted entry is prepared. */
             /* these should not need to modify the entry to be deleted -
                if for some reason they ever do, do not use e->ep_entry since
@@ -434,7 +450,7 @@ replace_entry:
                     delete_tombstone_entry = 0;
                 }
             } else {
-                if (is_tombstone_entry) { 
+                if (is_tombstone_entry) {
                     slapi_log_err(SLAPI_LOG_WARNING, "ldbm_back_delete",
                             "Attempt to Tombstone again a tombstone entry %s\n", dn);
                     delete_tombstone_entry = 1;
@@ -454,7 +470,14 @@ replace_entry:
                      * by entry_assign_operation_csn() if the dn is in an
                      * updatable replica.
                      */
-                    opcsn = entry_assign_operation_csn ( pb, e->ep_entry, NULL );
+                    if (entry_assign_operation_csn(pb, e->ep_entry, NULL, &opcsn) != 0) {
+                        slapi_log_err(SLAPI_LOG_ERR, "ldbm_back_delete",
+                                "failed to generate delete CSN for entry (%s), aborting operation\n",
+                                slapi_entry_get_dn(e->ep_entry));
+                        retval = -1;
+                        ldap_result_code = LDAP_OPERATIONS_ERROR;
+                        goto error_return;
+                    }
                 }
                 if (opcsn != NULL) {
                     if (!is_fixup_operation) {
@@ -477,14 +500,12 @@ replace_entry:
                 ldap_result_code = LDAP_UNWILLING_TO_PERFORM;
                 goto error_return;
             }
-        
+
 #ifdef DEBUG
             slapi_log_err(SLAPI_LOG_REPL, "ldbm_back_delete",
                     "entry: %s  - flags: delete %d is_tombstone_entry %d create %d \n",
                     dn, delete_tombstone_entry, is_tombstone_entry, create_tombstone_entry);
 #endif
-            /* Save away a copy of the entry, before modifications */
-            slapi_pblock_set(pb, SLAPI_ENTRY_PRE_OP, slapi_entry_dup(e->ep_entry));
 
             /*
              * Get the entry's parent. We do this here because index_read
@@ -494,7 +515,7 @@ replace_entry:
             slapi_sdn_get_backend_parent_ext(sdnp, &parentsdn, be, is_tombstone_entry);
             if (!slapi_sdn_isempty(&parentsdn)) {
                 struct backentry *parent = NULL;
-                char *pid_str = slapi_entry_attr_get_charptr(e->ep_entry, LDBM_PARENTID_STR);
+                char *pid_str = (char *)slapi_entry_attr_get_ref(e->ep_entry, LDBM_PARENTID_STR);
                 if (pid_str) {
                     /* First, try to get the direct parent. */
                     /*
@@ -508,7 +529,6 @@ replace_entry:
                     int cache_retry = 0;
 
                     pid = (ID)strtol(pid_str, (char **)NULL, 10);
-                    slapi_ch_free_string(&pid_str);
 
                     /*
                      * Its possible that the parent entry retrieved from the cache in id2entry
@@ -522,6 +542,7 @@ replace_entry:
                             if (cache_retry == RETRY_CACHE_LOCK &&
                                 cache_retry_count < LDBM_CACHE_RETRY_COUNT) {
                                 /* try again */
+                                CACHE_RETURN(&(inst->inst_cache), &parent);
                                 DS_Sleep(PR_MillisecondsToInterval(100));
                                 cache_retry_count++;
                                 continue;
@@ -752,8 +773,8 @@ replace_entry:
                 goto error_return;
             }
             retval = id2entry_add(be, tombstone, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
-                slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "delete 1 DB_LOCK_DEADLOCK\n");
+            if (DBI_RC_RETRY == retval) {
+                slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "delete 1 DBI_RC_RETRY\n");
                 /* Abort and re-try */
                 continue;
             }
@@ -768,13 +789,13 @@ replace_entry:
         } else {
             /* delete the entry from disk */
             retval = id2entry_delete(be, e, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "delete 2 DEADLOCK\n");
                 /* Retry txn */
                 continue;
             }
             if (retval) {
-                if (retval == DB_RUNRECOVERY || LDBM_OS_ERR_IS_DISKFULL(retval)) {
+                if (retval == DBI_RC_RUNRECOVERY || LDBM_OS_ERR_IS_DISKFULL(retval)) {
                     disk_full = 1;
                 }
                 DEL_SET_ERROR(ldap_result_code,
@@ -788,7 +809,7 @@ replace_entry:
             addordel_flags |= BE_INDEX_TOMBSTONE; /* tell index code we are deleting a tombstone */
         }
         retval = index_addordel_entry(be, e, addordel_flags, &txn);
-        if (DB_LOCK_DEADLOCK == retval) {
+        if (DBI_RC_RETRY == retval) {
             slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "delete 1 DEADLOCK\n");
             /* Retry txn */
             continue;
@@ -810,9 +831,9 @@ replace_entry:
             retval = index_addordel_string(be, SLAPI_ATTR_OBJECTCLASS,
                                            SLAPI_ATTR_VALUE_TOMBSTONE,
                                            tombstone->ep_id, BE_INDEX_ADD, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM,
-                              "ldbm_back_delete", "(adding %s) DB_LOCK_DEADLOCK\n",
+                              "ldbm_back_delete", "(adding %s) DBI_RC_RETRY\n",
                               SLAPI_ATTR_VALUE_TOMBSTONE);
                 /* Retry txn */
                 continue;
@@ -835,9 +856,9 @@ replace_entry:
                 retval = index_addordel_string(be, SLAPI_ATTR_TOMBSTONE_CSN,
                                                deletion_csn_str, tombstone->ep_id,
                                                BE_INDEX_ADD, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "Delete tombstone csn(adding %s) DB_LOCK_DEADLOCK\n",
+                                  "ldbm_back_delete", "Delete tombstone csn(adding %s) DBI_RC_RETRY\n",
                                   deletion_csn_str);
                     /* Retry txn */
                     continue;
@@ -859,9 +880,9 @@ replace_entry:
             retval = index_addordel_string(be, SLAPI_ATTR_UNIQUEID,
                                            slapi_entry_get_uniqueid(tombstone->ep_entry),
                                            tombstone->ep_id, BE_INDEX_ADD, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM,
-                              "ldbm_back_delete", "(adding %s) DB_LOCK_DEADLOCK\n",
+                              "ldbm_back_delete", "(adding %s) DBI_RC_RETRY\n",
                               SLAPI_ATTR_UNIQUEID);
                 /* Retry txn */
                 continue;
@@ -879,9 +900,9 @@ replace_entry:
             retval = index_addordel_string(be, SLAPI_ATTR_NSCP_ENTRYDN,
                                            slapi_sdn_get_ndn(&nscpEntrySDN),
                                            tombstone->ep_id, BE_INDEX_ADD, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM,
-                              "ldbm_back_delete", "(adding %s) DB_LOCK_DEADLOCK\n",
+                              "ldbm_back_delete", "(adding %s) DBI_RC_RETRY\n",
                               SLAPI_ATTR_NSCP_ENTRYDN);
                 /* Retry txn */
                 continue;
@@ -897,15 +918,13 @@ replace_entry:
                 goto error_return;
             }
             /* add a new usn to the entryusn index */
-            entryusn_str = slapi_entry_attr_get_charptr(tombstone->ep_entry,
-                                                        SLAPI_ATTR_ENTRYUSN);
+            entryusn_str = (char *)slapi_entry_attr_get_ref(tombstone->ep_entry, SLAPI_ATTR_ENTRYUSN);
             if (entryusn_str) {
                 retval = index_addordel_string(be, SLAPI_ATTR_ENTRYUSN,
                                                entryusn_str, tombstone->ep_id, BE_INDEX_ADD, &txn);
-                slapi_ch_free_string(&entryusn_str);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "(adding %s) DB_LOCK_DEADLOCK\n",
+                                  "ldbm_back_delete", "(adding %s) DBI_RC_RETRY\n",
                                   SLAPI_ATTR_ENTRYUSN);
                     /* Retry txn */
                     continue;
@@ -935,9 +954,9 @@ replace_entry:
                     retval = index_addordel_values_sv(be, LDBM_PARENTID_STR,
                                                       svals, NULL, e->ep_id,
                                                       BE_INDEX_ADD, &txn);
-                    if (DB_LOCK_DEADLOCK == retval) {
+                    if (DBI_RC_RETRY == retval) {
                         slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                      "ldbm_back_delete", "delete (updating " LDBM_PARENTID_STR ") DB_LOCK_DEADLOCK\n");
+                                      "ldbm_back_delete", "delete (updating " LDBM_PARENTID_STR ") DBI_RC_RETRY\n");
                         /* Retry txn */
                         continue;
                     }
@@ -953,29 +972,11 @@ replace_entry:
                         goto error_return;
                     }
                 }
-#if 0 /* The entryrdn element is already deleted in the index_addordel_entry */
-                retval = entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
-                    slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                "ldbm_back_delete", "(deleting entryrdn) DB_LOCK_DEADLOCK\n");
-                    /* Retry txn */
-                    continue;
-                }
-                if (0 != retval) {
-                    slapi_log_err(SLAPI_LOG_ERR,
-                                "ldbm_back_delete", "(deleting entryrdn) failed, err=%d %s\n",
-                                retval,
-                                (msg = dblayer_strerror( retval )) ? msg : "" );
-                    if (LDBM_OS_ERR_IS_DISKFULL(retval)) disk_full = 1;
-                    DEL_SET_ERROR(ldap_result_code,
-                                  LDAP_OPERATIONS_ERROR, retry_count);
-                    goto error_return;
-                }
-#endif
+
                 retval = entryrdn_index_entry(be, tombstone, BE_INDEX_ADD, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "(adding tombstone entryrdn) DB_LOCK_DEADLOCK\n");
+                                  "ldbm_back_delete", "(adding tombstone entryrdn) DBI_RC_RETRY\n");
                     /* Retry txn */
                     continue;
                 }
@@ -1001,9 +1002,9 @@ replace_entry:
             retval = index_addordel_string(be, SLAPI_ATTR_OBJECTCLASS,
                                            SLAPI_ATTR_VALUE_TOMBSTONE, e->ep_id,
                                            BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM,
-                              "ldbm_back_delete", "(deleting %s) 0 DB_LOCK_DEADLOCK\n",
+                              "ldbm_back_delete", "(deleting %s) 0 DBI_RC_RETRY\n",
                               SLAPI_ATTR_VALUE_TOMBSTONE);
                 /* Retry txn */
                 continue;
@@ -1026,9 +1027,9 @@ replace_entry:
                 retval = index_addordel_string(be, SLAPI_ATTR_TOMBSTONE_CSN,
                                                deletion_csn_str, e->ep_id,
                                                BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "delete tombstone csn(deleting %s) DB_LOCK_DEADLOCK\n",
+                                  "ldbm_back_delete", "delete tombstone csn(deleting %s) DBI_RC_RETRY\n",
                                   deletion_csn_str);
                     /* Retry txn */
                     continue;
@@ -1050,9 +1051,9 @@ replace_entry:
             retval = index_addordel_string(be, SLAPI_ATTR_UNIQUEID,
                                            slapi_entry_get_uniqueid(e->ep_entry),
                                            e->ep_id, BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM,
-                              "ldbm_back_delete", "(deleting %s) 1 DB_LOCK_DEADLOCK\n",
+                              "ldbm_back_delete", "(deleting %s) 1 DBI_RC_RETRY\n",
                               SLAPI_ATTR_UNIQUEID);
                 /* Retry txn */
                 continue;
@@ -1068,16 +1069,13 @@ replace_entry:
                               LDAP_OPERATIONS_ERROR, retry_count);
                 goto error_return;
             }
-
-            nscpedn = slapi_entry_attr_get_charptr(e->ep_entry,
-                                                   SLAPI_ATTR_NSCP_ENTRYDN);
+            nscpedn = (char *)slapi_entry_attr_get_ref(e->ep_entry, SLAPI_ATTR_NSCP_ENTRYDN);
             if (nscpedn) {
                 retval = index_addordel_string(be, SLAPI_ATTR_NSCP_ENTRYDN,
                                                nscpedn, e->ep_id, BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-                slapi_ch_free((void **)&nscpedn);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "(deleting %s) 2 DB_LOCK_DEADLOCK\n",
+                                  "ldbm_back_delete", "(deleting %s) 2 DBI_RC_RETRY\n",
                                   SLAPI_ATTR_NSCP_ENTRYDN);
                     /* Retry txn */
                     continue;
@@ -1095,16 +1093,14 @@ replace_entry:
                 }
             }
             /* delete usn from the entryusn index */
-            entryusn_str = slapi_entry_attr_get_charptr(e->ep_entry,
-                                                        SLAPI_ATTR_ENTRYUSN);
+            entryusn_str = (char *)slapi_entry_attr_get_ref(e->ep_entry, SLAPI_ATTR_ENTRYUSN);
             if (entryusn_str) {
                 retval = index_addordel_string(be, SLAPI_ATTR_ENTRYUSN,
                                                entryusn_str, e->ep_id,
                                                BE_INDEX_DEL | BE_INDEX_EQUALITY, &txn);
-                slapi_ch_free_string(&entryusn_str);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "(deleting %s) 3 DB_LOCK_DEADLOCK\n",
+                                  "ldbm_back_delete", "(deleting %s) 3 DBI_RC_RETRY\n",
                                   SLAPI_ATTR_ENTRYUSN);
                     /* Retry txn */
                     continue;
@@ -1124,9 +1120,9 @@ replace_entry:
             if (entryrdn_get_switch()) /* subtree-rename: on */
             {
                 retval = entryrdn_index_entry(be, e, BE_INDEX_DEL, &txn);
-                if (DB_LOCK_DEADLOCK == retval) {
+                if (DBI_RC_RETRY == retval) {
                     slapi_log_err(SLAPI_LOG_BACKLDBM,
-                                  "ldbm_back_delete", "(deleting entryrdn) DB_LOCK_DEADLOCK\n");
+                                  "ldbm_back_delete", "(deleting entryrdn) DBI_RC_RETRY\n");
                     /* Retry txn */
                     continue;
                 }
@@ -1150,7 +1146,7 @@ replace_entry:
             slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
                           "conn=%" PRIu64 " op=%d modify_update_all: old_entry=0x%p, new_entry=0x%p, rc=%d\n",
                           conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry, retval);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "4 DEADLOCK\n");
                 /* Retry txn */
                 continue;
@@ -1173,7 +1169,7 @@ replace_entry:
             !vlv_delete_search_entry(pb, e->ep_entry, inst)) {
             retval = vlv_update_all_indexes(&txn, be, pb, e, NULL);
 
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete", "DEADLOCK vlv_update_all_indexes\n");
                 /* Retry txn */
                 continue;
@@ -1201,7 +1197,7 @@ replace_entry:
 
         if (ruv_c_init) {
             retval = modify_update_all(be, pb, &ruv_c, &txn);
-            if (DB_LOCK_DEADLOCK == retval) {
+            if (DBI_RC_RETRY == retval) {
                 /* Abort and re-try */
                 continue;
             }
@@ -1356,6 +1352,11 @@ commit_return:
     goto common_return;
 
 error_return:
+    /* Revert the caches if this is the parent operation */
+    if (parent_op) {
+        revert_cache(inst, &parent_time);
+    }
+
     if (tombstone) {
         if (cache_is_in_cache(&inst->inst_cache, tombstone)) {
             tomb_ep_id = tombstone->ep_id; /* Otherwise, tombstone might have been freed. */
@@ -1374,7 +1375,8 @@ error_return:
         CACHE_RETURN(&inst->inst_cache, &tombstone);
         tombstone = NULL;
     }
-    if (retval == DB_RUNRECOVERY) {
+
+    if (retval == DBI_RC_RUNRECOVERY) {
         dblayer_remember_disk_filled(li);
         ldbm_nasty("ldbm_back_delete", "Delete", 79, retval);
         disk_full = 1;
@@ -1495,7 +1497,7 @@ common_return:
          */
         e = NULL;
     }
-    if (inst && inst->inst_ref_count) {
+    if (inst->inst_ref_count) {
         slapi_counter_decrement(inst->inst_ref_count);
     }
     if (ruv_c_init) {
@@ -1516,7 +1518,7 @@ diskfull_return:
     slapi_log_err(SLAPI_LOG_BACKLDBM, "ldbm_back_delete",
                   "conn=%" PRIu64 " op=%d modify_term: old_entry=0x%p, new_entry=0x%p, in_cache=%d\n",
                   conn_id, op_id, parent_modify_c.old_entry, parent_modify_c.new_entry,
-                  cache_is_in_cache(&inst->inst_cache, parent_modify_c.new_entry));
+                  inst ? cache_is_in_cache(&inst->inst_cache, parent_modify_c.new_entry):-1);
     myrc = modify_term(&parent_modify_c, be);
     if (free_delete_existing_entry) {
         done_with_pblock_entry(pb, SLAPI_DELETE_EXISTING_ENTRY);
