@@ -1,12 +1,15 @@
 /** BEGIN COPYRIGHT BLOCK
  * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
- * Copyright (C) 2005 Red Hat, Inc.
+ * Copyright (C) 2021 Red Hat, Inc.
+ * Copyright (C) 2020 William Brown <william@blackhats.net.au>
  * All rights reserved.
  *
  * License: GPL (version 3 or any later version).
  * See LICENSE for details.
  * END COPYRIGHT BLOCK **/
 
+#pragma once
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -48,12 +51,15 @@ extern "C" {
 #define SLAPI_SHUTDOWN_EXIT     3
 
 /* filter */
-#define SLAPI_FILTER_LDAPSUBENTRY 1
-#define SLAPI_FILTER_TOMBSTONE    2
-#define SLAPI_FILTER_RUV          4
-#define SLAPI_ENTRY_LDAPSUBENTRY  2
-#define SLAPI_FILTER_NORMALIZED_TYPE   8
-#define SLAPI_FILTER_NORMALIZED_VALUE 16
+typedef enum _slapi_filter_flags_t {
+    SLAPI_FILTER_LDAPSUBENTRY = 1,
+    SLAPI_FILTER_TOMBSTONE = 2,
+    SLAPI_FILTER_RUV = 4,
+    SLAPI_FILTER_NORMALIZED_TYPE = 8,
+    SLAPI_FILTER_NORMALIZED_VALUE = 16,
+    SLAPI_FILTER_INVALID_ATTR_UNDEFINE = 32,
+    SLAPI_FILTER_INVALID_ATTR_WARN = 64,
+} slapi_filter_flags;
 
 /*
     Optimized filter path. For example the following code was lifted from int.c (syntaxes plugin):
@@ -227,7 +233,8 @@ enum
     CSN_INVALID_PARAMETER, /* invalid function argument */
     CSN_INVALID_FORMAT,    /* invalid state format */
     CSN_LDAP_ERROR,        /* LDAP operation failed */
-    CSN_NSPR_ERROR         /* NSPR API failure */
+    CSN_NSPR_ERROR,        /* NSPR API failure */
+    CSN_TIME_ERROR         /* Error generating new CSN due to clock failure */
 };
 
 typedef struct csngen CSNGen;
@@ -266,7 +273,7 @@ void *csngen_register_callbacks(CSNGen *gen, GenCSNFn genFn, void *genArg, Abort
 void csngen_unregister_callbacks(CSNGen *gen, void *cookie);
 
 /* debugging function */
-void csngen_dump_state(const CSNGen *gen);
+void csngen_dump_state(const CSNGen *gen, int severity);
 
 /* this function tests csn generator */
 void csngen_test(void);
@@ -320,7 +327,7 @@ int slapi_entries_diff(Slapi_Entry **old_entries, Slapi_Entry **new_entries, int
 void set_attr_to_protected_list(char *attr, int flag);
 
 /* entrywsi.c */
-CSN *entry_assign_operation_csn(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *parententry);
+int32_t entry_assign_operation_csn(Slapi_PBlock *pb, Slapi_Entry *e, Slapi_Entry *parententry, CSN **opcsn);
 const CSN *entry_get_maxcsn(const Slapi_Entry *entry);
 void entry_set_maxcsn(Slapi_Entry *entry, const CSN *csn);
 const CSN *entry_get_dncsn(const Slapi_Entry *entry);
@@ -377,12 +384,15 @@ int32_t ndn_cache_init(void);
 void ndn_cache_destroy(void);
 int ndn_cache_started(void);
 void ndn_cache_get_stats(uint64_t *hits, uint64_t *tries, uint64_t *size, uint64_t *max_size, uint64_t *thread_size, uint64_t *evicts, uint64_t *slots, uint64_t *count);
+void ndn_cache_inc_import_task(void);
+void ndn_cache_dec_import_task(void);
 #define NDN_DEFAULT_SIZE 20971520 /* 20mb - size of normalized dn cache */
 
 /* filter.c */
 int filter_flag_is_set(const Slapi_Filter *f, unsigned char flag);
 char *slapi_filter_to_string(const Slapi_Filter *f, char *buffer, size_t bufsize);
 char *slapi_filter_to_string_internal(const struct slapi_filter *f, char *buf, size_t *bufsize);
+void slapi_filter_optimise(Slapi_Filter *f);
 
 /* operation.c */
 
@@ -426,6 +436,11 @@ char *slapi_filter_to_string_internal(const struct slapi_filter *f, char *buf, s
 #define OP_FLAG_BULK_IMPORT 0x800000             /* operation is bulk import */
 #define OP_FLAG_NOOP 0x01000000                  /* operation results from urp and
                                                   * should be ignored */
+#define OP_FLAG_ACTION_SKIP_PWDPOLICY 0x02000000 /* Skip applying pw policy rules to the password
+                                                  * change operation, as it's from an upgrade on
+                                                  * bind rather than a normal password change */
+#define OP_FLAG_SUBENTRIES_FALSE 0x04000000      /* Normal entries are visible and subentries are not */
+#define OP_FLAG_SUBENTRIES_TRUE 0x08000000       /* Subentries are visible and normal entries are not */
 
 /* reverse search states */
 #define REV_STARTED 1
@@ -439,6 +454,33 @@ int operation_is_flag_set(Slapi_Operation *op, int flag);
 unsigned long operation_get_type(Slapi_Operation *op);
 LDAPMod **copy_mods(LDAPMod **orig_mods);
 
+/* Structures use to collect statistics per operation */
+/* used for LDAP_STAT_READ_INDEX */
+struct component_keys_lookup
+{
+    char *index_type;
+    char *attribute_type;
+    char *key;
+    int id_lookup_cnt;
+    struct component_keys_lookup *next;
+};
+typedef struct op_search_stat
+{
+    struct component_keys_lookup *keys_lookup;
+    struct timespec keys_lookup_start;
+    struct timespec keys_lookup_end;
+} Op_search_stat;
+
+/* structure store in the operation extension */
+typedef struct op_stat
+{
+    Op_search_stat *search_stat;
+} Op_stat;
+
+void op_stat_init(void);
+Op_stat *op_stat_get_operation_extension(Slapi_PBlock *pb);
+void op_stat_set_operation_extension(Slapi_PBlock *pb, Op_stat *op_stat);
+
 /*
  * From ldap.h
  * #define LDAP_MOD_ADD            0x00
@@ -449,7 +491,6 @@ LDAPMod **copy_mods(LDAPMod **orig_mods);
  */
 #define LDAP_MOD_IGNORE 0x100
 
-
 /* dl.c */
 typedef struct datalist DataList;
 
@@ -578,6 +619,7 @@ typedef struct slapi_operation_parameters
             int search_sizelimit;
             int search_timelimit;
             struct slapi_filter *search_filter;
+            struct slapi_filter *search_filter_intended;
             char *search_strfilter;
             char **search_attrs;
             int search_attrsonly;
@@ -794,6 +836,7 @@ int slapi_lookup_instance_name_by_suffix(char *suffix,
 
 /* begin and end the task subsystem */
 void task_init(void);
+void task_cancel_all(void);
 void task_shutdown(void);
 void task_cleanup(void);
 
@@ -803,6 +846,9 @@ void task_cleanup(void);
 int pw_rever_encode(Slapi_Value **vals, char *attr_name);
 int pw_rever_decode(char *cipher, char **plain, const char *attr_name);
 
+int32_t update_pw_encoding(Slapi_PBlock *orig_pb, Slapi_Entry *e, Slapi_DN *sdn, char *cleartextpassword);
+
+
 /* config routines */
 
 int slapi_config_get_readonly(void);
@@ -895,16 +941,12 @@ int proxyauth_get_dn(Slapi_PBlock *pb, char **proxydnp, char **errtextp);
 #define SLAPI_PLUGIN_DB_DB2ARCHIVE_FN             223
 #define SLAPI_PLUGIN_DB_NEXT_SEARCH_ENTRY_FN      224
 #define SLAPI_PLUGIN_DB_FREE_RESULT_SET_FN        225
-#define SLAPI_PLUGIN_DB_SIZE_FN                   226
 #define SLAPI_PLUGIN_DB_TEST_FN                   227
 #define SLAPI_PLUGIN_DB_DB2INDEX_FN               228
 #define SLAPI_PLUGIN_DB_NEXT_SEARCH_ENTRY_EXT_FN  229
-#define SLAPI_PLUGIN_DB_ENTRY_RELEASE_FN          230
-#define SLAPI_PLUGIN_DB_INIT_INSTANCE_FN          231
 #define SLAPI_PLUGIN_DB_WIRE_IMPORT_FN            234
 #define SLAPI_PLUGIN_DB_UPGRADEDB_FN              235
 #define SLAPI_PLUGIN_DB_DBVERIFY_FN               236
-#define SLAPI_PLUGIN_DB_ADD_SCHEMA_FN             237
 #define SLAPI_PLUGIN_DB_SEARCH_RESULTS_RELEASE_FN 238
 #define SLAPI_PLUGIN_DB_PREV_SEARCH_RESULTS_FN    239
 #define SLAPI_PLUGIN_DB_UPGRADEDNFORMAT_FN        240
@@ -914,6 +956,7 @@ int proxyauth_get_dn(Slapi_PBlock *pb, char **proxydnp, char **errtextp);
 #define SLAPI_PLUGIN_DB_GET_INFO_FN               290
 #define SLAPI_PLUGIN_DB_SET_INFO_FN               291
 #define SLAPI_PLUGIN_DB_CTRL_INFO_FN              292
+#define SLAPI_PLUGIN_DB_COMPACT_FN                294
 
 /**** End of database plugin interface. **************************************/
 
@@ -1210,6 +1253,10 @@ const char *escape_string_for_filename(const char *str, char buf[BUFSIZ]);
 void strcpy_unescape_value(char *d, const char *s);
 void get_internal_conn_op (uint64_t *connid, int32_t *op_id, int32_t *op_internal_id, int32_t *op_nested_count);
 char *slapi_berval_get_string_copy(const struct berval *bval);
+char get_sep(char *path);
+int mkdir_p(char *dir, unsigned int mode);
+const char *ldif_getline_ro( const char **next);
+void dup_ldif_line(struct berval *copy, const char *line, const char *endline);
 
 /* lenstr stuff */
 
@@ -1363,6 +1410,9 @@ char *ldaputil_get_saslpath(void);
 int slapi_client_uses_non_nss(LDAP *ld);
 int slapi_client_uses_openssl(LDAP *ld);
 
+/* rewriters.c */
+int32_t rewriters_init(void);
+
 /* ssl.c */
 /*
  * If non NULL buf and positive bufsize is given,
@@ -1428,6 +1478,13 @@ util_cachesize_result util_is_cachesize_sane(slapi_pal_meminfo *mi, uint64_t *ca
  * \return -1 if the hardware detection failed. Any positive value is threads to use.
  */
 long util_get_hardware_threads(void);
+/**
+ * Retrieve the number of threads the server should run with based on this hardware.
+ *   positive return value is in the [min.max] range
+ *
+ * \return -1 if the hardware detection failed. Any positive value is threads to use.
+ */
+long util_get_capped_hardware_threads(long min, long max);
 
 /**
  * Write an error message to the given error buffer.
@@ -1441,6 +1498,15 @@ void slapi_create_errormsg(char *errorbuf, size_t len, const char *fmt, ...);
 struct slapi_entry *slapi_pblock_get_pw_entry(Slapi_PBlock *pb);
 void slapi_pblock_set_pw_entry(Slapi_PBlock *pb, struct slapi_entry *entry);
 
+uint32_t slapi_pblock_get_operation_notes(Slapi_PBlock *pb);
+void slapi_pblock_set_operation_notes(Slapi_PBlock *pb, uint32_t opnotes);
+void slapi_pblock_set_flag_operation_notes(Slapi_PBlock *pb, uint32_t opflag);
+void slapi_pblock_set_result_text_if_empty(Slapi_PBlock *pb, char *text);
+
+int32_t slapi_pblock_get_task_warning(Slapi_PBlock *pb);
+void slapi_pblock_set_task_warning(Slapi_PBlock *pb, task_warning warning);
+
+int slapi_exists_or_add_internal(Slapi_DN *dn, const char *filter, const char *entry, const char *modifier_name);
 
 #ifdef __cplusplus
 }
