@@ -1,19 +1,23 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2022 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
 # See LICENSE for details.
 # --- END COPYRIGHT BLOCK ---
 
+import collections
 import ldap
 import copy
 import os.path
-
 from lib389 import tasks
 from lib389._mapped_object import DSLdapObjects, DSLdapObject
-from lib389.lint import DSRILE0001
-from lib389._constants import DN_PLUGIN
+from lib389.lint import DSRILE0001, DSRILE0002, DSMOLE0001
+from lib389.utils import ensure_str, ensure_list_bytes
+from lib389.schema import Schema
+from lib389._constants import (
+        DN_PLUGIN, DN_MBO_TASK, DN_AUTOMEMBER_REBUILD_TASK, DN_FIXUP_LINKED_ATTIBUTES,
+        DN_EUUID_TASK)
 from lib389.properties import (
         PLUGINS_OBJECTCLASS_VALUE, PLUGIN_PROPNAME_TO_ATTRNAME,
         PLUGINS_ENABLE_ON_VALUE, PLUGINS_ENABLE_OFF_VALUE, PLUGIN_ENABLE
@@ -120,8 +124,22 @@ class AttributeUniquenessPlugin(Plugin):
     :type dn: str
     """
 
-    def __init__(self, instance, dn="cn=attribute uniqueness,cn=plugins,cn=config"):
+    _plugin_properties = {
+        'nsslapd-pluginEnabled': 'off',
+        'nsslapd-pluginPath': 'libattr-unique-plugin',
+        'nsslapd-pluginInitfunc': 'NSUniqueAttr_Init',
+        'nsslapd-pluginType': 'betxnpreoperation',
+        'nsslapd-plugin-depends-on-type': 'database',
+        'nsslapd-pluginId': 'NSUniqueAttr',
+        'nsslapd-pluginVendor': '389 Project',
+        'nsslapd-pluginVersion': 'none',
+        'nsslapd-pluginDescription': 'Enforce unique attribute values',
+    }
+
+    def __init__(self, instance, dn):
         super(AttributeUniquenessPlugin, self).__init__(instance, dn)
+        self._protected = False
+        self._create_objectclasses = ['top', 'nsslapdplugin', 'extensibleObject']
 
     ## These are some wrappers to the important attributes
     # This plugin will be "tricky" in that it can have "many" instance
@@ -158,6 +176,76 @@ class AttributeUniquenessPlugin(Plugin):
         self.set('uniqueness-across-all-subtrees', 'off')
 
 
+class AttributeUniquenessPlugins(DSLdapObjects):
+    """A DSLdapObjects entity which represents Attribute Uniqueness plugin instances
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn="cn=plugins,cn=config"):
+        super(DSLdapObjects, self).__init__(instance.verbose)
+        self._instance = instance
+        self._objectclasses = ['top', 'nsslapdplugin', 'extensibleObject']
+        self._filterattrs = ['cn', 'nsslapd-pluginPath']
+        self._childobject = AttributeUniquenessPlugin
+        self._basedn = basedn
+        # This is used to allow entry to instance to work
+        self._list_attrlist = ['dn', 'nsslapd-pluginPath']
+        self._search_filter = "(nsslapd-pluginInitfunc=NSUniqueAttr_Init)"
+        self._scope = ldap.SCOPE_SUBTREE
+        self._server_controls = None
+        self._client_controls = None
+
+    def list(self):
+        """Get a list of all plugin instances where nsslapd-pluginInitfunc: NSUniqueAttr_Init
+
+        :returns: A list of children entries
+        """
+
+        try:
+            results = self._instance.search_ext_s(
+                base=self._basedn,
+                scope=self._scope,
+                filterstr=self._search_filter,
+                attrlist=self._list_attrlist,
+                serverctrls=self._server_controls, clientctrls=self._client_controls
+            )
+            insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+        except ldap.NO_SUCH_OBJECT:
+            # There are no objects to select from, se we return an empty array
+            insts = []
+        return insts
+
+    def _get_dn(self, dn):
+        # This will yield and & filter for objectClass with as many terms as needed.
+        self._log.debug('_gen_dn filter = %s' % self._search_filter)
+        self._log.debug('_gen_dn dn = %s' % dn)
+        return self._instance.search_ext_s(
+            base=dn,
+            scope=ldap.SCOPE_BASE,
+            filterstr=self._search_filter,
+            attrlist=self._list_attrlist,
+            serverctrls=self._server_controls, clientctrls=self._client_controls
+        )
+
+    def _get_selector(self, selector):
+        # Filter based on the objectclasses and the basedn
+        # Based on the selector, we should filter on that too.
+        # This will yield and & filter for objectClass with as many terms as needed.
+        filterstr = "(&(cn=%s)%s)" % (selector, self._search_filter)
+        self._log.debug('_gen_selector filter = %s' % filterstr)
+        return self._instance.search_ext_s(
+            base=self._basedn,
+            scope=self._scope,
+            filterstr=filterstr,
+            attrlist=self._list_attrlist,
+            serverctrls=self._server_controls, clientctrls=self._client_controls
+        )
+
+
 class LdapSSOTokenPlugin(Plugin):
     """An instance of ldapssotoken plugin entry
 
@@ -208,8 +296,17 @@ class MEPConfig(DSLdapObject):
     def __init__(self, instance, dn):
         super(MEPConfig, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
-        self._must_attributes = ['cn']
-        self._create_objectclasses = ['top', 'extensibleObject']
+        schema = Schema(instance)
+        for oc in schema.get_objectclasses():
+            if oc.oid == '2.16.840.1.113730.3.2.336':
+                self._must_attributes = ['cn', 'originScope', 'originFilter',
+                                         'managedBase', 'managedTemplate']
+                self._create_objectclasses = ['top', 'mepConfigEntry']
+                break
+        else:
+            # Workaround for older versions without MEP schema
+            self._must_attributes = ['cn']
+            self._create_objectclasses = ['top', 'extensibleObject']
         self._protected = False
 
 
@@ -222,13 +319,52 @@ class MEPConfigs(DSLdapObjects):
     :type basedn: str
     """
 
-    def __init__(self, instance, basedn="cn=managed entries,cn=plugins,cn=config"):
+    def __init__(self, instance, basedn=None):
         super(MEPConfigs, self).__init__(instance)
-        self._objectclasses = ['top', 'extensibleObject']
+        schema = Schema(instance)
+        for oc in schema.get_objectclasses():
+            if oc.oid == '2.16.840.1.113730.3.2.336':
+                self._objectclasses = ['top', 'mepConfigEntry']
+                break
+        else:
+            # Workaround for older versions without MEP schema
+            self._objectclasses = ['top', 'extensibleObject']
         self._filterattrs = ['cn']
         self._childobject = MEPConfig
+        # So we can set the configArea easily
+        if basedn is None:
+            basedn = "cn=managed entries,cn=plugins,cn=config"
         self._basedn = basedn
 
+    def list(self):
+        """Get a list of children entries (DSLdapObject, Replica, etc.) using a base DN
+        and objectClasses of our object (DSLdapObjects, Replicas, etc.)
+
+        :returns: A list of children entries
+        """
+
+        # Filter based on the objectclasses and the basedn
+        insts = None
+        # This will yield and & filter for objectClass with as many terms as needed.
+        filterstr = self._get_objectclass_filter()
+        self._log.debug('list filter = %s' % filterstr)
+        try:
+            results = self._instance.search_ext_s(
+                base=self._basedn,
+                scope=self._scope,
+                filterstr=filterstr,
+                attrlist=self._list_attrlist,
+                serverctrls=self._server_controls, clientctrls=self._client_controls,
+                escapehatch='i am sure'
+            )
+            # def __init__(self, instance, dn=None):
+            insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+        except ldap.NO_SUCH_OBJECT:
+            # There are no objects to select from, se we return an empty array
+            insts = []
+        return insts
+
+
 
 class MEPTemplate(DSLdapObject):
     """A single instance of MEP template entry
@@ -243,7 +379,7 @@ class MEPTemplate(DSLdapObject):
         super(MEPTemplate, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['cn']
-        self._create_objectclasses = ['top', 'extensibleObject', 'mepTemplateEntry']
+        self._create_objectclasses = ['top', 'mepTemplateEntry']
         self._protected = False
 
 
@@ -258,7 +394,7 @@ class MEPTemplates(DSLdapObjects):
 
     def __init__(self, instance, basedn):
         super(MEPTemplates, self).__init__(instance)
-        self._objectclasses = ['top', 'extensibleObject']
+        self._objectclasses = ['top', 'mepTemplateEntry']
         self._filterattrs = ['cn']
         self._childobject = MEPTemplate
         self._basedn = basedn
@@ -297,7 +433,6 @@ class ReferentialIntegrityPlugin(Plugin):
             'referint-logfile',
             'referint-membership-attr',
         ])
-        self._lint_functions = [self._lint_update_delay]
 
     def create(self, rdn=None, properties=None, basedn=None):
         """Create an instance of the plugin"""
@@ -309,11 +444,66 @@ class ReferentialIntegrityPlugin(Plugin):
             properties['referint-logfile'] = referint_log
         return super(ReferentialIntegrityPlugin, self).create(rdn, properties, basedn)
 
+    @classmethod
+    def lint_uid(cls):
+        return 'refint'
+
     def _lint_update_delay(self):
         if self.status():
             delay = self.get_attr_val_int("referint-update-delay")
             if delay is not None and delay != 0:
-                return DSRILE0001
+                report = copy.deepcopy(DSRILE0001)
+                report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                report['check'] = f'refint:update_delay'
+                yield report
+
+    def _lint_attr_indexes(self):
+        if self.status():
+            from lib389.backend import Backends
+            backends = Backends(self._instance).list()
+            attrs = self.get_attr_vals_utf8_l("referint-membership-attr")
+            container = self.get_attr_val_utf8_l("nsslapd-plugincontainerscope")
+            for backend in backends:
+                suffix = backend.get_attr_val_utf8_l('nsslapd-suffix')
+                if suffix == "cn=changelog":
+                    # Always skip retro changelog
+                    continue
+                if container is not None:
+                    # Check if this backend is in the scope
+                    if not container.endswith(suffix):
+                        # skip this backend that is not in the scope
+                        continue
+                indexes = backend.get_indexes()
+                for attr in attrs:
+                    report = copy.deepcopy(DSRILE0002)
+                    try:
+                        index = indexes.get(attr)
+                        types = index.get_attr_vals_utf8_l("nsIndexType")
+                        valid = False
+                        if "eq" in types:
+                            valid = True
+
+                        if not valid:
+                            report['detail'] = report['detail'].replace('ATTR', attr)
+                            report['detail'] = report['detail'].replace('BACKEND', suffix)
+                            report['fix'] = report['fix'].replace('ATTR', attr)
+                            report['fix'] = report['fix'].replace('BACKEND', suffix)
+                            report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                            report['items'].append(suffix)
+                            report['items'].append(attr)
+                            report['check'] = f'refint:attr_indexes'
+                            yield report
+                    except:
+                        # No index at all, bad
+                        report['detail'] = report['detail'].replace('ATTR', attr)
+                        report['detail'] = report['detail'].replace('BACKEND', suffix)
+                        report['fix'] = report['fix'].replace('ATTR', attr)
+                        report['fix'] = report['fix'].replace('BACKEND', suffix)
+                        report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                        report['items'].append(suffix)
+                        report['items'].append(attr)
+                        report['check'] = f'refint:attr_indexes'
+                        yield report
 
     def get_update_delay(self):
         """Get referint-update-delay attribute"""
@@ -330,6 +520,21 @@ class ReferentialIntegrityPlugin(Plugin):
 
         self.set('referint-update-delay', str(value))
 
+    def get_log_file(self):
+        """Get referint log file"""
+
+        return self.get_attr_val_utf8('referint-logfile')
+
+    def get_log_file_formatted(self):
+        """Get referint log file"""
+
+        return self.display_attr('referint-logfile')
+
+    def set_log_file(self, value):
+        """Set referint log file"""
+
+        self.set('referint-logfile', value)
+
     def get_membership_attr(self, formatted=False):
         """Get referint-membership-attr attribute"""
 
@@ -570,6 +775,58 @@ class MemberOfPlugin(Plugin):
         self._create_objectclasses.extend(['extensibleObject'])
         self._must_attributes.extend(['memberOfGroupAttr', 'memberOfAttr'])
 
+    @classmethod
+    def lint_uid(cls):
+        return 'memberof'
+
+    def _lint_member_attr_indexes(self):
+        if self.status():
+            from lib389.backend import Backends
+            backends = Backends(self._instance).list()
+            attrs = self.get_attr_vals_utf8_l("memberofgroupattr")
+            container = self.get_attr_val_utf8_l("nsslapd-plugincontainerscope")
+            for backend in backends:
+                suffix = backend.get_attr_val_utf8_l('nsslapd-suffix')
+                if suffix == "cn=changelog":
+                    # Always skip retro changelog
+                    continue
+                if container is not None:
+                    # Check if this backend is in the scope
+                    if not container.endswith(suffix):
+                        # skip this backend that is not in the scope
+                        continue
+                indexes = backend.get_indexes()
+                for attr in attrs:
+                    report = copy.deepcopy(DSMOLE0001)
+                    try:
+                        index = indexes.get(attr)
+                        types = index.get_attr_vals_utf8_l("nsIndexType")
+                        valid = False
+                        if "eq" in types:
+                            valid = True
+
+                        if not valid:
+                            report['detail'] = report['detail'].replace('ATTR', attr)
+                            report['detail'] = report['detail'].replace('BACKEND', suffix)
+                            report['fix'] = report['fix'].replace('ATTR', attr)
+                            report['fix'] = report['fix'].replace('BACKEND', suffix)
+                            report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                            report['items'].append(suffix)
+                            report['items'].append(attr)
+                            report['check'] = f'memberof:attr_indexes'
+                            yield report
+                    except:
+                        # No index at all, bad
+                        report['detail'] = report['detail'].replace('ATTR', attr)
+                        report['detail'] = report['detail'].replace('BACKEND', suffix)
+                        report['fix'] = report['fix'].replace('ATTR', attr)
+                        report['fix'] = report['fix'].replace('BACKEND', suffix)
+                        report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
+                        report['items'].append(suffix)
+                        report['items'].append(attr)
+                        report['check'] = f'memberof:attr_indexes'
+                        yield report
+
     def get_attr(self):
         """Get memberofattr attribute"""
 
@@ -750,7 +1007,10 @@ class MemberOfPlugin(Plugin):
         task_properties = {'basedn': basedn}
         if _filter is not None:
             task_properties['filter'] = _filter
-        task.create(properties=task_properties)
+        try:
+            task.create(properties=task_properties)
+        except ldap.NO_SUCH_OBJECT:
+            raise ValueError("The fixup task can not be run, the memberOf plugin is not fully enabled.")
 
         return task
 
@@ -774,6 +1034,23 @@ class MemberOfSharedConfig(DSLdapObject):
         self._exit_code = None
 
 
+class MemberOfSharedConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents MemberOf config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn=None):
+        super(MemberOfSharedConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'extensibleObject']
+        self._filterattrs = ['cn']
+        self._childobject = MemberOfSharedConfig
+        self._basedn = basedn
+
+
 class RetroChangelogPlugin(Plugin):
     """An instance of Retro Changelog plugin entry
 
@@ -883,23 +1160,16 @@ class AutoMembershipPlugin(Plugin):
 
         return task
 
+    def abort_fixup(self):
+        """Create an automember abort rebuild task
 
-class AutoMembershipRegexRule(DSLdapObject):
-    def __init__(self, instance, dn=None):
-        super(AutoMembershipRegexRule, self).__init__(instance, dn)
-        self._rdn_attribute = 'cn'
-        self._must_attributes = ['cn', 'autoMemberTargetGroup']
-        self._create_objectclasses = ['top', 'autoMemberRegexRule']
-        self._protected = False
+        :returns: an instance of Task(DSLdapObject)
+        """
 
+        task = tasks.AutomemberAbortRebuildTask(self._instance)
+        task.create()
 
-class AutoMembershipRegexRules(DSLdapObjects):
-    def __init__(self, instance, basedn="cn=Auto Membership Plugin,cn=plugins,cn=config"):
-        super(AutoMembershipRegexRules, self).__init__(instance)
-        self._objectclasses = ['top', 'autoMemberRegexRule']
-        self._filterattrs = ['cn']
-        self._childobject = AutoMembershipRegexRule
-        self._basedn = basedn
+        return task
 
 
 class AutoMembershipDefinition(DSLdapObject):
@@ -911,7 +1181,7 @@ class AutoMembershipDefinition(DSLdapObject):
     :type dn: str
     """
 
-    def __init__(self, instance, dn=None):
+    def __init__(self, instance, dn):
         super(AutoMembershipDefinition, self).__init__(instance, dn)
         self._rdn_attribute = 'cn'
         self._must_attributes = ['cn', 'autoMemberScope', 'autoMemberFilter', 'autoMemberGroupingAttr']
@@ -1097,21 +1367,19 @@ class LinkedAttributesPlugin(Plugin):
     def __init__(self, instance, dn="cn=Linked Attributes,cn=plugins,cn=config"):
         super(LinkedAttributesPlugin, self).__init__(instance, dn)
 
-    def fixup(self, basedn, _filter=None):
+    def fixup(self, linkdn):
         """Create a fixup linked attributes task
 
-        :param basedn: Basedn to fix up
-        :type basedn: str
-        :param _filter: a filter for entries to fix up
-        :type _filter: str
+        :param linkdn: Link DN to fix up
+        :type linkdn: str
 
         :returns: an instance of Task(DSLdapObject)
         """
 
         task = tasks.FixupLinkedAttributesTask(self._instance)
-        task_properties = {'basedn': basedn}
-        if _filter is not None:
-            task_properties['filter'] = _filter
+        task_properties = {}
+        if linkdn is not None:
+            task_properties['linkdn'] = linkdn
         task.create(properties=task_properties)
 
         return task
@@ -1147,6 +1415,7 @@ class LinkedAttributesConfigs(DSLdapObjects):
         super(LinkedAttributesConfigs, self).__init__(instance)
         self._objectclasses = ['top', 'extensibleObject']
         self._filterattrs = ['cn']
+        self._scope = ldap.SCOPE_ONELEVEL
         self._childobject = LinkedAttributesConfig
         self._basedn = basedn
 
@@ -1163,6 +1432,112 @@ class PassThroughAuthenticationPlugin(Plugin):
     def __init__(self, instance, dn="cn=Pass Through Authentication,cn=plugins,cn=config"):
         super(PassThroughAuthenticationPlugin, self).__init__(instance, dn)
 
+    def get_urls(self):
+        """Get all URLs from nsslapd-pluginargNUM attributes
+
+        :returns: a list
+        """
+
+        attr_dict = collections.OrderedDict(sorted(self.get_all_attrs().items()))
+        result = {}
+        for attr, value in attr_dict.items():
+            if attr.startswith("nsslapd-pluginarg"):
+                result[attr] = ensure_str(value[0])
+        return result
+
+
+class POSIXWinsyncPlugin(Plugin):
+    """A single instance of Posix Winsync API plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=Posix Winsync API,cn=plugins,cn=config"):
+        super(POSIXWinsyncPlugin, self).__init__(instance, dn)
+
+    def fixup(self, basedn, _filter=None):
+        """Create a memberuid task
+
+        :param basedn: Basedn to fix up
+        :type basedn: str
+        :param _filter: a filter for entries to fix up
+        :type _filter: str
+
+        :returns: an instance of Task(DSLdapObject)
+        """
+
+        task = tasks.MemberUidFixupTask(self._instance)
+        task_properties = {'basedn': basedn}
+        if _filter is not None:
+            task_properties['filter'] = _filter
+        task.create(properties=task_properties)
+
+        return task
+
+
+class PAMPassThroughAuthPlugin(Plugin):
+    """A single instance of PAM Pass Through Auth plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=PAM Pass Through Auth,cn=plugins,cn=config"):
+        super(PAMPassThroughAuthPlugin, self).__init__(instance, dn)
+
+
+class PAMPassThroughAuthConfig(Plugin):
+    """A single instance of PAM Pass Through Auth config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    _plugin_properties = {
+        'cn' : 'USN',
+        'nsslapd-pluginEnabled': 'off',
+        'nsslapd-pluginPath': 'libpam-passthru-plugin',
+        'nsslapd-pluginInitfunc': 'pam_passthruauth_init',
+        'nsslapd-pluginType': 'betxnpreoperation',
+        'nsslapd-plugin-depends-on-type': 'database',
+        'nsslapd-pluginId': 'PAM',
+        'nsslapd-pluginVendor': '389 Project',
+        'nsslapd-pluginVersion': '1.3.7.0',
+        'nsslapd-pluginDescription': 'PAM Pass Through Auth plugin'
+    }
+
+    def __init__(self, instance, dn=None):
+        super(PAMPassThroughAuthConfig, self).__init__(instance, dn)
+        self._rdn_attribute = 'cn'
+        self._must_attributes = ['cn']
+        self._create_objectclasses = ['top', 'extensibleObject', 'nsslapdplugin', 'pamConfig']
+        self._protected = False
+
+
+class PAMPassThroughAuthConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents PAM Pass Through Auth config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn="cn=PAM Pass Through Auth,cn=plugins,cn=config"):
+        super(PAMPassThroughAuthConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'extensibleObject', 'nsslapdplugin', 'pamConfig']
+        self._filterattrs = ['cn']
+        self._scope = ldap.SCOPE_ONELEVEL
+        self._childobject = PAMPassThroughAuthConfig
+        self._basedn = basedn
+
 
 class USNPlugin(Plugin):
     """A single instance of USN (Update Sequence Number) plugin entry
@@ -1421,7 +1796,7 @@ class RootDNAccessControlPlugin(Plugin):
     def get_allow_ip(self):
         """Get rootdn-allow-ip attribute"""
 
-        return self.get_attr_val_utf8('rootdn-allow-ip')
+        return self.get_attr_vals_utf8('rootdn-allow-ip')
 
     def get_allow_ip_formatted(self):
         """Display rootdn-allow-ip attribute"""
@@ -1446,7 +1821,7 @@ class RootDNAccessControlPlugin(Plugin):
     def get_deny_ip(self):
         """Remove all rootdn-deny-ip attribute"""
 
-        return self.get_attr_val_utf8('rootdn-deny-ip')
+        return self.get_attr_vals_utf8('rootdn-deny-ip')
 
     def get_deny_ip_formatted(self):
         """Display rootdn-deny-ip attribute"""
@@ -1582,6 +1957,40 @@ class AccountPolicyConfigs(DSLdapObjects):
         self._basedn = basedn
 
 
+class AccountPolicyEntry(DSLdapObject):
+    """A single instance of Account Policy Plugin entry which is used for CoS
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(AccountPolicyEntry, self).__init__(instance, dn)
+        self._rdn_attribute = 'cn'
+        self._must_attributes = ['cn']
+        self._create_objectclasses = ['top', 'accountpolicy']
+        self._protected = False
+
+
+class AccountPolicyEntries(DSLdapObjects):
+    """A DSLdapObjects entity which represents Account Policy Plugin entry which is used for CoS
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn):
+        super(AccountPolicyConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'accountpolicy']
+        self._filterattrs = ['cn']
+        self._childobject = AccountPolicyEntry
+        self._basedn = basedn
+
+
 class DNAPlugin(Plugin):
     """A single instance of Distributed Numeric Assignment plugin entry
 
@@ -1629,6 +2038,179 @@ class DNAPluginConfigs(DSLdapObjects):
         self._basedn = basedn
 
 
+class DNAPluginSharedConfig(DSLdapObject):
+    """A single instance of DNA Plugin config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn=None):
+        super(DNAPluginSharedConfig, self).__init__(instance, dn)
+        self._rdn_attribute = 'dnaHostname'
+        self._must_attributes = ['dnaHostname', 'dnaPortNum']
+        self._create_objectclasses = ['top', 'dnaSharedConfig']
+        self._protected = False
+
+    def create(self, properties=None, basedn=None, ensure=False):
+        """The shared config DNA plugin entry has two RDN values
+        The function takes care about that special case
+        """
+
+        for attr in self._must_attributes:
+            if properties.get(attr, None) is None:
+                raise ldap.UNWILLING_TO_PERFORM('Attribute %s must not be None' % attr)
+
+        assert basedn is not None, "Base DN should be specified"
+
+        # Make a DN with the two items RDN and base DN
+        decomposed_dn = [[('dnaHostname', properties['dnaHostname'], 1),
+                          ('dnaPortNum', properties['dnaPortNum'], 1)]] + ldap.dn.str2dn(basedn)
+        dn = ldap.dn.dn2str(decomposed_dn)
+
+        exists = False
+        if ensure:
+            # If we are running in stateful ensure mode, we need to check if the object exists, and
+            # we can see the state that it is in.
+            try:
+                self._instance.search_ext_s(dn, ldap.SCOPE_BASE, self._object_filter, attrsonly=1,
+                                            serverctrls=self._server_controls, clientctrls=self._client_controls,
+                                            escapehatch='i am sure')
+                exists = True
+            except ldap.NO_SUCH_OBJECT:
+                pass
+
+        if exists and ensure:
+            # update properties
+            self._log.debug('Exists %s' % dn)
+            self._dn = dn
+            # Now use replace_many to setup our values
+            mods = []
+            for k, v in list(properties.items()):
+                mods.append((ldap.MOD_REPLACE, k, v))
+            self._instance.modify_ext_s(self._dn, mods, serverctrls=self._server_controls,
+                                        clientctrls=self._client_controls, escapehatch='i am sure')
+        else:
+            self._log.debug('Creating %s' % dn)
+            mods = [('objectclass', ensure_list_bytes(self._create_objectclasses))]
+            # Bring our mods to one type and do ensure bytes on the list
+            for attr, value in properties.items():
+                if not isinstance(value, list):
+                    value = [value]
+                mods.append((attr, ensure_list_bytes(value)))
+            # We rely on exceptions here to indicate failure to the parent.
+            self._log.debug('Creating entry %s : %s' % (dn, mods))
+            self._instance.add_ext_s(dn, mods, serverctrls=self._server_controls, clientctrls=self._client_controls,
+                                     escapehatch='i am sure')
+            # If it worked, we need to fix our instance dn
+            self._dn = dn
+
+        return self
+
+
+class DNAPluginSharedConfigs(DSLdapObjects):
+    """A DSLdapObjects entity which represents DNA Plugin config entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param basedn: Base DN for all account entries below
+    :type basedn: str
+    """
+
+    def __init__(self, instance, basedn=None):
+        super(DNAPluginSharedConfigs, self).__init__(instance)
+        self._objectclasses = ['top', 'dnaSharedConfig']
+        self._filterattrs = ['dnaHostname', 'dnaPortNum']
+        self._childobject = DNAPluginSharedConfig
+        self._basedn = basedn
+
+    def create(self, properties=None):
+        """Create an object under base DN of our entry
+
+        :param properties: Attributes for the new entry
+        :type properties: dict
+
+        :returns: DSLdapObject of the created entry
+        """
+
+        co = self._entry_to_instance(dn=None, entry=None)
+        return co.create(properties, self._basedn)
+
+
+class MemberOfFixupTasks(DSLdapObjects):
+    """A DSLdapObjects entity which represents memberOf fixup tasks
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param task_dn: dn for a specific task
+    :type basedn: str
+    """
+
+    def __init__(self, instance, task_dn=None):
+        super(MemberOfFixupTasks, self).__init__(instance)
+        self._objectclasses = ['top']
+        self._filterattrs = ['cn']
+        self._childobject = DSLdapObject
+        self._basedn = DN_MBO_TASK
+        self._scope = ldap.SCOPE_ONELEVEL
+
+
+class AutoMembershipFixupTasks(DSLdapObjects):
+    """A DSLdapObjects entity which represents automember fixup tasks
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param task_dn: dn for a specific task
+    :type basedn: str
+    """
+
+    def __init__(self, instance, task_dn=None):
+        super(AutoMembershipFixupTasks, self).__init__(instance)
+        self._objectclasses = ['top']
+        self._filterattrs = ['cn']
+        self._childobject = DSLdapObject
+        self._basedn = DN_AUTOMEMBER_REBUILD_TASK
+        self._scope = ldap.SCOPE_ONELEVEL
+
+
+class LinkedAttributesFixupTasks(DSLdapObjects):
+    """A DSLdapObjects entity which represents automember fixup tasks
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param task_dn: dn for a specific task
+    :type basedn: str
+    """
+
+    def __init__(self, instance, task_dn=None):
+        super(LinkedAttributesFixupTasks, self).__init__(instance)
+        self._objectclasses = ['top']
+        self._filterattrs = ['cn']
+        self._childobject = DSLdapObject
+        self._basedn = DN_FIXUP_LINKED_ATTIBUTES
+        self._scope = ldap.SCOPE_ONELEVEL
+
+
+class EntryUUIDFixupTasks(DSLdapObjects):
+    """A DSLdapObjects entity which represents automember fixup tasks
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param task_dn: dn for a specific task
+    :type basedn: str
+    """
+
+    def __init__(self, instance, task_dn=None):
+        super(EntryUUIDFixupTasks, self).__init__(instance)
+        self._objectclasses = ['top']
+        self._filterattrs = ['cn']
+        self._childobject = DSLdapObject
+        self._basedn = DN_EUUID_TASK
+        self._scope = ldap.SCOPE_ONELEVEL
+
+
 class Plugins(DSLdapObjects):
     """A DSLdapObjects entity which represents plugin entry
 
@@ -1791,3 +2373,61 @@ class PluginsLegacy(object):
         self.conn.modify_s(dn, [(ldap.MOD_REPLACE,
                                  PLUGIN_PROPNAME_TO_ATTRNAME[PLUGIN_ENABLE],
                                  PLUGINS_ENABLE_OFF_VALUE)])
+
+
+class BitwisePlugin(Plugin):
+    """A single instance of Bitwise plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=Bitwise Plugin,cn=plugins,cn=config"):
+        super(BitwisePlugin, self).__init__(instance, dn)
+
+
+class EntryUUIDPlugin(Plugin):
+    """The EntryUUID plugin configuration
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=entryuuid,cn=plugins,cn=config"):
+        super(EntryUUIDPlugin, self).__init__(instance, dn)
+
+    def fixup(self, basedn, _filter=None):
+        """Create an entryuuid fixup task
+
+        :param basedn: Basedn to fix up
+        :type basedn: str
+        :param _filter: a filter for entries to fix up
+        :type _filter: str
+
+        :returns: an instance of Task(DSLdapObject)
+        """
+
+        task = tasks.EntryUUIDFixupTask(self._instance)
+        task_properties = {'basedn': basedn}
+        if _filter is not None:
+            task_properties['filter'] = _filter
+        task.create(properties=task_properties)
+
+        return task
+
+class ContentSyncPlugin(Plugin):
+    """A single instance of Content Sync (aka syncrepl) plugin entry
+
+    :param instance: An instance
+    :type instance: lib389.DirSrv
+    :param dn: Entry DN
+    :type dn: str
+    """
+
+    def __init__(self, instance, dn="cn=Content Synchronization,cn=plugins,cn=config"):
+        super(ContentSyncPlugin, self).__init__(instance, dn)
+
