@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2021 Red Hat, Inc.
 # All rights reserved.
 #
 # License: GPL (version 3 or any later version).
@@ -19,6 +19,7 @@ import time
 import shutil
 import subprocess
 import tarfile
+import urllib
 import re
 import glob
 import pwd
@@ -26,52 +27,16 @@ import grp
 import logging
 import ldap
 import shlex
-import socket
-import getpass
-
 # from .nss_ssl import nss_create_new_database
 from threading import Timer
 from lib389.paths import Paths
 from lib389._constants import *
-from lib389._ldifconn import LDIFConn
 from lib389.properties import *
 from lib389.utils import (
-    is_a_dn,
-    getcfgdsuserdn,
-    getcfgdsinfo,
-    getcfgdsuserdn,
-    update_newhost_with_fqdn,
-    get_sbin_dir,
-    get_server_user,
-    getdomainname,
-    isLocalHost,
-    formatInfData,
-    getserverroot,
-    update_admin_domain,
-    getadminport,
-    getdefaultsuffix,
     ensure_bytes,
     ensure_str,
-    socket_check_open,)
-from lib389.passwd import password_hash, password_generate
-
-
-try:
-    # There are too many issues with this on EL7
-    # Out of the box, it's just outright broken ...
-    import six.moves.urllib.request
-    import six.moves.urllib.parse
-    import six.moves.urllib.error
-    import six
-except ImportError:
-    pass
-
-MAJOR, MINOR, _, _, _ = sys.version_info
-
-if MAJOR >= 3:
-    import configparser
-else:
-    import ConfigParser as configparser
+    ds_is_older,
+)
 
 __all__ = ['DirSrvTools']
 try:
@@ -287,7 +252,7 @@ class DirSrvTools(object):
             dirsrv.dbdir: directory where is stored the database
                           (e.g. /var/lib/dirsrv/slapd-standalone/db)
             dirsrv.changelogdir: directory where is stored the changelog
-                                (e.g. /var/lib/dirsrv/slapd-master/changelogdb)
+                                (e.g. /var/lib/dirsrv/slapd-supplier/changelogdb)
         """
 
         # First check it if already exists a backup file
@@ -521,14 +486,9 @@ class DirSrvTools(object):
     @staticmethod
     def removeInstance(dirsrv):
         """run the remove instance command"""
-        if hasattr(dirsrv, 'prefix'):
-            prefix = dirsrv.prefix
-        else:
-            prefix = None
-
-        prog = os.path.join(_ds_paths.sbin_dir, PATH_REMOVE_DS)
+        prog = os.path.join(_ds_paths.sbin_dir, 'dsctl')
         try:
-            cmd = [prog, '-i', 'slapd-{}'.format(dirsrv.serverid)]
+            cmd = "%s slapd-%s remove --do-it" % (prog, dirsrv.serverid)
             log.info('Running: {}'.format(" ".join(cmd)))
             subprocess.check_call(cmd)
         except subprocess.CalledProcessError as e:
@@ -549,7 +509,8 @@ class DirSrvTools(object):
                 instance.prefix
                 instance.backup
         '''
-        instance = lib389.DirSrv(verbose=True)
+        from lib389 import DirSrv
+        instance = DirSrv(verbose=True)
         instance.allocate(args)
 
         return instance
@@ -586,224 +547,6 @@ class DirSrvTools(object):
 
         return None
 
-    @staticmethod
-    def createInstance(args, verbose=0):
-        """Create a new instance of directory server and return a connection
-        to it.
-
-        This function:
-        - guesses the hostname where to create the DS, using
-        localhost by default;
-        - figures out if the given hostname is the local host or not.
-
-        @param args -  a dict with the following values {
-            # new instance compulsory values
-            'newinstance': 'rpolli',
-            'newsuffix': 'dc=example,dc=com',
-            'newhost': 'localhost.localdomain',
-            'newport': 22389,
-            'newrootpw': 'password',
-
-            # optionally register instance on an admin tree
-            'have_admin': True,
-
-            # optionally directory where to store instance backup
-            'backupdir': [ /tmp ]
-
-            # you can configure a new dirsrv-admin
-            'setup_admin': True,
-
-            # or you need the dirsrv-admin to be already setup
-            'cfgdshost': 'localhost.localdomain',
-            'cfgdsport': 22389,
-            'cfgdsuser': 'admin',
-            'cfgdspwd': 'admin',
-
-        }
-        """
-        cfgdn = lib389.CFGSUFFIX
-        isLocal = update_newhost_with_fqdn(args)
-
-        # use prefix if binaries are relocated
-        sroot = args.get('sroot', '')
-        prefix = args.setdefault('prefix', '')
-
-        # get the backup directory to store instance backup
-        backupdir = args.get('backupdir', '/tmp')
-
-        # new style - prefix or FHS?
-        args['new_style'] = not args.get('sroot')
-
-        # do we have ds only or ds+admin?
-        if 'no_admin' not in args:
-            ds_admin_path = os.path.join(_ds_paths.sbin_dir, PATH_SETUP_DS_ADMIN)
-            if os.path.isfile(ds_admin_path):
-                args['have_admin'] = True
-
-        # set default values
-        args['have_admin'] = args.get('have_admin', False)
-        args['setup_admin'] = args.get('setup_admin', False)
-
-        # get default values from adm.conf
-        if args['new_style'] and args['have_admin']:
-            admconf = LDIFConn(
-                args['prefix'] + PATH_ADM_CONF)
-            args['admconf'] = admconf.get('')
-
-        # next, get the configuration ds host and port
-        if args['have_admin']:
-            args['cfgdshost'], args['cfgdsport'], cfgdn = getcfgdsinfo(args)
-        #
-        # if a Config DS is passed, get the userdn. This creates
-        # a connection to the given DS. If you don't want to connect
-        # to this server you should pass 'setup_admin' too.
-        #
-        if args['have_admin'] and not args['setup_admin']:
-            cfgconn = getcfgdsuserdn(cfgdn, args)
-
-        # next, get the server root if not given
-        if not args['new_style']:
-            getserverroot(cfgconn, isLocal, args)
-        # next, get the admin domain
-        if args['have_admin']:
-            update_admin_domain(isLocal, args)
-        # next, get the admin server port and any other information -
-        # close the cfgconn
-        if args['have_admin'] and not args['setup_admin']:
-            asport, secure = getadminport(cfgconn, cfgdn, args)
-        # next, get the posix username
-        get_server_user(args)
-        # fixup and verify other args
-        args['newport'] = args.get('newport', 389)
-        args['newrootdn'] = args.get('newrootdn', DN_DM)
-        args['newsuffix'] = args.get('newsuffix',
-                                     getdefaultsuffix(args['newhost']))
-
-        if not isLocal or 'cfgdshost' in args:
-            if 'admin_domain' not in args:
-                args['admin_domain'] = getdomainname(args['newhost'])
-            if isLocal and 'cfgdspwd' not in args:
-                args['cfgdspwd'] = "dummy"
-            if isLocal and 'cfgdshost' not in args:
-                args['cfgdshost'] = args['newhost']
-            if isLocal and 'cfgdsport' not in args:
-                args['cfgdsport'] = 55555
-        missing = False
-        for param in ('newhost', 'newport', 'newrootdn', 'newrootpw',
-                      'newinstance', 'newsuffix'):
-            if param not in args:
-                log.error("missing required argument: ", param)
-                missing = True
-        if missing:
-            raise InvalidArgumentError("missing required arguments")
-
-        # try to connect with the given parameters
-        try:
-            newconn = lib389.DirSrv(args['newhost'], args['newport'],
-                                    args['newrootdn'], args['newrootpw'],
-                                    args['newinstance'])
-            newconn.prefix = prefix
-            newconn.backupdir = backupdir
-            newconn.isLocal = isLocal
-            if args['have_admin'] and not args['setup_admin']:
-                newconn.asport = asport
-                newconn.cfgdsuser = args['cfgdsuser']
-                newconn.cfgdspwd = args['cfgdspwd']
-
-            host = args['newhost']
-            port = args['newport']
-            print("Warning: server at %s:%s " % (host, port) +
-                  "already exists, returning connection to it")
-            return newconn
-        except ldap.SERVER_DOWN:
-            pass  # not running - create new one
-
-        if not isLocal or 'cfgdshost' in args:
-            for param in ('cfgdshost', 'cfgdsport', 'cfgdsuser', 'cfgdspwd',
-                          'admin_domain'):
-                if param not in args:
-                    print("missing required argument", param)
-                    missing = True
-        if not isLocal and not asport:
-            print("missing required argument admin server port")
-            missing = True
-        if missing:
-            raise InvalidArgumentError("missing required arguments")
-
-        # construct a hash table with our CGI arguments - used with cgiPost
-        # and cgiFake
-        cgiargs = {
-            'servname': args['newhost'],
-            'servport': args['newport'],
-            'rootdn': args['newrootdn'],
-            'rootpw': args['newrootpw'],
-            'servid': args['newinstance'],
-            'suffix': args['newsuffix'],
-            'servuser': args['newuserid'],
-            'start_server': 1
-        }
-        if 'cfgdshost' in args:
-            cgiargs['cfg_sspt_uid'] = args['cfgdsuser']
-            cgiargs['cfg_sspt_uid_pw'] = args['cfgdspwd']
-            cgiargs['ldap_url'] = "ldap://%s:%d/%s" % (
-                args['cfgdshost'], args['cfgdsport'], cfgdn)
-            cgiargs['admin_domain'] = args['admin_domain']
-
-        if not isLocal:
-            DirSrvTools.cgiPost(args['newhost'], asport, args['cfgdsuser'],
-                                args['cfgdspwd'],
-                                "/slapd/Tasks/Operation/Create",
-                                verbose, secure, cgiargs)
-        elif not args['new_style']:
-            prog = sroot + "/bin/slapd/admin/bin/ds_create"
-            if not os.access(prog, os.X_OK):
-                prog = sroot + "/bin/slapd/admin/bin/ds_newinstance"
-            DirSrvTools.cgiFake(sroot, verbose, prog, cgiargs)
-        else:
-            prog = ''
-            if args['have_admin']:
-                prog = os.path.join(_ds_paths.sbin_dir, PATH_SETUP_DS_ADMIN)
-            else:
-                prog = os.path.join(_ds_paths.sbin_dir, PATH_SETUP_DS)
-
-            if not os.path.isfile(prog):
-                log.error("Can't find file: %r, removing extension" % prog)
-                prog = prog[:-3]
-
-            content = formatInfData(args)
-            DirSrvTools.runInfProg(prog, content, verbose)
-
-        newconn = lib389.DirSrv(args['newhost'], args['newport'],
-                                args['newrootdn'], args['newrootpw'],
-                                args['newinstance'])
-        newconn.prefix = prefix
-        newconn.backupdir = backupdir
-        newconn.isLocal = isLocal
-        # Now the admin should have been created
-        # but still I should have taken all the required infos
-        # before.
-        if args['have_admin'] and not args['setup_admin']:
-            newconn.asport = asport
-            newconn.cfgdsuser = args['cfgdsuser']
-            newconn.cfgdspwd = args['cfgdspwd']
-        return newconn
-
-    @staticmethod
-    def createAndSetupReplica(createArgs, repArgs):
-        """
-        Pass this sub two dicts - the first one is a dict suitable to create
-        a new instance - see createInstance for more details
-        the second is a dict suitable for replicaSetupAll -
-        see replicaSetupAll
-        """
-        conn = DirSrvTools.createInstance(createArgs)
-        if not conn:
-            print("Error: could not create server", createArgs)
-            return 0
-
-        conn.replicaSetupAll(repArgs)
-        return conn
-
     @staticmethod
     def makeGroup(group=DEFAULT_USER):
         """
@@ -903,53 +646,55 @@ class DirSrvTools(object):
 
             /prefix/lib[64]/dirsrv/slapd-INSTANCE/
         '''
+        if ds_is_older('1.4.0'):
+            libdir = os.path.join(_ds_paths.lib_dir, 'dirsrv')
 
-        libdir = os.path.join(_ds_paths.lib_dir, 'dirsrv')
+            # Gather all the instances so we can adjust the permissions, otherwise
+            servers = []
+            path = os.path.join(_ds_paths.sysconf_dir, 'dirsrv')
+            for files in os.listdir(path):
+                if files.startswith('slapd-') and not files.endswith('.removed'):
+                    servers.append(os.path.join(libdir, files))
 
-        # Gather all the instances so we can adjust the permissions, otherwise
-        servers = []
-        path = os.path.join(_ds_paths.sysconf_dir, 'dirsrv')
-        for files in os.listdir(path):
-            if files.startswith('slapd-') and not files.endswith('.removed'):
-                servers.append(os.path.join(libdir, files))
+            if len(servers) == 0:
+                # This should not happen
+                log.fatal('runUpgrade: no servers found!')
+                assert False
 
-        if len(servers) == 0:
-            # This should not happen
-            log.fatal('runUpgrade: no servers found!')
-            assert False
+            '''
+            The setup script calls things like /lib/dirsrv/slapd-instance/db2bak,
+            etc, and when we run the setup perl script it gets permission denied
+            as the default permissions are 750.  Adjust the permissions to 755.
+            '''
+            for instance in servers:
+                for files in os.listdir(instance):
+                    os.chmod(os.path.join(instance, files), 755)
 
-        '''
-        The setup script calls things like /lib/dirsrv/slapd-instance/db2bak,
-        etc, and when we run the setup perl script it gets permission denied
-        as the default permissions are 750.  Adjust the permissions to 755.
-        '''
-        for instance in servers:
-            for files in os.listdir(instance):
-                os.chmod(os.path.join(instance, files), 755)
-
-        # Run the "upgrade"
-        try:
-            prog = os.path.join(_ds_paths.sbin_dir, PATH_SETUP_DS)
-            process = subprocess.Popen([prog, '--update'], shell=False,
+            # Run the "upgrade"
+            try:
+                prog = os.path.join(_ds_paths.sbin_dir, PATH_SETUP_DS)
+                process = subprocess.Popen([prog, '--update'], shell=False,
                                        stdin=subprocess.PIPE)
-            # Answer the interactive questions, as "--update" currently does
-            # not work with INF files
-            process.stdin.write('yes\n')
-            if(online):
-                process.stdin.write('online\n')
-                for x in servers:
-                    process.stdin.write(DN_DM + '\n')
-                    process.stdin.write(PW_DM + '\n')
-            else:
-                process.stdin.write('offline\n')
-            process.stdin.close()
-            process.wait()
-            if process.returncode != 0:
-                log.fatal('runUpgrade failed!  Error: %s ' % process.returncode)
-                assert(False)
-        except:
-            log.fatal('runUpgrade failed!')
-            raise
+                # Answer the interactive questions, as "--update" currently does
+                # not work with INF files
+                process.stdin.write(b'yes\n')
+                if(online):
+                    process.stdin.write(b'online\n')
+                    for x in servers:
+                        process.stdin.write(ensure_bytes(DN_DM + '\n'))
+                        process.stdin.write(ensure_bytes(PW_DM + '\n'))
+                else:
+                    process.stdin.write(b'offline\n')
+                process.stdin.close()
+                process.wait()
+                if process.returncode != 0:
+                    log.fatal('runUpgrade failed!  Error: %s ' % process.returncode)
+                    assert(False)
+            except:
+                log.fatal('runUpgrade failed!')
+                raise
+        else:
+            pass
 
     @staticmethod
     def searchFile(filename, pattern):
