@@ -1,5 +1,5 @@
 # --- BEGIN COPYRIGHT BLOCK ---
-# Copyright (C) 2015 Red Hat, Inc.
+# Copyright (C) 2022 Red Hat, Inc.
 # Copyright (C) 2019 William Brown <william@blackhats.net.au>
 # All rights reserved.
 #
@@ -24,6 +24,7 @@ except ImportError:
         p.stdout, p.stdin = popen2(cmd_l)
         return p
 
+import json
 import re
 import os
 import logging
@@ -31,22 +32,32 @@ import shutil
 import ldap
 import socket
 import time
+import stat
+from datetime import (datetime, timedelta)
 import sys
 import filecmp
 import pwd
-import six
 import shlex
+import operator
 import subprocess
+import math
+# Setuptools ships with 'packaging' module, let's use it from there
+try:
+    from pkg_resources.extern.packaging.version import LegacyVersion
+# Fallback to a normal 'packaging' module in case 'setuptools' is stripped
+except:
+    from packaging.version import LegacyVersion
 from socket import getfqdn
 from ldapurl import LDAPUrl
 from contextlib import closing
 
 import lib389
-from lib389.paths import Paths
+from pathlib import Path
+from lib389.paths import ( Paths, DEFAULTS_PATH )
 from lib389.dseldif import DSEldif
 from lib389._constants import (
         DEFAULT_USER, VALGRIND_WRAPPER, DN_CONFIG, CFGSUFFIX, LOCALHOST,
-        ReplicaRole, CONSUMER_REPLICAID
+        ReplicaRole, CONSUMER_REPLICAID, SENSITIVE_ATTRS, DEFAULT_DB_LIB
     )
 from lib389.properties import (
         SER_HOST, SER_USER_ID, SER_GROUP_ID, SER_STRICT_HOSTNAME_CHECKING, SER_PORT,
@@ -56,8 +67,12 @@ from lib389.properties import (
 
 MAJOR, MINOR, _, _, _ = sys.version_info
 
+DEBUGGING = os.getenv('DEBUGGING', default=False)
+
 log = logging.getLogger(__name__)
 
+selinux_fcontext_info = None
+
 #
 # Various searches to be used in getEntry
 #   eg getEntry(*searches['NAMINGCONTEXTS'])
@@ -71,101 +86,101 @@ searches = {
 
 # Map table for pseudolocalized strings
 _chars = {
-    " ": six.u("\u2003"),
-    "!": six.u("\u00a1"),
-    "\"": six.u("\u2033"),
-    "#": six.u("\u266f"),
-    "$": six.u("\u20ac"),
-    "%": six.u("\u2030"),
-    "&": six.u("\u214b"),
-    "'": six.u("\u00b4"),
-    ")": six.u("}"),
-    "(": six.u("{"),
-    "*": six.u("\u204e"),
-    "+": six.u("\u207a"),
-    ",": six.u("\u060c"),
-    "-": six.u("\u2010"),
-    ".": six.u("\u00b7"),
-    "/": six.u("\u2044"),
-    "0": six.u("\u24ea"),
-    "1": six.u("\u2460"),
-    "2": six.u("\u2461"),
-    "3": six.u("\u2462"),
-    "4": six.u("\u2463"),
-    "5": six.u("\u2464"),
-    "6": six.u("\u2465"),
-    "7": six.u("\u2466"),
-    "8": six.u("\u2467"),
-    "9": six.u("\u2468"),
-    ":": six.u("\u2236"),
-    ";": six.u("\u204f"),
-    "<": six.u("\u2264"),
-    "=": six.u("\u2242"),
-    ">": six.u("\u2265"),
-    "?": six.u("\u00bf"),
-    "@": six.u("\u055e"),
-    "A": six.u("\u00c5"),
-    "B": six.u("\u0181"),
-    "C": six.u("\u00c7"),
-    "D": six.u("\u00d0"),
-    "E": six.u("\u00c9"),
-    "F": six.u("\u0191"),
-    "G": six.u("\u011c"),
-    "H": six.u("\u0124"),
-    "I": six.u("\u00ce"),
-    "J": six.u("\u0134"),
-    "K": six.u("\u0136"),
-    "L": six.u("\u013b"),
-    "M": six.u("\u1e40"),
-    "N": six.u("\u00d1"),
-    "O": six.u("\u00d6"),
-    "P": six.u("\u00de"),
-    "Q": six.u("\u01ea"),
-    "R": six.u("\u0154"),
-    "S": six.u("\u0160"),
-    "T": six.u("\u0162"),
-    "U": six.u("\u00db"),
-    "V": six.u("\u1e7c"),
-    "W": six.u("\u0174"),
-    "X": six.u("\u1e8a"),
-    "Y": six.u("\u00dd"),
-    "Z": six.u("\u017d"),
-    "[": six.u("\u2045"),
-    "\\": six.u("\u2216"),
-    "]": six.u("\u2046"),
-    "^": six.u("\u02c4"),
-    "_": six.u("\u203f"),
-    "`": six.u("\u2035"),
-    "a": six.u("\u00e5"),
-    "b": six.u("\u0180"),
-    "c": six.u("\u00e7"),
-    "d": six.u("\u00f0"),
-    "e": six.u("\u00e9"),
-    "f": six.u("\u0192"),
-    "g": six.u("\u011d"),
-    "h": six.u("\u0125"),
-    "i": six.u("\u00ee"),
-    "j": six.u("\u0135"),
-    "k": six.u("\u0137"),
-    "l": six.u("\u013c"),
-    "m": six.u("\u0271"),
-    "n": six.u("\u00f1"),
-    "o": six.u("\u00f6"),
-    "p": six.u("\u00fe"),
-    "q": six.u("\u01eb"),
-    "r": six.u("\u0155"),
-    "s": six.u("\u0161"),
-    "t": six.u("\u0163"),
-    "u": six.u("\u00fb"),
-    "v": six.u("\u1e7d"),
-    "w": six.u("\u0175"),
-    "x": six.u("\u1e8b"),
-    "y": six.u("\u00fd"),
-    "z": six.u("\u017e"),
-    "{": six.u("("),
-    "}": six.u(")"),
-    "|": six.u("\u00a6"),
-    "~": six.u("\u02de"),
+    " ": u"\u2003",
+    "!": u"\u00a1",
+    "\"": u"\u2033",
+    "#": u"\u266f",
+    "$": u"\u20ac",
+    "%": u"\u2030",
+    "&": u"\u214b",
+    "'": u"\u00b4",
+    ")": u"}",
+    "(": u"{",
+    "*": u"\u204e",
+    "+": u"\u207a",
+    ",": u"\u060c",
+    "-": u"\u2010",
+    ".": u"\u00b7",
+    "/": u"\u2044",
+    "0": u"\u24ea",
+    "1": u"\u2460",
+    "2": u"\u2461",
+    "3": u"\u2462",
+    "4": u"\u2463",
+    "5": u"\u2464",
+    "6": u"\u2465",
+    "7": u"\u2466",
+    "8": u"\u2467",
+    "9": u"\u2468",
+    ":": u"\u2236",
+    ";": u"\u204f",
+    "<": u"\u2264",
+    "=": u"\u2242",
+    ">": u"\u2265",
+    "?": u"\u00bf",
+    "@": u"\u055e",
+    "A": u"\u00c5",
+    "B": u"\u0181",
+    "C": u"\u00c7",
+    "D": u"\u00d0",
+    "E": u"\u00c9",
+    "F": u"\u0191",
+    "G": u"\u011c",
+    "H": u"\u0124",
+    "I": u"\u00ce",
+    "J": u"\u0134",
+    "K": u"\u0136",
+    "L": u"\u013b",
+    "M": u"\u1e40",
+    "N": u"\u00d1",
+    "O": u"\u00d6",
+    "P": u"\u00de",
+    "Q": u"\u01ea",
+    "R": u"\u0154",
+    "S": u"\u0160",
+    "T": u"\u0162",
+    "U": u"\u00db",
+    "V": u"\u1e7c",
+    "W": u"\u0174",
+    "X": u"\u1e8a",
+    "Y": u"\u00dd",
+    "Z": u"\u017d",
+    "[": u"\u2045",
+    "\\": u"\u2216",
+    "]": u"\u2046",
+    "^": u"\u02c4",
+    "_": u"\u203f",
+    "`": u"\u2035",
+    "a": u"\u00e5",
+    "b": u"\u0180",
+    "c": u"\u00e7",
+    "d": u"\u00f0",
+    "e": u"\u00e9",
+    "f": u"\u0192",
+    "g": u"\u011d",
+    "h": u"\u0125",
+    "i": u"\u00ee",
+    "j": u"\u0135",
+    "k": u"\u0137",
+    "l": u"\u013c",
+    "m": u"\u0271",
+    "n": u"\u00f1",
+    "o": u"\u00f6",
+    "p": u"\u00fe",
+    "q": u"\u01eb",
+    "r": u"\u0155",
+    "s": u"\u0161",
+    "t": u"\u0163",
+    "u": u"\u00fb",
+    "v": u"\u1e7d",
+    "w": u"\u0175",
+    "x": u"\u1e8b",
+    "y": u"\u00fd",
+    "z": u"\u017e",
+    "{": u"(",
+    "}": u")",
+    "|": u"\u00a6",
+    "~": u"\u02de",
 }
 
 #
@@ -173,6 +188,36 @@ _chars = {
 #
 
 
+def selinux_present():
+    """
+    Determine if selinux libraries are on a system, and if so, if we are in
+    a state to consume them (enabled, disabled).
+
+    :returns: bool
+    """
+    status = False
+
+    try:
+        import selinux
+        if selinux.is_selinux_enabled():
+            # We have selinux, lets see if we are allowed to configure it.
+            # (just checking the uid for now - we may rather check if semanage command success)
+            if os.geteuid() != 0:
+                log.info('Non privileged user cannot use semanage, will not relabel ports or files.' )
+            elif not os.path.exists('/usr/sbin/semanage'):
+                log.info('semanage command is not installed, will not relabel ports or files. Please install policycoreutils-python-utils.' )
+            else:
+                status = True
+        else:
+            # We have the module, but it's disabled.
+            log.info('selinux is disabled, will not relabel ports or files.' )
+    except ImportError:
+        # No python module, so who knows what state we are in.
+        log.error('selinux python module not found, will not relabel files.' )
+
+    return status
+
+
 def selinux_restorecon(path):
     """
     Relabel a filesystem rooted at path.
@@ -184,11 +229,11 @@ def selinux_restorecon(path):
     try:
         import selinux
     except ImportError:
-        log.error('selinux python module not found, skipping relabel path %s' % path)
+        log.debug('selinux python module not found, skipping relabel path %s' % path)
         return
 
     if not selinux.is_selinux_enabled():
-        log.error('selinux is disabled, skipping relabel path %s' % path)
+        log.debug('selinux is disabled, skipping relabel path %s' % path)
         return
 
     try:
@@ -197,69 +242,203 @@ def selinux_restorecon(path):
         log.debug("Failed to run restorecon on: " + path)
 
 
+def _parse_semanage_fcontexts(cmd, regex=r"^(/[^ ]*)[^:=]+:[^:]*:([^:]*):.*$", reject=None):
+    """Parse semanage fcontext -L output
+    """
+    if reject is None:
+        reject = {}
+    info = {}
+    result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    args = ' '.join(ensure_list_str(result.args))
+    stdout = ensure_str(result.stdout)
+    stderr = ensure_str(result.stderr)
+    if result.returncode:
+        log.debug("CMD: {args} returned {result.returncode} STDOUT: {stdout} STDERR: {stderr}")
+        return info
+    for m in re.finditer(regex, stdout, flags=re.MULTILINE):
+        if not m.group(1) in reject:
+            info[m.group(1)] = m.group(2)
+    return info
+
+
+def _get_selinux_fcontext_info():
+    global selinux_fcontext_info
+    if not selinux_fcontext_info:
+        local = _parse_semanage_fcontexts(["semanage", "fcontext", "-l", "-C"])
+        selinux_fcontext_info = {
+            "local"  : local,
+            "policy" : _parse_semanage_fcontexts(["semanage", "fcontext", "-l"], reject=local),
+            "equiv"  : _parse_semanage_fcontexts(["semanage", "fcontext", "-l"], regex=r"^(/[^ ]*) += +(.*)$"),
+        }
+    return selinux_fcontext_info
+
+
+def resolve_selinux_path(path):
+    '''Return the path as expected by semanage fcontext'''
+    path = str(Path(path).resolve())
+    if selinux_present():
+        _get_selinux_fcontext_info()
+        equiv = selinux_fcontext_info['equiv']
+        for r in equiv:
+            if path.startswith(r):
+                path = path.replace(r, equiv[r])
+    return path
+
+
+def selinux_label_file(path, label):
+    """
+    Set set an SELinux label to a file (or a directory)
+
+    :param path: The file path
+    :type path: str
+    :param label: The label to set (None to unset a label)
+    :type path: str
+    :raises: ValueError: Error message
+    """
+    if not selinux_present():
+        return
+    _get_selinux_fcontext_info()
+    if os.path.exists(path):
+        path = resolve_selinux_path(path)
+    local = selinux_fcontext_info['local']
+    policy = selinux_fcontext_info['policy']
+    if path in local:
+        if local[path] == label:
+            return
+        log.debug(f"Removing SELinux file context {path} with label {local[path]}.")
+        subprocess.run(["semanage", "fcontext", "-d", path])
+        del local[path]
+    if path in policy:
+        if policy[path] == label:
+            return
+        raise ValueError(f'Cannot change file context for {path} because it is defined in SELinux policy. Please choose another path.')
+    if label:
+        try:
+            log.debug(f"Setting label {label} in SELinux file context {path}.")
+            result = subprocess.run(["semanage", "fcontext", "-a", "-t", label, path],
+                                     stdout=subprocess.PIPE,
+                                     stderr=subprocess.PIPE)
+            args = ' '.join(ensure_list_str(result.args))
+            stdout = ensure_str(result.stdout)
+            stderr = ensure_str(result.stderr)
+            if result.returncode != 0:
+                log.error(f"ERROR CMD: {args} ; STDOUT: {stdout} ; STDERR: {stderr}")
+                result.check_returncode()
+            local[path] = label
+        except (OSError, subprocess.CalledProcessError) as e:
+            raise ValueError(f"Failed to set SElinux label {label} on {path}: {str(e)}")
+    if os.path.exists(path):
+        # pytest fails if I use selinux_restorecon(path)
+        subprocess.run(["restorecon", "-R", path])
+
+
+def selinux_clean_files_label(all=False):
+    """Remove dirsrv_*_t labels from either removed files or all files."""
+    if not selinux_present():
+        return
+    _get_selinux_fcontext_info()
+    local = { **selinux_fcontext_info['local'] }
+    for path, label in local.items():
+        if label in ( 'dirsrv_config_t', 'dirsrv_tmpfs_t', 'dirsrv_var_lib_t', 'dirsrv_var_lock_t', 'dirsrv_var_log_t', 'dirsrv_var_run_t', ):
+            if all or not os.path.exists(path):
+                selinux_label_file(path, None)
+
+
+def selinux_clean_ports_label():
+    """Remove labels from all port having ldap_port_t labels."""
+    if not selinux_present():
+        return
+    cmd=[ 'semanage', 'port', '--list', '-C' ]
+    res = _parse_semanage_fcontexts(cmd, regex=r"^(ldap_port_t) *tcp *(.*)$")
+    if 'ldap_port_t' in res:
+        for port in res['ldap_port_t'].split(","):
+            # Cannot use selinux_label_port here because port may be a range.
+            subprocess.run(["semanage", "port", "-d", "-p", "tcp", port.strip()])
+
+
+def _get_selinux_port_policies(port):
+    """Get a list of selinux port policies for the specified port, 'tcp' protocol and
+    excluding 'unreserved_port_t', 'reserved_port_t', 'ephemeral_port_t' labels"""
+
+    # [2:] - cut first lines containing the headers. [:-1] - empty line
+    policy_lines = subprocess.check_output(["semanage", "port", "-l"], encoding='utf-8').split("\n")[2:][:-1]
+    policies = []
+    for line in policy_lines:
+        data = line.split()
+        ports_list = []
+        for p in data[2:]:
+            if "," in p:
+                p = p[:-1]
+            if "-" in p:
+                p = range(int(p.split("-")[0]), int(p.split("-")[1]))
+            else:
+                p = [int(p)]
+            ports_list.extend(p)
+        if data[1] == 'tcp' and port in ports_list and \
+           data[0] not in ['hi_reserved_port_t', 'unreserved_port_t', 'reserved_port_t', 'ephemeral_port_t']:
+            policies.append({'protocol': data[1], 'type': data[0], 'ports': ports_list})
+    return policies
+
+
 def selinux_label_port(port, remove_label=False):
     """
     Either set or remove an SELinux label(ldap_port_t) for a TCP port
 
-    :param port: The TCP port to be labelled
+    :param port: The TCP port to be labeled
     :type port: str
     :param remove_label: Set True if the port label should be removed
     :type remove_label: boolean
     :raises: ValueError: Error message
     """
+    if port is None:
+        return
     try:
         import selinux
     except ImportError:
-        log.error('selinux python module not found, skipping port labeling.')
+        log.debug('selinux python module not found, skipping port labeling.')
         return
 
-    try:
-        import sepolicy
-    except ImportError:
-        log.error('sepolicy python module not found, skipping port labeling.')
-        return
-
-    if not selinux.is_selinux_enabled():
-        log.error('selinux is disabled, skipping port relabel')
+    if not selinux_present():
+        log.debug('selinux is disabled, skipping port relabel')
         return
 
     # We only label ports that ARE NOT in the default policy that comes with
     # a RH based system.
+    port = int(port)
     selinux_default_ports = [389, 636, 3268, 3269, 7389]
     if port in selinux_default_ports:
-        log.debug('port %s already in %s, skipping port relabel' % (port, selinux_default_ports))
+        log.debug('port {} already in {}, skipping port relabel'.format(port, selinux_default_ports))
         return
-
     label_set = False
     label_ex = None
 
-    policies = [p for p in sepolicy.info(sepolicy.PORT)
-                if p['protocol'] == 'tcp'
-                if port in range(p['low'], p['high'] + 1)
-                if p['type'] not in ['unreserved_port_t', 'reserved_port_t', 'ephemeral_port_t']]
+    policies = _get_selinux_port_policies(port)
 
     for policy in policies:
         if "ldap_port_t" == policy['type']:
             label_set = True  # Port already has our label
-            if policy['low'] != policy['high']:
-                # We have a range
-                if port in range(policy['low'], policy['high'] + 1):
-                    # The port is within the range, just return
-                    return
+            if port in policy['ports']:
+                # The port is within the range, just return
+                return
             break
         elif not remove_label:
             # Port belongs to someone else (bad)
             # This is only an issue during setting a label, not removing a label
-            raise ValueError("Port {} was already labelled with: ({})  Please choose a different port number".format(port, policy['type']))
+            raise ValueError("Port {} was already labeled with: ({})  Please choose a different port number".format(port, policy['type']))
 
     if (remove_label and label_set) or (not remove_label and not label_set):
-        for i in range(3):
-
+        for i in range(5):
             try:
-                subprocess.check_call(["semanage", "port",
-                                       "-d" if remove_label else "-a",
-                                       "-t", "ldap_port_t",
-                                       "-p", "tcp", str(port)])
+                result = subprocess.run(["semanage", "port",
+                                         "-d" if remove_label else "-a",
+                                         "-t", "ldap_port_t",
+                                         "-p", "tcp", str(port)],
+                                         stdout=subprocess.PIPE,
+                                         stderr=subprocess.PIPE)
+                args = ' '.join(ensure_list_str(result.args))
+                stdout = ensure_str(result.stdout)
+                stderr = ensure_str(result.stderr)
+                log.debug(f"CMD: {args} ; STDOUT: {stdout} ; STDERR: {stderr}")
                 return
             except (OSError, subprocess.CalledProcessError) as e:
                 label_ex = e
@@ -436,8 +615,10 @@ def valgrind_enable(sbin_dir, wrapper=None):
     :raise EnvironmentError: If script is not run as 'root'
     '''
 
-    if os.geteuid() != 0:
-        log.error('This script must be run as root to use valgrind')
+    if not os.access(sbin_dir, os.W_OK):
+        # Note: valgrind has no limitation but  ns-slapd must be replaced
+        # This check allows non root user to use custom install prefix
+        log.error('This script must be run as root to use valgrind (Should at least be able to write in {sbin_dir})')
         raise EnvironmentError
 
     if not wrapper:
@@ -483,7 +664,20 @@ def valgrind_enable(sbin_dir, wrapper=None):
                       e.strerror)
 
     # Disable selinux
-    os.system('setenforce 0')
+    if os.geteuid() == 0:
+        os.system('setenforce 0')
+
+    # Disable systemd by turning off with_system in .inf file
+    old_path = Paths()._get_defaults_loc(DEFAULTS_PATH)
+    new_path = f'{old_path}.orig'
+    os.rename(old_path, new_path)
+    with open(new_path, 'rt') as fin:
+       with open(old_path, 'wt') as fout:
+            for line in fin:
+                if line.startswith('with_systemd'):
+                    fout.write('with_systemd = 0\n')
+                else:
+                    fout.write(line)
 
     log.info('Valgrind is now enabled.')
 
@@ -500,8 +694,10 @@ def valgrind_disable(sbin_dir):
     :raise EnvironmentError: If script is not run as 'root'
     '''
 
-    if os.geteuid() != 0:
-        log.error('This script must be run as root to use valgrind')
+    if not os.access(sbin_dir, os.W_OK):
+        # Note: valgrind has no limitation but  ns-slapd must be replaced
+        # This check allows non root user to use custom install prefix
+        log.error('This script must be run as root to use valgrind (Should at least be able to write in {sbin_dir})')
         raise EnvironmentError
 
     nsslapd_orig = '%s/ns-slapd' % sbin_dir
@@ -525,7 +721,14 @@ def valgrind_disable(sbin_dir):
                          e.strerror)
 
     # Enable selinux
-    os.system('setenforce 1')
+    if os.geteuid() == 0:
+        os.system('setenforce 1')
+
+    # Restore .inf file (for systemd)
+    new_path = Paths()._get_defaults_loc(DEFAULTS_PATH)
+    old_path = f'{new_path}.orig'
+    if os.path.exists(old_path):
+        os.replace(old_path, new_path)
 
     log.info('Valgrind is now disabled.')
 
@@ -551,7 +754,7 @@ def valgrind_get_results_file(dirsrv_inst):
 
     # Run the command and grab the output
     p = os.popen(cmd)
-    results_file = p.readline()
+    results_file = p.readline().strip()
     p.close()
 
     return results_file
@@ -609,7 +812,8 @@ def isLocalHost(host_name):
         Uses gethostbyname()
     """
     # first see if this is a "well known" local hostname
-    if host_name == 'localhost' or \
+    if host_name is None or \
+       host_name == 'localhost' or \
        host_name == 'localhost.localdomain' or \
        host_name == socket.gethostname():
         return True
@@ -926,7 +1130,7 @@ def formatInfData(args):
     content = ("[General]" "\n")
     content += ("FullMachineName= %s\n" % args[SER_HOST])
     content += ("SuiteSpotUserID= %s\n" % args[SER_USER_ID])
-    content += ("nSuiteSpotGroup= %s\n" % args[SER_GROUP_ID])
+    content += ("SuiteSpotGroup= %s\n" % args[SER_GROUP_ID])
     content += ("StrictHostCheck= %s\n" % args[SER_STRICT_HOSTNAME_CHECKING])
 
     if args.get('have_admin'):
@@ -969,25 +1173,25 @@ def formatInfData(args):
 
 def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
     """Generate a host, port, secure port, server ID and replica ID
-    for the selected role and instance number. I.e. inst_num=1, role="master".
+    for the selected role and instance number. I.e. inst_num=1, role="supplier".
 
     @param inst_num - an instance number in a range from 1 to 99
-    @param role - ReplicaRole.STANDALONE, ReplicaRole.MASTER, ReplicaRole.HUB, ReplicaRole.CONSUMER
+    @param role - ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER, ReplicaRole.HUB, ReplicaRole.CONSUMER
     @return - the dict with next keys: host, port, secure port, server id and replica id
     """
 
     if inst_num not in range(1, 100):
         raise ValueError("Instance number should be in a range from 1 to 99")
 
-    if role not in (ReplicaRole.STANDALONE, ReplicaRole.MASTER, ReplicaRole.HUB, ReplicaRole.CONSUMER):
-        raise ValueError('Role should be {}, {}, {}, {}'.format(ReplicaRole.STANDALONE, ReplicaRole.MASTER,
+    if role not in (ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER, ReplicaRole.HUB, ReplicaRole.CONSUMER):
+        raise ValueError('Role should be {}, {}, {}, {}'.format(ReplicaRole.STANDALONE, ReplicaRole.SUPPLIER,
                                                                 ReplicaRole.HUB, ReplicaRole.CONSUMER))
 
     instance_data = {}
     relevant_num = 38900
 
     # Set relevant number for ports
-    if role == ReplicaRole.MASTER:
+    if role == ReplicaRole.SUPPLIER:
         relevant_num += 100
     elif role == ReplicaRole.HUB:
         relevant_num += 200
@@ -995,7 +1199,7 @@ def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
         relevant_num += 300
 
     # Define replica ID
-    if role == ReplicaRole.MASTER:
+    if role == ReplicaRole.SUPPLIER:
         replica_id = inst_num
     else:
         replica_id = CONSUMER_REPLICAID
@@ -1011,7 +1215,7 @@ def generate_ds_params(inst_num, role=ReplicaRole.STANDALONE):
     return instance_data
 
 
-def get_ds_version():
+def get_ds_version(paths=None):
     """
     Return version of ns-slapd installed on this system. This is determined by the defaults.inf
     file, so is correct for the built and installed ns-slapd binary. This function works without
@@ -1019,18 +1223,75 @@ def get_ds_version():
 
     returns a string of the form: 1.3.4.8
     """
-    p = Paths()
-    return p.version
+    if paths is None:
+        paths = Paths()
+    return paths.version
 
 
-def ds_is_older(ver):
-    """Return True if current version of ns-slapd is older than provided
-    version"""
-    return get_ds_version() < ver
+def ds_is_related(relation, *ver, instance=None):
+    """
+    Return a result of a comparison between the current version of ns-slapd and a provided version.
+    """
+    ops = {'older': operator.lt,
+           'newer': operator.ge}
+    if instance is None:
+        ds_ver = get_ds_version()
+    else:
+        ds_ver = get_ds_version(instance.ds_paths)
+    if len(ver) > 1:
+        for cmp_ver in ver:
+            if cmp_ver.startswith(ds_ver[:3]):
+                return ops[relation](LegacyVersion(ds_ver),LegacyVersion(cmp_ver))
+    else:
+        return ops[relation](LegacyVersion(ds_ver), LegacyVersion(ver[0]))
 
 
-def ds_is_newer(ver):
-    return get_ds_version() >= ver
+def ds_is_older(*ver, instance=None):
+    """
+    Return True if the current version of ns-slapd is older than a provided version
+    """
+    return ds_is_related('older', *ver, instance=instance)
+
+
+def ds_is_newer(*ver, instance=None):
+    """
+    Return True if the current version of ns-slapd is newer than a provided version
+    """
+    return ds_is_related('newer', *ver, instance=instance)
+
+def ds_supports_new_changelog():
+    """
+    Return True if the current version of ns-slapd supports changelogs under cn=changelog,cn=<backend>,cn=ldbm..
+    """
+    return ds_is_newer('1.4.4.3')
+
+def gentime_to_datetime(gentime):
+    """Convert Generalized time to datetime object
+
+    :param gentime: Time in the format - YYYYMMDDHHMMSSZ (20170126120000Z)
+    :type password: str
+    :returns: datetime.datetime object
+    """
+
+    return datetime.strptime(gentime, '%Y%m%d%H%M%SZ')
+
+
+def gentime_to_posix_time(gentime):
+    """Convert Generalized time to POSIX time format
+
+    :param gentime: Time in the format - YYYYMMDDHHMMSSZ (20170126120000Z)
+    :type password: str
+    :returns: Epoch time int
+    """
+    time_tuple = (int(gentime[:4]),
+                  int(gentime[4:6]),
+                  int(gentime[6:8]),
+                  int(gentime[8:10]),
+                  int(gentime[10:12]),
+                  int(gentime[12:14]),
+                  0, 0, 0)
+
+    return time.mktime(time_tuple)
 
 
 def getDateTime():
@@ -1044,11 +1305,28 @@ def getDateTime():
 
 
 def socket_check_open(host, port):
-    with closing(socket.socket(socket.AF_INET6, socket.SOCK_STREAM)) as sock:
-        if sock.connect_ex((host, port)) == 0:
-            return True
-        else:
-            return False
+    """
+    Check if a socket can be opened.  Need to handle cases where IPv6 is completely
+    disabled.
+    """
+    try:
+        # Trying IPv6 first ...
+        family = socket.AF_INET6
+        with closing(socket.socket(family, socket.SOCK_STREAM)) as sock:
+            if sock.connect_ex((host, port)) == 0:
+                return True
+            else:
+                return False
+    except OSError:
+        # No IPv6, adjust hostname, and try IPv4 ...
+        family = socket.AF_INET
+        if host == "::1":
+            host = "127.0.0.1"
+        with closing(socket.socket(family, socket.SOCK_STREAM)) as sock:
+            if sock.connect_ex((host, port)) == 0:
+                return True
+            else:
+                return False
 
 
 def ensure_bytes(val):
@@ -1099,7 +1377,7 @@ def ensure_dict_str(val):
 
 
 def pseudolocalize(string):
-    pseudo_string = six.u("")
+    pseudo_string = u""
     for char in string:
         try:
             pseudo_string += _chars[char]
@@ -1157,9 +1435,10 @@ def get_ldapurl_from_serverid(instance):
         return ("ldap://{}:{}".format(host, port), None)
 
 
-def get_instance_list(prefix=None):
+def get_instance_list():
     # List all server instances
-    conf_dir = (prefix or "") + "/etc/dirsrv/"
+    paths = Paths()
+    conf_dir = os.path.join(paths.sysconf_dir, 'dirsrv')
     insts = []
     try:
         for inst in os.listdir(conf_dir):
@@ -1167,9 +1446,22 @@ def get_instance_list(prefix=None):
                 insts.append(inst)
     except OSError as e:
         log.error("Failed to check directory: {} - {}".format(conf_dir, str(e)))
+
     insts.sort()
     return insts
 
+
+# used by arg parse completers
+def instance_choices(prefix, parsed_args, **kwargs):
+    insts = get_instance_list()
+    inst_names = []
+    for inst in insts:
+        inst_names.append(inst)
+        inst_names.append(inst.replace("slapd-", ""))
+
+    return inst_names
+
+
 def get_user_is_ds_owner():
     # Check if we have permission to administer the DS instance. This is required
     # for some tasks such as installing, killing, or editing configs for the
@@ -1186,3 +1478,265 @@ def get_user_is_ds_owner():
     return False
 
 
+def get_user_is_root():
+    cur_uid = os.getuid()
+    if cur_uid == 0:
+        # We are root, we have permission
+        return True
+    return False
+
+
+def basedn_to_ldap_dns_uri(basedn):
+    # ldap:///dc%3Dexample%2Cdc%3Dcom
+    return "ldaps:///" + basedn.replace("=", "%3D").replace(",", "%2C")
+
+
+def display_log_value(attr, value, hide_sensitive=True):
+    # Mask all the sensitive attribute values
+    if DEBUGGING or not hide_sensitive:
+        return value
+    else:
+        if attr.lower() in SENSITIVE_ATTRS:
+            if type(value) in (list, tuple):
+                return list(map(lambda _: '********', value))
+            else:
+                return '********'
+        else:
+            return value
+
+
+def display_log_data(data, hide_sensitive=True):
+    # Take a dict and mask all the sensitive data
+    return {a: display_log_value(a, v, hide_sensitive) for a, v in data.items()}
+
+
+def convert_bytes(bytes):
+    bytes = int(bytes)
+    if bytes == 0:
+        return "0 B"
+    size_name = ["B", "KB", "MB", "GB", "TB", "PB"]
+    i = int(math.floor(math.log(bytes, 1024)))
+    pow = math.pow(1024, i)
+    siz = round(bytes / pow, 2)
+    return "{} {}".format(siz, size_name[i])
+
+
+def search_filter_escape_bytes(bytes_value):
+    """ Convert a byte sequence to a properly escaped for LDAP (format BACKSLASH HEX HEX) string"""
+    # copied from https://github.com/cannatag/ldap3/blob/master/ldap3/utils/conv.py
+    if str is not bytes:
+        if isinstance(bytes_value, str):
+            bytes_value = bytearray(bytes_value, encoding='utf-8')
+        return ''.join([('\\%02x' % int(b)) for b in bytes_value])
+    else:
+        raise RuntimeError('Running with Python 2 is unsupported')
+
+
+def print_nice_time(seconds):
+    """Convert seconds to a pretty format
+    """
+    seconds = int(seconds)
+    d, s = divmod(seconds, 24*60*60)
+    h, s = divmod(s, 60*60)
+    m, s = divmod(s, 60)
+    d_plural = ""
+    h_plural = ""
+    m_plural = ""
+    s_plural = ""
+    if d > 1:
+        d_plural = "s"
+    if h != 1:
+        h_plural = "s"
+    if m != 1:
+        m_plural = "s"
+    if s != 1:
+        s_plural = "s"
+    if d > 0:
+        return f'{d:d} day{d_plural}, {h:d} hour{h_plural}, {m:d} minute{m_plural}, {s:d} second{s_plural}'
+    elif h > 0:
+        return f'{h:d} hour{h_plural}, {m:d} minute{m_plural}, {s:d} second{s_plural}'
+    elif m > 0:
+        return f'{m:d} minute{m_plural}, {s:d} second{s_plural}'
+    else:
+        return f'{s:d} second{s_plural}'
+
+
+def copy_with_permissions(source, target):
+    """Copy a file while preserving all permissions"""
+
+    shutil.copy2(source, target)
+    st = os.stat(source)
+    os.chown(target, st[stat.ST_UID], st[stat.ST_GID])
+    os.chmod(target, st[stat.ST_MODE])
+
+
+def cmp(self, x, y):
+    """
+    Replacement for built-in function cmp that was removed in Python 3
+
+    Compare the two objects x and y and return an integer according to
+    the outcome. The return value is negative if x < y, zero if x == y
+    and strictly positive if x > y.
+    """
+    return (x > y) - (x < y)
+
+
+def is_valid_hostname(hostname):
+    if len(hostname) > 255:
+        return False
+    if hostname[-1] == ".":
+        hostname = hostname[:-1] # strip exactly one dot from the right, if present
+    allowed = re.compile("(?!-)[A-Z\d-]{1,63}(?<!-)$", re.IGNORECASE)
+    return all(allowed.match(x) for x in hostname.split("."))
+
+
+def get_default_db_lib():
+    return os.getenv('NSSLAPD_DB_LIB', default=DEFAULT_DB_LIB)
+
+
+def is_fips():
+    if os.path.exists('/proc/sys/crypto/fips_enabled'):
+        with open('/proc/sys/crypto/fips_enabled', 'r') as f:
+            state = f.readline().strip()
+            if state == '1':
+                return True
+            else:
+                return False
+    else:
+        return False
+
+
+def convert_timestamp(timestamp):
+    """Convert createtimestamp to ctime: 20170405184656Z ----> Wed Apr  5 19:46:56 2017
+    :param timestamp - A timestamp from the server
+    :return - the ctime of the timestamp
+    """
+    if timestamp:
+        time_tuple = (int(timestamp[:4]), int(timestamp[4:6]), int(timestamp[6:8]),
+                      int(timestamp[8:10]), int(timestamp[10:12]), int(timestamp[12:14]),
+                      0, 0, 0)
+        secs = time.mktime(time_tuple)
+        return time.ctime(secs)
+    else:
+        return ""
+
+
+def elapsed_time(timestamp1, timestamp2):
+    """Take two timestamps from the server and get the diff
+    """
+    if timestamp1 and timestamp2:
+        time1_tuple = (int(timestamp1[:4]), int(timestamp1[4:6]), int(timestamp1[6:8]),
+                       int(timestamp1[8:10]), int(timestamp1[10:12]), int(timestamp1[12:14]),
+                       0, 0, 0)
+        time2_tuple = (int(timestamp2[:4]), int(timestamp2[4:6]), int(timestamp2[6:8]),
+                       int(timestamp2[8:10]), int(timestamp2[10:12]), int(timestamp2[12:14]),
+                       0, 0, 0)
+        time1_secs = time.mktime(time1_tuple)
+        time2_secs = time.mktime(time2_tuple)
+        elapsed_secs = time2_secs - time1_secs
+        return str(timedelta(seconds=elapsed_secs))
+    else:
+        return ""
+
+
+def get_task_status(inst, log, taskObj, dn=None, show_log=False, watch=False, use_json=False):
+    """Get the status of a Task, if "watching" keep looping over the tasks
+    :param inst: DirSrv Object
+    :type inst: Object
+    :param log: Logging Object
+    :type log: Object
+    :param taskObj: Lib389 Object (task object from plugins.py)
+    :type taskObj: Object
+    :param dn: A task DN to get the status for (if omitted all tasks for checked)
+    :type dn: str
+    :param show_log: Show the task's log attribute (could be lengthy)
+    :type show_log: boolean
+    :param watch: Keep looping over the task until it completes
+    :type watch: boolean
+    :param use_json: Report the status in JSON
+    :type use_json: boolean
+    """
+
+    tasks = taskObj(inst).list()
+    all_finished = False
+    while not all_finished:
+        task_list = []
+        all_finished = True
+        for task in tasks:
+            task_base_dn = task.get_attr_val_utf8_l('basedn')
+            if task_base_dn is None:
+                # Try linked attr, it uses a different attr name
+                task_base_dn = task.get_attr_val_utf8_l('linkdn')
+            task_filter = task.get_attr_val_utf8_l('filter')
+            status = task.get_attr_val_utf8('nsTaskStatus')
+            exitcode = task.get_attr_val_utf8('nsTaskExitcode')
+            task_log =  task.get_attr_val_utf8('nsTaskLog')
+            task_created = task.get_attr_val_utf8('nsTaskCreated')
+            task_ended = task.get_attr_val_utf8('modifyTimestamp')
+            if task_base_dn is None:
+                task_base_dn = ""
+            if task_filter is None:
+                task_filter = ""
+            if status is None:
+                status = ""
+            if exitcode is None:
+                all_finished = False
+                task_ended = ""
+                exitcode = "Not finished ..."
+                elapsed_time_str = ""
+            else:
+                # Task is finished, use start and end times to calc elapsed time
+                elapsed_time_str = elapsed_time(task_created, task_ended)
+
+            if not show_log:
+                task_log = ""
+
+            if (dn is not None and dn.lower() == task.dn.lower()) or dn is None:
+                    task_list.append({
+                        'dn': task.dn,
+                        'basedn': task_base_dn,
+                        'filter': task_filter,
+                        'log': task_log,
+                        'status': status,
+                        'created': task_created,
+                        'created_str': convert_timestamp(task_created),
+                        'ended': task_ended,
+                        'ended_str': convert_timestamp(task_ended),
+                        'elapsed_time':  elapsed_time_str,
+                        'exitcode': exitcode
+                    })
+
+        # Display the status
+        if use_json:
+            log.info(json.dumps({"type": "list", "items": task_list}, indent=4))
+        else:
+            num_of_tasks = len(task_list)
+            count = 1
+            plural = ""
+            if num_of_tasks != 1:
+                plural = "s"
+            log.info(f'Found {num_of_tasks} fix-up task{plural}\n')
+
+            for task in task_list:
+                log.info(f"[{count}] Task: {task['dn']}")
+                log.info("-" * 80)
+                log.info(f" - Base DN:       {task['basedn']}")
+                if task['filter']:
+                    log.info(f" - Filter:        {task['filter']}")
+                log.info(f" - Status:        {task['status']}")
+                log.info(f" - Started:       {task['created_str']} ({task['created']})")
+                if task['ended_str']:
+                    log.info(f" - Ended:         {task['ended_str']} ({task['ended']})")
+                else:
+                    log.info(" - Ended:")
+                log.info(f" - Elapsed Time:  {task['elapsed_time']}")
+                log.info(f" - Exit Code:     {task['exitcode']}")
+                if show_log:
+                    log.info(f" - Log:\n{task['log']}")
+                log.info("")
+                count += 1
+        if not watch or all_finished:
+            break
+        elif not use_json:
+            log.info("Refreshing status ...")
+        time.sleep(2)
